package com.chinacreator.kafka.handle.impl;

import com.chinacreator.kafka.handle.KakfaBaseHandle;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.thread.AsynTfBhTradeThread;
import com.chinacreator.kafka.thread.NewProdECApiThread;
import com.chinacreator.kafka.util.ThreadLimitUtil;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * AsynTfBhTrade处理类
 *
 * @author zhengrong.yan
 * @date 20210219
 */
@Component("AsynTfBhTradeHandle")
public class AsynTfBhTradeHandle extends KakfaBaseHandle {

    private static Logger log = Logger.getLogger(NewProdInfoHandle.class);

    @Autowired
    private SourceDataRecDao sourceDataRecDao;

    public void doHandle(HashMap conf) {
        boolean hasCancel = false;
        try {
            KafkaConsumer<String, String> consumer = super.init(conf);
            ExecutorService executor = Executors.newFixedThreadPool(3);
            executor.submit(new AsynTfBhTradeThread(conf, consumer, sourceDataRecDao));
            log.info("Threadkey=>" + super.getThreadkey() + "客户端初始化完成");
        } catch (Exception e) {
            e.printStackTrace();
            hasCancel = true;
            log.error(super.getThreadkey() + ":实例消费线程出现异常，" + e.getMessage(), e);
        } finally {
            if (hasCancel) {
                ThreadLimitUtil.removeKey(super.getThreadkey());
            }
        }
    }
}
