package com.chinacreator.kafka.handle.impl;

import com.chinacreator.kafka.handle.KakfaBaseHandle;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.thread.AsynTfBhTradeThread;
import com.chinacreator.kafka.thread.SyngnathusThread;
import com.chinacreator.kafka.util.ThreadLimitUtil;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * SyngnathusHandle处理类
 * 银龄天龙卡数据获取
 * @author xu.zhou
 * @date 20210908
 */
@Component("SyngnathusHandle")
public class SyngnathusHandle extends KakfaBaseHandle {

    private static Logger log = Logger.getLogger(NewProdInfoHandle.class);

    @Autowired
    private SourceDataRecDao sourceDataRecDao;
    
	//银龄天龙卡产品编码，多个用|分隔
	@Value("${config.yltlk.productid}")
	private String yltlcroductid;

    public void doHandle(HashMap conf) {
        boolean hasCancel = false;
        try {
            KafkaConsumer<String, String> consumer = super.init(conf);
            ExecutorService executor = Executors.newFixedThreadPool(3);
            executor.submit(new SyngnathusThread(conf, consumer, sourceDataRecDao, yltlcroductid));
            log.info("Threadkey=>" + super.getThreadkey() + "客户端初始化完成");
        } catch (Exception e) {
            e.printStackTrace();
            hasCancel = true;
            log.error(super.getThreadkey() + ":实例消费线程出现异常，" + e.getMessage(), e);
        } finally {
            if (hasCancel) {
                ThreadLimitUtil.removeKey(super.getThreadkey());
            }
        }
    }
}

