package com.chinacreator.kafka.mapper;

import java.util.HashMap;
import java.util.List;

import org.apache.ibatis.annotations.Param;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.bean.ProvinceBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.sync.bean.BoYuanLogBean;

/**
 * 渠道同步dao
 * @author xu.zhou
 * @date 20210618
 */
public interface ChannelSyncDao {
	
	/**
	 * 查询同步业务所有配置信息
	 * @return
	 */
	List<HashMap> getSyncBusi();
	
	/**
	 * 查询订购关系表要同步的数据
	 * @param sql
	 * @return
	 */
	List<HashMap<String, String>> execQryRelWaitSql(String sql);
	
	/**
	 * 添加待同步数据
	 * @param params
	 * @return
	 */
	int addWaitData(HashMap<String, String> params);
	
	/**
	 * 更新订购同步记录表的数据为同步中
	 * @param recIds
	 * @return
	 */
	int updBatchOrderRecSyncing(List<String> recIds);
	
	/**
	 * 根据ID更新TD_CHSYNC_ORDER_REC记录为处理中
	 * @param recid
	 * @return
	 */
	int updOrderRecSyncing(@Param("recid")String recid);
	
	/**
	 * 根据SPID获取待同步的数据
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getSyncWaitData(String spid, int page);
	
	/**
	 * 获取订购记录表的数据
	 * @return
	 */
	List<HashMap<String, String>> getSyncRecData(@Param("page")int page);
	
	/**
	 * 批量更新数据为同步中
	 * @param recIds
	 * @return
	 */
	int updBatchWaitSyncing(List<String> recIds);
	
	/**
	 * 更新数据为同步中
	 * @param id
	 * @return
	 */
	int updWaitSyncing(@Param("id")String id);
	
	/**
	 * 根据id更新同步结果
	 * @param id
	 * @param resultcode
	 * @param resultinfo
	 * @return
	 */
	int updWaitSyncRes(@Param("id")String id, @Param("resultcode")String resultcode, @Param("resultinfo")String resultinfo);
	
	/**
	 * 根据ID更新订购同步记录表的同步结果
	 * @param id
	 * @param syncsucceed
	 * @return
	 */
	int updOrderRecRes(@Param("id")String id, @Param("syncsucceed")String syncsucceed);
	
	/**
	 * 根据RECID获取所有同步的数据
	 * @param recid
	 * @return
	 */
	List<HashMap<String, String>> getWaitDataByRecid(@Param("recid")String recid);
	
	/**
	 * 根据RECID更新对应同步数据
	 * 用于数据迁移到历史表
	 * @param recid
	 * @return
	 */
	int updWaitUpdrelByRecid(@Param("recid")String recid);
	
	/**
	 * 根据渠道获取接口配置信息
	 * @param channel 接口渠道标识
	 * @return
	 */
	List<HashMap<String, String>> getInvokeConf(@Param("channel")String channel);
	
	
	/**
	 * 根据渠道和订购类型获取接口配置信息
	 * @param channel 接口渠道标识
	 * @param ordertype 操作类型，0订购，1退订 
	 * @return
	 */
	@Cacheable(value="invokeConf")
	List<HashMap<String, String>> getInvokeConfByOrdertype(@Param("channel")String channel, @Param("ordertype")String ordertype);
	
	/**
	 * 根据SPID获取订购同步渠道配置
	 * @param spid
	 * @param orderchannel
	 * @return
	 */
	List<HashMap<String, String>> getOrderSyncConf(@Param("spid")String spid, @Param("orderchannel")String orderchannel);
	
	/**
	 * 根据SPID获取退订同步渠道配置
	 * @param spid
	 * @param orderchannel
	 * @param cancelchannel
	 * @return
	 */
	List<HashMap<String, String>> getCancelSyncConf(@Param("spid")String spid, @Param("orderchannel")String orderchannel, @Param("cancelchannel")String cancelchannel);
	
	/**
	 * 获取REC表处理状态为2，处理时间小于当前时间10分钟的数据，用于更新REC表的状态
	 * @return
	 */
	List<HashMap<String, String>> getRecUpdData();
	
	/**
	 * 获取同步已完成的数据，用于数据迁移
	 * @return
	 */
	List<String> getWaitDoneData();
	
	/**
	 * 获取REC表三天前已完成的数据
	 * @return
	 */
	List<String> getRecDoneData();
	
	int execDelSql(String sql);
	
	int execAddSql(String sql);
	
	/**
	 * 更新WAIT表状态为处理中超过5分钟的数据为待处理
	 * @return
	 */
	int updWaitSyncError();
	
	/**
	 * 获取表配置TB_CSMAKEUP_PROD_CONF
	 * @param spid
	 * @return
	 */
	List<HashMap> getCsmakeUpBusiConf(@Param("spid")String spid);
	
	String getNo();
	
	/**
	 * 添加记录到TD_CSMAKEUP_ORDER_REC
	 * @param params
	 * @return
	 */
	int addOrderRec(HashMap params);
	
	/**
	 * 获取渠道名称
	 * @param channel
	 * @return
	 */
	@Cacheable(value="channelname")
	String getChannelName(@Param("channel")String channel);
	
	/**
	 * 获取SPID的配置信息
	 * @param spid
	 * @return
	 */
	@Cacheable(value="tbspinfo")
	List<HashMap<String, String>> getTbSpInfo(@Param("spid")String spid);
	
	/**
	 * 添加柏源同步日志
	 * @param bean
	 * @return
	 */
	int insertBoYuanLogBean(BoYuanLogBean bean);
	
	/**
	 * 获取TB_SP_AOP_CONFIG表配置信息
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getAopConf(@Param("spid")String spid);
	
	
	/**
	 * 获取TB_CHSYNC_QOS_PACKAGEIDCONF表配置信息
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getQosPackageId(@Param("spid")String spid);

	/**
	 * 获取TB_SP_INFO表信息
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getSpInfo(@Param("spid")String spid);

	/**
	 * 获取TB_CDM_FLOW_CONF表信息
	 * @param cpid
	 * @return
	 */
	List<HashMap<String, String>> getCdnFlowConfPid(@Param("cpid")String cpid);

	/**
	 *
	 * 获取TD_ORDER_RELATIONS二级渠道
	 * @param id
	 * @return
	 */
	String getOrderchannel2(@Param("id")String id);

	/**
	 *
	 * 获取扩展字段
	 * @param  spid
	 * @param  orderchannel
	 * @param  invokechannel
	 * @return
	 */
	List<HashMap<String, String>> getExtend(@Param("spid")String spid,@Param("orderchannel")String orderchannel,@Param("invokechannel")String invokechannel);

	/**
	 *
	 * 禁用同步任务
	 * @param  spid
	 * @param  orderchannel
	 * @param  invokechannel
	 * @return
	 */
	int updateStatus(@Param("spid")String spid,@Param("orderchannel")String orderchannel,@Param("invokechannel")String invokechannel);

	/**
	 *
	 * 查询spid 是否在表TB_CHSYNC_CHANNEL2中
	 * @param  spid
	 * @return
	 */
	int chackSpid(@Param("spid")String spid,String invokechannel);

	/**
	 *
	 * 获取二级渠道同步信息   TB_CHSYNC_CHANNEL2
	 * @param  spid
	 * @return
	 */
	List<HashMap<String, String>> getSyncChannel2(@Param("spid")String spid, @Param("channel")String orderchannel,@Param("channel2")String orderchannel2);

	List<HashMap> getOrderRealById(@Param("id") String id);

	/**
	 * 获取TB_SP_QOS_CONFIG表配置信息
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getSpAopConf(@Param("spid")String spid);

}
