package com.chinacreator.kafka.mapper;

import java.util.HashMap;
import java.util.List;

import org.apache.ibatis.annotations.Param;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.bean.ProvinceBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.sync.bean.BoYuanLogBean;

/**
 * 渠道同步dao
 * @author xu.zhou
 * @date 20210618
 */
public interface HuaShengSyncDao {
	
	/**
	 * 根据SPID获取待同步的数据
	 * @param spid
	 * @return
	 */
	List<HashMap<String, String>> getSyncWaitData(int page);
	
	/**
	 * 更新数据为同步中
	 * @param id
	 * @return
	 */
	int updWaitSyncing(@Param("id")String id);
	
	/**
	 * 查询接口配置信息
	 * @param projname
	 * @param invokename
	 * @return
	 */
	@Cacheable(value="kafkaorder-projinvokeConf")
	List<HashMap<String, String>> getProjinvokeConf(String projname, String invokename);
		
	/**
	 * 批量更新数据为同步中
	 * @param ids
	 * @return
	 */
	int updBatchWaitSyncing(List<String> ids);

	/**
	 * 根据id更新同步结果
	 * @param id
	 * @param resultcode
	 * @param resultinfo
	 * @return
	 */
	int updWaitSyncRes(@Param("id")String id, @Param("resultcode")String resultcode, @Param("resultinfo")String resultinfo, @Param("inputparams")String inputparams);
	
	/**
	 * 更新WAIT表状态为处理中超过5分钟的数据为待处理
	 * @return
	 */
	int updWaitSyncError();
	
	/**
	 * 获取渠道配置信息
	 * @param channel
	 * @return
	 */
	@Cacheable(value="kafkaorder-channelInfo")
	List<HashMap<String, String>> getChannelInfo(@Param("channel")String channel);
}
