package com.chinacreator.kafka.mapper;

import java.util.HashMap;
import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.bean.ProvinceBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
@Mapper
public interface KafkaInfoDao {

	List<HashMap> getTopicConf();

	/**
	 * 产品ID与SPID匹配信息表
	 * @return
	 */
	@Cacheable(value="productConf")
	List<HashMap> getProductConf(@Param("productid")String productid);



	List<HashMap> getProductConfAll();

	/**
	 * 获取定时任务配置
	 * @return
	 */
	List<TbKafkaJobConf> getJobConf();

	/**
	 * 添加数据到TD_KAFKA_REQUEST表
	 * @param bean
	 * @return
	 */
	int addSyncPutData(KafkaBean bean);

	/**
	 * 获取字典表数据
	 * @param name
	 * @return
	 */
	@Cacheable(value="dictionary")
	String getDictionaryValue(@Param("name")String name);

	/**
	 * 获取要同步的数据TD_KAFKA_REQUEST
	 * @param page
	 * @return
	 */
	List<KafkaBean> qrySyncData(@Param("page")int page);

	/**
	 * 批量更新为正在处理状态TD_KAFKA_REQUEST
	 * @param list
	 * @return
	 */
	int updBatchRequestData(List<KafkaBean> list);

	/**
	 * 更新处理结果TD_KAFKA_REQUEST
	 * @param bean
	 * @return
	 */
	int updRequestDataRes(@Param("resultcode")String resultcode, @Param("errorinfo")String errorinfo, @Param("id")String id);

	/**
	 * 写订购日志TL_ORDER_LOG
	 * @param info
	 * @return
	 */
	int writeLogOrder(OrderInfo info);

	/**
	 * 写退订日志TL_ORDER_LOG
	 * @param info
	 * @return
	 */
	int writeLogCancel(OrderInfo info);

	/**
	 * 同步订购操作TD_ORDER_RELATIONS
	 * @param info
	 * @return
	 */
	int syncOrder(OrderInfo info);

	/**
	 * 同步退订操作TD_ORDER_RELATIONS
	 * @param info
	 * @return
	 */
	int syncCancel(OrderInfo info);

	/**
	 * 获取省份地市信息
	 * @param callerhead
	 * @return
	 */
	ProvinceBean getProvInfo(@Param("callerhead")String callerhead);

	/**
	 * 获取用户的订购关系
	 * @param userid
	 * @return
	 */
	List<OrderInfo> findByUser(@Param("userid")String userid);

	/**
	 * 获取当前时间
	 * @return
	 */
	String getCurrentTime();

	/**
	 * 获取指定日期当月的最后一天最后一秒
	 * @param datestr
	 * @return
	 */
	String getEndDayOfCurrentMonth(@Param("datestr")String datestr);

	/**
	 * 获取监控配置
	 * @return
	 */
	List<HashMap> getMonitorConf();

	/**
	 * 更新监控值
	 * @param sql
	 * @return
	 */
	int updMonitorBySql(String sql);

	/**
	 * 执行动态SQL
	 * @param sql
	 * @return
	 */
	int execSql(String sql);

	/**
	 * 获取迁移配置信息
	 * @return
	 */
	List<HashMap> getSrcMoveConf();

	/**
	 * 获取业务订购退订限制配置信息
	 * @param spid
	 * @return
	 */
	@Cacheable(value="spbusiLimitConf")
	List<HashMap> getSpBusiLimitConf(@Param("spid")String spid);

	/**
	 * 根据spid获取cpid
	 * @param spid
	 * @return
	 */
	String getCpid(@Param("spid")String spid);
}
