package com.chinacreator.kafka.mapper;

import com.chinacreator.kafka.bean.KuaishouBean;
import com.chinacreator.kafka.bean.KuaishouPushBean;
import com.chinacreator.kafka.bean.OrderInfo;
import java.util.HashMap;
import java.util.List;
import org.apache.ibatis.annotations.Param;

/**
 * 快手推送处理
 * @author xu.zhou
 * @date 20200507
 */
public interface KuaishouDao{
	
  /**
   * 添加数据
   * @param list
   * @throws SQLException  ORA-00001违反唯一约束
   */
  	public void insert(KuaishouBean bean);
    
  	/**
  	 * 添加推送数据
  	 * @param bean
  	 * @throws SQLException
  	 */
  	public void insertPush(KuaishouPushBean bean);

  	/**
  	 * 根据手机号码和月份查询数据
  	 * @param serialNumber
  	 * @param pushmonth
  	 * @return
  	 * @throws SQLException
  	 */
  	public List<KuaishouBean> query(@Param("serialNumber")String serialNumber, @Param("pushmonth")String pushmonth);
  	
  	/**
  	 * 根据ELEMENTID获取SPID
  	 * @param elementid
  	 * @return
  	 * @throws SQLException
  	 */
  	public String getSpidByElementid(@Param("elementid")String elementid);
  	
  	/**
  	 * 获取ID
  	 * @return
  	 * @throws SQLException
  	 */
  	public String getNo();
  
  	/**
  	 * 查询当月已推送的数据
  	 * @param serialNumber 手机号码
  	 * @param pushmonth 推送月份
  	 * @param pushtype 推送类型：0为流量已耗尽，1为流量未耗尽，2为月末推送     
  	 * @return
  	 */
  	public List<KuaishouPushBean> queryPush(@Param("serialNumber")String serialNumber,@Param("pushmonth")String pushmonth, @Param("pushtype")String pushtype);
  

  /**
   * 查询订购关系
   * @param userid
   * @param pushmonthFistDay 流量数据月份的第一天第一秒
   * @return
   * @throws SQLException
   */
  public List<OrderInfo> findByUser(@Param("userid")String userid,@Param("pushmonthFistDay")String pushmonthFistDay);
  
  /**
   * 查询用户快手有效的订购关系的SPID，剔除重复数据
   * @param userid
   * @param pushmonthFistDay 流量数据月份的第一天第一秒
   * @return
   * @throws SQLException
   */
  public List<OrderInfo> findspidByUser(@Param("userid")String userid,@Param("pushmonthFistDay")String pushmonthFistDay);


	/**
	 * 下发短信结束后插入表
	 * @return
	 * @throws SQLException
	 */
	public int sendSmsRec(HashMap map);
	
	/**
	 * 更新发送短信结果
	 * @param map
	 * @return
	 */
	public int updSmsRec(HashMap map);
	
	/**
	 * 查询是否有记录
	 * @param map
	 * @return
	 */
	public int qrySmsRec(HashMap map);
 
}