package com.chinacreator.kafka.quartz;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.ChengkaDeal;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 橙卡数据处理类
 * 对应原处理类：/kafka-ftp/src/com/chinacreator/kafka/ChengkaMain.java
 * @author xu.zhou
 * @date 20201216
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ChenkaQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("chengka");
	
	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			KafkaInfoDao kIDao = (KafkaInfoDao) SpringContextUtils.getBean(KafkaInfoDao.class);
			//获取要处理的数据
			List<SourceDataRecBean> list = dao.getChengkaSrcData(jobConf.getPAGES());
			logger.info("获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				//批量更新为待处理状态
				dao.updBatchChengkaSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(10, 20, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					ChengkaDeal prothread = new ChengkaDeal(bean, list.size(), threadSignal, logger, dao, kIDao);
					executorService.execute(prothread);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("橙卡数据处理出现异常，"+e.getMessage(),e);
		}	
	}
}
