package com.chinacreator.kafka.quartz;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.util.ParamsConfig;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 抖音数据处理
 * 对应原工程文件：/kafka-ftp/src/com/chinacreator/kafka/DouyinMain.java
 * @author xu.zhou
 * @date 20201216
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class DouyinQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("douyinDeal");
	
	//private static Logger logger = Logger.getLogger(DouyinQuartz.class);
	
	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			ParamsConfig pc = (ParamsConfig)SpringContextUtils.getBean(ParamsConfig.class);
			String douyinurl = pc.douyinurl;
			String douyinpwd = pc.douyinpwd;
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
			String starttime = f.format(new Date());
			logger.info("开始查询数据 ，" + starttime);
			//获取要处理的数据
			List<SourceDataRecBean> list = dao.getDouyinSrcData(jobConf.getPAGES());
			logger.info("查询数据结束，开始时间=>"+starttime+"，结束时间=>"+f.format(new Date())+", 获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				//批量更新为待处理状态
				dao.updBatchDouyinSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					DouyinDeal prothread = new DouyinDeal(bean, list.size(), threadSignal, logger, dao, douyinurl, douyinpwd);
					executorService.execute(prothread);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("抖音数据处理出现异常，"+e.getMessage(),e);
		}	
	}
}
