package com.chinacreator.kafka.quartz;

import com.chinacreator.kafka.bean.ReflowBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.FlashSMSDeal;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.jboss.logging.Logger;
import org.quartz.*;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.*;

/**
 * @Author shuiying.ou
 * @Date 2022-06-07 13:59:22
 * @DESCRIPTION 定时扫描下发的回流短信
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class FlashSMSQuartz extends BaseService implements Job {


    private static Logger logger = Logger.getLogger("flashsms");

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        //获取配置信息
        TbKafkaJobConf jobConf = (TbKafkaJobConf) jobExecutionContext.getJobDetail().getJobDataMap().get("data");
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String starttime = f.format(new Date());
        ActivityReflowDao activityReflowDao = SpringContextUtils.getBean(ActivityReflowDao.class);

        try {
            logger.info("开始查询数据 ，" + starttime);
            //获取要处理的数据
            List<ReflowBean> originalList = activityReflowDao.findSms();
            logger.info("查询数据结束，开始时间=>" + starttime + "，结束时间=>" + f.format(new Date()) + ", 获取数据条数=> " + originalList.size());

            if (originalList.size() > 0) {
                CountDownLatch threadSignal = new CountDownLatch(originalList.size());
                ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
                for (ReflowBean bean : originalList) {
                    FlashSMSDeal deal = new FlashSMSDeal(bean, threadSignal, logger, activityReflowDao);
                    executorService.execute(deal);
                }
                //等待多线程处理完
                executorService.shutdown();
                try {
                    executorService.awaitTermination(1L, TimeUnit.MINUTES);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            } else {
                sleep(Long.parseLong(jobConf.getSLEEPTIME() + ""), logger);
            }
            logger.info("数据处理完成");

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("处理出现异常，" + e.getMessage(), e);
        }

    }
}

