package com.chinacreator.kafka.quartz;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.util.ParamsConfig;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 快手数据移动到历史表
 * @author xu.zhou
 * @date 20201222
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class KuaishouMoveQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("movedata");
	
	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			List<String> list = new ArrayList<String>();
			String extend1 = jobConf.getEXTEND1();		
			
			//要处理的表配置化，多个表名用#分隔
			if(extend1 != null && !"".equals(extend1) && !"null".equals(extend1)){
				list = Arrays.asList(extend1.split("#"));
			}
			
			//要重试的错误编码
			String retrycodes = "";
			if(list != null && list.size() > 0){
				for(String s : list){
					if(!StringUtils.isEmpty(s)){
						retrycodes += ",'"+s+"'";
					}
				}
			}

			if(!"".equals(retrycodes)){
				retrycodes = retrycodes.substring(1);
			}

			String sql = "";
	
			//移除历史数据.移除6小时之前且已处理成功的数据
			sql = "SELECT ID FROM (SELECT ID FROM TD_KAFKA_SRCDATA_KUAISHOU WHERE RESULTCODE = '0' AND INSERTTIME < SYSDATE -6/24) WHERE ROWNUM <= "+jobConf.getPAGES();
			moveHis(dao, sql, jobConf.getSLEEPTIME());
			
			if(!"".equals(retrycodes)){
				//失败数据重试
				sql = " UPDATE TD_KAFKA_SRCDATA_KUAISHOU SET RESULTCODE = '1' WHERE  RESULTCODE IN ("+retrycodes+") ";
				updRetry(dao,sql);
			}
			
			//更新数据为正在处理的数据为待处理，更新时间小于当前时间30分钟
			sql = " UPDATE TD_KAFKA_SRCDATA_KUAISHOU SET RESULTCODE = '1' WHERE  RESULTCODE = '2' AND UPDTIME < SYSDATE -30/(24*60) ";
			updEffic(dao,sql);
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("移动历史出现异常，"+e.getMessage(),e);
		}	
	}
	
	//移除历史数据
	private void moveHis(SourceDataRecDao dao, String sql, int sleeptime){
		List<String> ids = dao.getMoveSrcDataBySql(sql);
		logger.info("获取数据条数=> " + ids.size());
		if(ids != null && ids.size()>0){
			//使用事务控制
			PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
			TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
			TransactionStatus transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
			try {
				int rows = dao.moveBatchKuaishouSrcData(ids);
				logger.info("移动数据行数=>"+rows);
				rows = dao.delBatchKuaishouSrcData(ids);
				logger.info("删除数据行数=>"+rows);
				dataSourceTransactionManager.commit(transactionStatus);//提交
			} catch (Exception e) {
				e.printStackTrace();
				dataSourceTransactionManager.rollback(transactionStatus);
				logger.error("移动出现异常，"+e.getMessage(),e);
			}
		}
	}
	
	//失败数据重试
	private void updRetry(SourceDataRecDao dao, String sql){
		int rows = dao.updSrcDataBySql(sql);
		logger.info("更新重试数据条数=>"+rows);
	}
	
	//更新数据为正在处理的数据为待处理
	private void updEffic(SourceDataRecDao dao, String sql){
		int rows = dao.updSrcDataBySql(sql);
		logger.info("更新数据为正在处理的数据为待处理条数=>"+rows);
	}
}
