package com.chinacreator.kafka.quartz;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import com.chinacreator.kafka.bean.KuaishouBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KuaishouDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.service.KuaishouDeal;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 抖音数据处理
 * 对应原工程文件：/kafka-ftp/src/com/chinacreator/kafka/KuaishouMain.java
 * @author xu.zhou
 * @date 20201216
需求名称：
0427-快手及抖音定向流量包流量状态数据同步接口-林珍
 
【提供给快手侧的接口】
1、将kafka传来的快手流量数据进行判断，仅传给快速快手“快手流量耗尽”以及“快手流量未耗尽”的2个状态。判断场景如下：
1）当用户当月订购一个快手前向/后向产品，将该用户“快手流量未耗尽”状态同步给快手，后续当月收到该产品流量耗尽且用户未订购其它快手产品时，则同步给快手“快手流量耗尽”状态。
2）当用户当月订购一个快手前向/后向产品，且在同步给快手“快手流量耗尽”状态前，当月又订购其他快手产品时，此时不同步“快手流量未耗尽”状态。
3）当用户当月订购一个快手前向/后向产品，且在同步给快手“快手流量耗尽”状态后，当月又订购其他快手产品时，此时需向快手侧同步“快手流量未耗尽”状态。
后续收到kafka当月所有快手流量都耗尽后，再同步给快手“快手流量耗尽”状态。

2、每月月底最后一天，对快手前向产品未退订状态的用户，进行状态恢复，并在当日批量同步给快手侧。

请评估工期和接口何时可提供，谢谢！

快手产品分类：
1. 后向：快手后向结束时间只到月底，不重复追加，也就是重复订购无效
2. 前向：包月

快手推送逻辑：
场景1（只订购一个快手产品）：
用户订购快手前向8元（之前未订购任何快手产品），给快手推送“快手流量未耗尽”，当收到快手kafka的流量消耗100%的数据，则向快手推送“快手流量已耗尽”。

场景2（订购了多个快手产品，流量不连续，单个产品流量消耗有100%，都推送“流量已耗尽”）：
用户4号先订购快手前向8元（之前未订购任何快手产品），给快手推送“快手流量未耗尽”，
6号收到kafka的流量消耗100%的数据，向快手推送“流量已耗尽”。
用户8号又订购快手前向15元，给快手推送“流量未耗尽”，
10号收到kafka的流量消耗100%的数据，向快手推送“流量已耗尽”。

场景3（订购了多个快手产品，流量连续，所有产品流量都消耗100%时才推送“流量已耗尽”）：
用户4号先订购快手前向8元（之前未订购任何快手产品），给快手推送“流量未耗尽”，
用户5号又订购快手前向15元，不向快手推送“流量未耗尽”，
6号收到kafka的流量消耗100%的快手8元数据，此时不向快手推送“流量已耗尽”，因为没收到kafka的流量消耗100%的快手15元数据。
10号收到kafka的流量消耗100%的快手15元数据，向快手推送“流量已耗尽”。

总结：要给快手推送“流量已耗尽”必须用户订购的所有快手产品都要收到kafka的流量消耗100%的通知才能推送。

“流量已耗尽”的具体处理方式：
1. 获取当月快手推送已消耗完的所有SPID数据（以DEAL_TIME时间值为准，每5分钟扫一次表），可能有一条，可能有多条，
1.1 判断是否还订购了其他快手产品
1.1.1 是，不推送，因为没有收到所有产品消耗完的快手数据。
1.1.2 否，再判断当月是否已经推送过“已消耗完”，如果没推送过，则触发推送，如果已经推送过，则不处理。

“流量未耗尽”的具体处理方式：
1. 当用户订购快手产品时触发（给订购关系表添加触发器，有订购时把数据添加到指定表）
2. 先判断当月是否有推送过“流量未耗尽”，防止用户当月退订后当月又重新订购（根据SPIDS是否包含），如果有，则推送，反之不推送。

定时任务：
月初凌晨查询所有订购快手前向且未退订或当月退订的用户，剔重，向快手推送“流量未耗尽”，推送前查询推送记录表有没有月初推送类型，防止重复推送。 * 

订购关系表增加触发器，把订购kuaishou的业务数据添加到表TD_KUAISHOU_ORDER

相关表：
1. TD_KAFKA_SRCDATA_KUAISHOU KAFKA原始数据表
2. TD_KUAISHOU_ORDER 快手订购记录表，通过订购关系表的触发器生成数据
3. TD_KAFKA_KUAISHOU_PUSH 快手流量是否耗尽推送记录表
4. TD_KAFKA_KUAISHOU 快手KAFKA流量消耗通知数据（从KAFKA原始数据里获取，同一手机号码同一产品同一月份只录入一条数据）
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class KuaishouQuartz extends BaseService implements Job {
	
	private static Logger logger = Logger.getLogger("kuaishouDeal");
		
	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			KuaishouDao kuaishouDao = (KuaishouDao) SpringContextUtils.getBean(KuaishouDao.class);
			SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
			String starttime = f.format(new Date());
			logger.info("开始查询数据 ，" + starttime);
			//获取要处理的数据
			List<SourceDataRecBean> list = dao.getKuaishouSrcData(jobConf.getPAGES());
			logger.info("查询数据结束，开始时间=>"+starttime+"，结束时间=>"+f.format(new Date())+", 获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				List<KuaishouBean> resList = this.parseData(list);
				//批量更新为待处理状态
				dao.updBatchKuaishouSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(20, 30, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (KuaishouBean bean : resList) {
					KuaishouDeal prothread = new KuaishouDeal(bean, resList.size(), threadSignal, logger, dao, kuaishouDao);
					executorService.execute(prothread);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("快手数据处理出现异常，"+e.getMessage(),e);
		}	
	}
	
	//去除重复数据，获取解析后的数据
	private List<KuaishouBean> parseData(List<SourceDataRecBean> list){
		logger.info("去重复前数据个数："+list.size());
		HashMap<String, Object> map = new HashMap<String, Object>();
		List<KuaishouBean> resList = new ArrayList<KuaishouBean>();
		KuaishouBean bean = null;
		SourceDataRecBean tmp = null;
		for(int i=0; i<list.size(); i++){
			tmp = list.get(i);
			//{"SERIAL_NUMBER":"15500155945","ELEMENT_ID":"8664946","REMAIN":"812","KAFKA_TYPE":"r","DEAL_TIME":"20201210163657","PERCENT":"80","EPARCHY_CODE":"0434","USER_ID":"9019062464979374","ADD_PRODUCT_ID":"90526927","USED_VALUE":"3.21","PROVINCE_CODE":"90"}
			bean = (KuaishouBean) JsonUtil.jsonToBean(tmp.getSourcedata(), KuaishouBean.class);
			bean.setGETTIME(tmp.getInserttime());
			bean.setDEAL_MONTH(bean.getDEAL_TIME().substring(0, 6));
			bean.setFILENAME(tmp.getId()); //设置文件名为入库表的ID
			bean.setID(tmp.getId());
			//if(map.containsKey(bean.getSERIAL_NUMBER()+bean.getSPID()+bean.getDEAL_MONTH())){
			//按手机号码和月份进行剔重处理
			if(map.containsKey(bean.getSERIAL_NUMBER()+bean.getDEAL_MONTH())){
				//重复数据，不处理
				list.remove(tmp);
				i--;
				logger.info("重复数据，"+tmp.getId());
			}else{
				//map.put(bean.getSERIAL_NUMBER()+bean.getSPID()+bean.getDEAL_MONTH(), tmp);
				map.put(bean.getSERIAL_NUMBER()+bean.getDEAL_MONTH(), tmp);
				resList.add(bean);
			}
		}
		logger.info("去重复后数据个数："+list.size());
		return resList;
	}
}
