package com.chinacreator.kafka.quartz;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.util.ParamsConfig;
import com.chinacreator.kafka.util.SpringContextUtils;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 监控
 * @author xu.zhou
 * @date 20201223
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class MonitorQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger(MonitorQuartz.class);
	
	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			KafkaInfoDao kafkaInfo = (KafkaInfoDao) SpringContextUtils.getBean(KafkaInfoDao.class);
			List<HashMap> monitorList = kafkaInfo.getMonitorConf();
			//logger.info("monitorList=>"+monitorList);
			String monisql = "";
			String monitype = "";
			String id = "";
			int rows = 0;
			String exesql = "";
			//logger.info("consLiveMap=>"+ThreadLimitUtil.consLiveMap);
			for(HashMap hm : monitorList){	
				monisql = (String)hm.get("MONISQL"); //执行SQL，或者其他变量
				monitype = (String)hm.get("MONITYPE");
				id = hm.get("ID").toString();
				//执行监控SQL
				if("1".equals(monitype) && monisql != null){					
					exesql = monisql.replace("#ID#", id);
					try {
						rows = kafkaInfo.updMonitorBySql(exesql);
						//System.out.println(id+"，执行SQL影响行："+rows);
					} catch (Exception e) {
						e.printStackTrace();
						logger.error(exesql+"，执行SQL出现异常，"+e.getMessage(),e);
					}
				//执行topic消费端存活监控
				}else if("2".equals(monitype) && monisql != null){
					String consLiveMaxTime = (String)hm.get("EXTEND1");
					if(StringUtils.isEmpty(consLiveMaxTime)){
						consLiveMaxTime = "300";
					}
					//获取topic消费存活状态
					boolean res = ThreadLimitUtil.hasConsLive(monisql, Long.parseLong(consLiveMaxTime));
					if(res){
						//设置监控结果为活跃状态
						exesql = "UPDATE TD_KAFKA_MONITOR_REC SET UPDTIME = SYSDATE, MONIVAL = 0 WHERE ID = '"+id+"'";
					}else{
						//设置监控结果为消亡状态
						exesql = "UPDATE TD_KAFKA_MONITOR_REC SET UPDTIME = SYSDATE, MONIVAL = 1 WHERE ID = '"+id+"'";
					}
					try {
						rows = kafkaInfo.updMonitorBySql(exesql);
						//System.out.println(id+"，执行SQL影响行："+rows);
					} catch (Exception e) {
						e.printStackTrace();
						logger.error(exesql+"，执行SQL出现异常，"+e.getMessage(),e);
					}
					/****
					只有活跃状态的才验证是否积压
					判断队列是否积压，如果是，则更新监控数据为2，代表当前topic消费有积压，并且超过5分钟
					****/
					if(res && ThreadLimitUtil.hasStockRecords(monisql)){
						//设置监控结果为积压状态
						exesql = "UPDATE TD_KAFKA_MONITOR_REC SET UPDTIME = SYSDATE, MONIVAL = 2 WHERE ID = '"+id+"'";
						try {
							rows = kafkaInfo.updMonitorBySql(exesql);
							//System.out.println(id+"，执行SQL影响行："+rows);
						} catch (Exception e) {
							e.printStackTrace();
							logger.error(exesql+"，执行SQL出现异常，"+e.getMessage(),e);
						}
					}
				}
			}			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("监控定时任务出现异常，"+e.getMessage(),e);
		}	
	}
}
