package com.chinacreator.kafka.quartz;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 历史数据迁移、失败数据重试、超时数据重试
 * @author xu.zhou
 * @date 20210112
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class MoveQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("movedata");
	
	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		logger.info("/***********************迁移数据执行开始，"+f.format(new Date())+"*********************/");
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			KafkaInfoDao kiDao = (KafkaInfoDao) SpringContextUtils.getBean(KafkaInfoDao.class);
			//获取迁移执行SQL配置数据
			List<HashMap> confList = kiDao.getSrcMoveConf();
			if(confList != null && confList.size()>0){
				String execsql = "";
				String srctabname = ""; //源数据表名
				String movetabname = "";//历史数据表名
				for(HashMap tmpMap : confList){
					logger.info(tmpMap.get("NAME")+"======START========"+f.format(new Date()));
					execsql = (String)tmpMap.get("EXECSQL");
					if("1".equals(tmpMap.get("BUSITYPE"))){//1数据迁移
						srctabname = (String)tmpMap.get("SRCTABNAME");
						movetabname = (String)tmpMap.get("HISTABNAME");
						//迁移历史数据
						moveHis(dao, execsql, srctabname, movetabname);
					}else if("2".equals(tmpMap.get("BUSITYPE"))){//2直接执行SQL
						execSql(dao,execsql);
					}
					//更新执行时间
					execsql = "UPDATE TB_KAFKA_SRCMOVE_CONF SET UPDTIME = SYSDATE WHERE ID = '"+tmpMap.get("ID")+"'";
					execSql(dao,execsql);
					logger.info(tmpMap.get("NAME")+"======END========"+f.format(new Date()));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("移动历史出现异常，"+e.getMessage(),e);
		}	
		logger.info("/***********************迁移数据执行完成，"+f.format(new Date())+"*********************/");
	}
	
	/**
	 * 执行SQL
	 * @param execsql
	 */
	private void execSql(SourceDataRecDao dao, String execsql){
		try {
			int rows = dao.execSql(execsql);
			logger.info("执行SQL=>"+execsql+"，影响行=>"+rows);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("执行SQL=>"+execsql+"，执行SQL出现异常，"+e.getMessage(),e);
		}
	}
	
	/**
	 * 迁移历史数据
	 * @param dao
	 * @param sql 获取要迁移数据的SQL
	 * @param srctabname 源数据表名
	 * @param movetabname 历史数据个表名
	 */
	private void moveHis(SourceDataRecDao dao, String sql, String srctabname, String movetabname){
		String execsql = sql;
		try {
			if(StringUtils.isEmpty(srctabname) || StringUtils.isEmpty(movetabname)){
				logger.info("源数据表名或历史数据表名为空，流程中止。srctabname=>"+srctabname+"，movetabname=>"+movetabname);
				return;
			}else{
				execsql = execsql.replace("#SRCTABNAME#", srctabname);
			}
			logger.info("获取要迁移的数据执行SQL=>"+execsql);
			List<String> ids = dao.getMoveSrcDataBySql(execsql);
			logger.info(srctabname+"，获取数据条数=> " + ids.size());
			if(ids != null && ids.size()>0){
				String instr = ids.toString().replaceAll(", ", "','").replace("[", "('").replace("]", "')");
				//使用事务控制
				PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
				TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
				TransactionStatus transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
				try {
					execsql = "INSERT INTO "+movetabname+" SELECT * FROM "+srctabname+" WHERE ID IN "+instr;	
					logger.info("迁移执行SQL=>"+execsql);
					int rows = dao.execSql(execsql);
					logger.info("迁移数据行数=>"+rows);
					execsql = " DELETE FROM "+srctabname+" WHERE ID IN "+instr;
					logger.info("删除执行SQL=>"+execsql);
					rows = dao.execSql(execsql);
					logger.info("删除数据行数=>"+rows);
					dataSourceTransactionManager.commit(transactionStatus);//提交
				} catch (Exception e) {
					e.printStackTrace();
					dataSourceTransactionManager.rollback(transactionStatus);
					logger.error("迁移历史数据出现异常，"+e.getMessage()+"，执行SQL=>"+execsql,e);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("迁移历史数据出现异常，"+e.getMessage()+"，执行SQL=>"+execsql,e);
		}
	}
}
