package com.chinacreator.kafka.quartz;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.QueenCardBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.QueenCardDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.QueenCardDeal;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ParamsConfig;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 处理女王卡原始数据表
 * 对应原工程文件：/kafka-ftp/src/com/chinacreator/kafka/Main.java
 * @author xu.zhou
 * @date 20201217
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class QueenCardQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("queencard");

	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			ParamsConfig pc = (ParamsConfig)SpringContextUtils.getBean(ParamsConfig.class);
			//#女王卡产品编码，多个用|分隔
			String qcproductid = pc.qcproductid;
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			QueenCardDao qcDao = (QueenCardDao)SpringContextUtils.getBean(QueenCardDao.class);
			//获取要处理的数据
			List<SourceDataRecBean> list = dao.getQueenCardSrcData(jobConf.getPAGES());
			logger.info("获取数据条数=> " + list.size());
			parseData(list); //按手机号码去除重复数据
			if(list != null && list.size()>0){
				//批量更新为待处理状态
				dao.updBatchQueenCardSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					QueenCardDeal deal = new QueenCardDeal(bean, list.size(), threadSignal, logger, dao, qcproductid, qcDao);
					executorService.execute(deal);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("女王卡数据处理出现异常，"+e.getMessage(),e);
		}	
	}
	
	/**
	 * 按手机号码去除重复数据
	 * @param list
	 */
	private void parseData(List<SourceDataRecBean> list){
		if(list == null || list.size() == 0){
			return;
		}
		logger.info("去重复前数据个数："+list.size());
		HashMap<String, Object> map = new HashMap<String, Object>();
		SourceDataRecBean tmp = null;
		String userid = "";
		for(int i = 0; i<list.size(); i++){
			tmp = list.get(i);
			userid = this.getUserid(tmp);
			if(userid != null && !"".equals(userid)){
				if(map.containsKey(userid)){
					//重复数据，不处理
					list.remove(tmp);
					i--;
					logger.info("重复数据，"+tmp.getId());
				}else{
					map.put(userid, "");
				}
			}
		}
		logger.info("去重复后数据个数："+list.size());
	}
	
	/**
	 * 获取手机号码
	 * @param sdrBean
	 * @return
	 */
	private String getUserid(SourceDataRecBean sdrBean){
		String userid = "";
		String logname = sdrBean.getOldlogname();
		String jsondata = sdrBean.getSourcedata();
		if (logname.contains("bg")) {
			BGbean bgBean = (BGbean) JsonUtil.jsonToBean(jsondata, BGbean.class);
			userid = bgBean.getSERIAL_NUMBER();
		}else if(logname.contains("msglog")){
			QueenCardBean bean = (QueenCardBean) JsonUtil.jsonToBean(jsondata, QueenCardBean.class);
			userid = bean.getSerialNumber();
		}else{
			
		}
		return userid;
	}
}
