package com.chinacreator.kafka.quartz;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.service.SyncPutDeal;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 中间表数据TD_KAFKA_SRCDATA解析后入库到TD_KAFKA_REQUEST
 * 对应原工程文件：/kafka-ftp/src/com/chinacreator/kafka/Main.java
 * @author xu.zhou
 * @date 20201217
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class SyncPutQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("syncput");
	
	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			SourceDataRecDao dao = (SourceDataRecDao) SpringContextUtils.getBean(SourceDataRecDao.class);
			KafkaInfoDao kidao = (KafkaInfoDao) SpringContextUtils.getBean(KafkaInfoDao.class);

			//获取要处理的数据
			List<SourceDataRecBean> list = dao.getCommSrcData(jobConf.getPAGES());
			logger.info("获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				//获取产品配置
				List<HashMap> confList = kidao.getProductConfAll();
				//产品ID配置信息
				HashMap<String, String> productmap = getProductinfo(confList);
				//批量更新为待处理状态
				dao.updBatchCommSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					SyncPutDeal deal = new SyncPutDeal(bean, list.size(), threadSignal, logger, dao, productmap, kidao);
					executorService.execute(deal);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("订购数据处理出现异常，"+e.getMessage(),e);
		}	
	}
	
	/**
	 * 获取产品ID配置信息，转化成map
	 * @param confList
	 * @return
	 */
	private HashMap<String, String> getProductinfo(List<HashMap> confList){
		HashMap<String, String> productmap = new HashMap<String, String>();
		for(HashMap hm : confList){
			productmap.put(hm.get("PRODUCTID").toString(), "{\"cpid\":\""+hm.get("CPID")+"\",\"spid\":\""+hm.get("SPID")+"\",\"mutex\":\""+(hm.get("MUTEX") == null ? "" : hm.get("MUTEX"))+"\"}");
		}
		return productmap;
	}
}
