package com.chinacreator.kafka.quartz;

import com.chinacreator.kafka.bean.ReflowBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.ThreeReflowBackDeal;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.jboss.logging.Logger;
import org.quartz.*;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.*;


/**
 * @Author shuiying.ou
 * @Date 2022-07-25 11:44
 * @DESCRIPTION 三级回流用户月挽回活动
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ThreeReflowBackQuartz extends BaseService implements Job {


    private static Logger logger = Logger.getLogger("threeReflowback");
    KafkaInfoDao kafkaInfoDao = SpringContextUtils.getBean(KafkaInfoDao.class);


    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        //获取配置信息
        TbKafkaJobConf jobConf = (TbKafkaJobConf) jobExecutionContext.getJobDetail().getJobDataMap().get("data");
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String starttime = f.format(new Date());
        ActivityReflowDao activityReflowDao = SpringContextUtils.getBean(ActivityReflowDao.class);

        try {
            logger.info("开始查询数据 ，" + starttime);
            //获取三级回流用户的在总数据
            Integer totalSize = activityReflowDao.selectTotalData();
            HashMap<String, Object> map = activityReflowDao.getNumber("threeReflowBack");
            Integer value = Integer.parseInt(map.get("VALUE").toString());
            Integer count = Integer.parseInt(map.get("REMARK2").toString());
            List<ReflowBean> originalList = null;
            // 计算出每次查询的条数
            int number = new BigDecimal(totalSize).divide(BigDecimal.valueOf(value), BigDecimal.ROUND_UP).intValue();
            if (!count.equals(value - 1)){
                originalList = activityReflowDao.selectThreeReflowData(number);
                logger.info("查询数据结束，开始时间=>" + starttime + "，结束时间=>" + f.format(new Date()) + ", 获取数据总条数=> " +totalSize + ", 每次需要处理的条数=> " + originalList.size());
            }else {
                // 最后一次将剩下的数据全部查出
                originalList = activityReflowDao.selectOtherData();
                logger.info("查询数据结束，开始时间=>" + starttime + "，结束时间=>" + f.format(new Date()) + ", 获取数据总条数=> " +totalSize + ", 此次处理的条数=> " + originalList.size());
            }

            if (originalList.size() > 0) {
                for (ReflowBean reflowBean : originalList) {
                    // 将数据状态改为处理中
                    reflowBean.setBackcode("2");
                    activityReflowDao.updateBackData(reflowBean.getBackcode(), reflowBean.getId());
                }
                CountDownLatch threadSignal = new CountDownLatch(originalList.size());
                ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
                for (ReflowBean reflowBean : originalList) {
                    ThreeReflowBackDeal deal = new ThreeReflowBackDeal(reflowBean, threadSignal, logger, activityReflowDao,kafkaInfoDao);
                    executorService.execute(deal);
                }
                //等待多线程处理完
                executorService.shutdown();
                try {
                    executorService.awaitTermination(1L, TimeUnit.MINUTES);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                for (ReflowBean reflowBean : originalList) {
                    reflowBean.setBackcode("0");
                    // 将数据状态改为已处理
                    activityReflowDao.updateBackData(reflowBean.getBackcode(), reflowBean.getId());
                }
                if (count.equals(value - 1)) {
                    count = 0;
                    activityReflowDao.updateNumber(count);
                }else {
                    count = count+1;
                    activityReflowDao.updateNumber(count);
                }
            } else {
                sleep(Long.parseLong(jobConf.getSLEEPTIME() + ""), logger);
            }
            logger.info("数据处理完成");

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("处理出现异常，" + e.getMessage(), e);
        }

    }
}
