package com.chinacreator.kafka.quartz;

import com.chinacreator.kafka.bean.CheckData;
import com.chinacreator.kafka.bean.ReflowBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.ReflowNoteDeal;
import com.chinacreator.kafka.service.ThreeReflowDeal;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.jboss.logging.Logger;
import org.quartz.*;
import org.springframework.jdbc.object.BatchSqlUpdate;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.*;


/**
 * @Author shuiying.ou
 * @Date 2022-08-03
 * @DESCRIPTION 三级回流活动定时任务
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ThreeReflowQuartz extends BaseService implements Job {


    private static Logger logger = Logger.getLogger("threeReflow");

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        //获取配置信息
        TbKafkaJobConf jobConf = (TbKafkaJobConf) jobExecutionContext.getJobDetail().getJobDataMap().get("data");
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String starttime = f.format(new Date());
        ActivityReflowDao activityReflowDao = SpringContextUtils.getBean(ActivityReflowDao.class);
        KafkaInfoDao kafkaInfoDao = SpringContextUtils.getBean(KafkaInfoDao.class);

        try {
            logger.info("开始查询数据 ，" + starttime);
            //获取要处理的三级回流数据
            List<ReflowBean> originalList = activityReflowDao.qryThreeReflowData();
            logger.info("查询数据结束，开始时间=>" + starttime + "，结束时间=>" + f.format(new Date()) + ", 获取数据条数=> " + originalList.size());

            if (originalList.size() > 0) {
//                     activityReflowDao.updBatchRefolwData(originalList);
                for (ReflowBean reflowBean : originalList) {
                    reflowBean.setResultcode("2");
                    reflowBean.setResultinfo("处理中");
                    // 将数据状态改为处理中
                    activityReflowDao.updateThreeReflowData(reflowBean.getResultcode(),reflowBean.getResultinfo(),reflowBean.getId());
                }
                CountDownLatch threadSignal = new CountDownLatch(originalList.size());
                ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
                for (ReflowBean reflowBean : originalList) {
                    ThreeReflowDeal deal = new ThreeReflowDeal(reflowBean, threadSignal, logger,activityReflowDao,kafkaInfoDao);
                    executorService.execute(deal);
                }
                //等待多线程处理完
                executorService.shutdown();
                try {
                    executorService.awaitTermination(1L, TimeUnit.MINUTES);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            } else {
                sleep(Long.parseLong(jobConf.getSLEEPTIME() + ""), logger);
            }
            logger.info("数据处理完成");

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("处理出现异常，" + e.getMessage(), e);
        }

    }
}
