package com.chinacreator.kafka.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.ChengkaBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;


/**
 * 橙卡流量数据扣减处理类
 * @author xu.zhou
 * @date 20201208
 */
public class ChengkaDeal implements Runnable{
	
	private Logger logger;
	
	private SourceDataRecDao dao;
	
	private KafkaInfoDao kIDao;
	
	private int totalSize;

	private CountDownLatch threadSignal;
	
	private SourceDataRecBean sdrBean;
	
	public ChengkaDeal(SourceDataRecBean sdrBean, int totalSize, CountDownLatch threadSignal,Logger logger, SourceDataRecDao dao, KafkaInfoDao kIDao) {
		this.logger = logger;
		this.dao = dao;
		this.sdrBean = sdrBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.kIDao = kIDao;
	}
	
	public void run(){
		long start = System.currentTimeMillis();
		String resultcode = "";
		String errorinfo = "";
		try {
			String params = sdrBean.getSourcedata();
			ChengkaBean bean = (ChengkaBean) JsonUtil.jsonToBean(params, ChengkaBean.class);
			bean.setGETTIME(sdrBean.getInserttime());
			bean.setFILENAME(sdrBean.getId());
			bean.setDEAL_MONTH(bean.getArrivalTime().substring(0, 6));
			
			List<HashMap> confList = kIDao.getProductConf(bean.getProductId());
			if(confList != null && confList.size()>0){
				HashMap hm = confList.get(0);
				bean.setSPID(hm.get("SPID").toString());
				bean.setCPID(hm.get("CPID").toString());
				Date now = new Date(); // 创建一个Date对象，获取当前时间
				// 指定格式化格式
				SimpleDateFormat f = new SimpleDateFormat("yyyyMM");
				String month = f.format(now); // 将当前时间袼式化为指定的格式
				if(bean.getCycleId()!=null && (month).equals(bean.getCycleId())){
					dao.addChengkaData(bean);
					resultcode = "0";
					errorinfo = "OK";
				}else if(bean.getCycleId()!=null){
					resultcode = "8001";
					errorinfo = "不是当月账期数据=>"+bean.getProductId();
				}else {
					resultcode = "8003";
					errorinfo = "无CycleId字段，数据不做处理=>"+bean.getProductId();
				}
			}else{
				resultcode = "8002";
				errorinfo = "元素ID未匹配到SPID=>"+bean.getProductId();
			}
		} catch (Exception e) {
			//logger.error("推送数据出错：文件名=》"+filename+"，异常信息=》"+e.getMessage()+", 推送数据："+data); 
			e.printStackTrace();	
			logger.error("推送异常，"+e.getMessage()+"=>"+sdrBean.getId(),e);
			resultcode = "8000";
			errorinfo = "出现异常=>"+e.getMessage();
		} finally {
			updProcRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - start)+" ms"+", 推送结果："+resultcode+"，id："+sdrBean.getId());
		}
	}
	
	
	/**
	 * 更新
	 */
	private void updProcRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}
			//String sql = "UPDATE TD_KAFKA_SRCDATA_CHENGKA SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+sdrBean.getId()+"' ";
			//更新处理状态
			//int rows = dao.updSrcDataBySql(sql);
			//logger.info("更新数据完成，影响行："+rows);
			int rows = dao.updChengkaSrcDataRes(resultcode, errorinfo, sdrBean.getId());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+sdrBean.getId());
		} 
	}

	public static void main(String[] args) {
		Date now = new Date(); // 创建一个Date对象，获取当前时间
		// 指定格式化格式
		SimpleDateFormat f = new SimpleDateFormat("yyyyMM");
		String month = f.format(now);
		System.out.println(month);
	}
	

}
