package com.chinacreator.kafka.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import org.jboss.logging.Logger;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.ChengkaBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.AESUtil;
import com.chinacreator.kafka.util.DesUtil;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.MD5;
import com.chinacreator.kafka.util.URLUtil;

/**
 * 橙卡数据处理类
 * @author xu.zhou
 * @date 20201208
 */
public class ChengkaFlowDeal implements Runnable{
	
	private Logger logger;
	
	private SourceDataRecDao dao;
	
	private int totalSize;

	private CountDownLatch threadSignal;
	
	private ChengkaBean bean;
	
	//流量扣减地址
	private String SPFLOWCONSURL;
	
	//调接口渠道标识
	private String CONSCHANNEL;
	
	//调接口PWD
	private String CONSPWD;
	
	public ChengkaFlowDeal(ChengkaBean bean, int totalSize, CountDownLatch threadSignal,Logger logger, SourceDataRecDao dao, String SPFLOWCONSURL, String CONSCHANNEL, String CONSPWD) {
		this.logger = logger;
		this.dao = dao;
		this.bean = bean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.SPFLOWCONSURL = SPFLOWCONSURL;
		this.CONSCHANNEL = CONSCHANNEL;
		this.CONSPWD = CONSPWD;
	}
	
	public void run(){
		long start = System.currentTimeMillis();
		String resultcode = "-1";
		String errorinfo = "";
		try {
			String userid = AESUtil.encrypt(bean.getSerialNumber(), CONSPWD);
			String timestamp = System.currentTimeMillis() / 1000 + "";
			String consflow = "88888";
			String constime = bean.getArrivalTime();
			String signature = MD5.MD5Encode(userid + bean.getSPID() + CONSCHANNEL + timestamp + consflow + constime + CONSPWD);
			String dealUrl = SPFLOWCONSURL + "?userid=" + userid + "&spid=" + bean.getSPID() +
					"&consflow=" + consflow + "&channel=" + CONSCHANNEL +
					"&signature=" + signature + "&constime=" + constime + "&timestamp=" + timestamp;
			String response = URLUtil.get(dealUrl);
			errorinfo =(String) JsonUtil.getJsonValue(response, "resultInfo");
			resultcode = (String) JsonUtil.getJsonValue(response, "resultCode");
			logger.info("橙卡流量扣减调用接口扣减结果" + dealUrl + "===>" + response);
		} catch (Exception e) {
			e.printStackTrace();	
			logger.error("推送异常，"+e.getMessage()+"=>"+bean.getID(),e);
			resultcode = "8000";
			errorinfo = "出现异常=>"+e.getMessage();
		} finally {
			updProcRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - start)+" ms"+", 推送结果："+resultcode+"，id："+bean.getID());
		}
	}
	
	/**
	 * 更新
	 */
	private void updProcRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}
			//String sql = "UPDATE TD_KAFKA_CHENGKA SET STATUS = '0', CUTTIME = SYSDATE, RESULTCODE='"+resultcode+"',ERRORINFO='"+errorinfo+"' WHERE STATUS = '2' AND ID ='"+bean.getID()+"'";
			//更新处理状态
			//int rows = dao.updSrcDataBySql(sql);
			//logger.info("更新数据完成，影响行："+rows);
			
			int rows = dao.updChengkaFlowDataRes(resultcode, errorinfo, bean.getID());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+bean.getID());
		} 
	}
	

}
