package com.chinacreator.kafka.service;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;

/**
 * 橙卡数据处理类
 * @author xu.zhou
 * @date 20201208
 */
@Component
public class ChengkaService extends BaseService{
	
	private static Logger logger = Logger.getLogger(ChengkaService.class);
	
	@Autowired
	SourceDataRecDao dao;
	
	@Autowired
	KafkaInfoDao kIDao;
	
	@Value("${chengka.data.page}")
	private int page;
	
	public void process(){
		try {
			List<SourceDataRecBean> list = dao.getChengkaSrcData(page);
			logger.info("获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				//批量更新为待处理状态
				dao.updBatchChengkaSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(10, 20, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					ChengkaDeal prothread = new ChengkaDeal(bean, list.size(), threadSignal, logger, dao, kIDao);
					executorService.execute(prothread);
				}
				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(6000,logger);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("橙卡数据处理出现异常，"+e.getMessage());
		}
	}
}
