package com.chinacreator.kafka.service;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import org.jboss.logging.Logger;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.DesUtil;
import com.chinacreator.kafka.util.MD5;
import com.chinacreator.kafka.util.URLUtil;

/**
 * 抖音数据处理类
 * @author xu.zhou
 * @date 20201208
 */
public class DouyinDeal implements Runnable{
	
	private Logger logger;
	
	private SourceDataRecDao dao;
	
	private String douyinurl;
	
	private int totalSize;

	private String douyinpwd;
	
	private CountDownLatch threadSignal;
	
	private SourceDataRecBean sdrBean;
	
	public DouyinDeal(SourceDataRecBean sdrBean, int totalSize, CountDownLatch threadSignal,Logger logger, SourceDataRecDao dao, String douyinurl, String douyinpwd) {
		this.logger = logger;
		this.dao = dao;
		this.sdrBean = sdrBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.douyinurl = douyinurl;
		this.douyinpwd = douyinpwd;
	}
	
	public void run(){
		long start = System.currentTimeMillis();
		String result = "";
		String invokedata = "";
		try {
			String params = sdrBean.getSourcedata();
			String timestamp= (System.currentTimeMillis())/1000+"";
			params = DesUtil.encode(params, douyinpwd);
			String signature = MD5.MD5Encode(params+timestamp+douyinpwd);
			JSONObject reqJson = new JSONObject();
			reqJson.put("params", params);
			reqJson.put("timestamp", timestamp);
			reqJson.put("signature", signature);
			invokedata = reqJson.toJSONString();
			//logger.info("接口地址douyinurl=>"+douyinurl+", 调接口参数invokedata=>"+invokedata);
			//{"resultcode":"0","errorinfo":"ok"}
			result = URLUtil.postJson(douyinurl, invokedata, 1000 * 30);
			logger.info("调接口返回信息result=>"+result+"， 推送数据："+sdrBean.getId());		
		} catch (Exception e) {
			//logger.error("推送数据出错：文件名=》"+filename+"，异常信息=》"+e.getMessage()+", 推送数据："+data); 
			e.printStackTrace();	
			logger.error("推送异常，"+e.getMessage());
			result = "{\"resultcode\":\"8000\",\"errorinfo\":\"推送异常，"+e.getMessage()+"\"}";
		} finally {
			updProcRes(result, invokedata);
			threadSignal.countDown();
			logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - start)+" ms"+", 推送结果："+result+"，id："+sdrBean.getId());
		}
	}
	
	/**
	 * 验证调接口结果，失败的就重试
	 * @param result
	 * @param invokedata
	 * @return
	 */
	private String valiRes(String result, String invokedata){
		String res = "";
		try {
			if (result == null || "".equals(result)) {
				res = errorRetry(invokedata);
			}else{
				 //result不是json字符串时会报异常
				 Map map = (Map)JSON.parse(result);
			     if (!"0".equals(map.get("resultcode"))){ 
			    	 res = errorRetry(invokedata);
			     }else{
			    	 res = result;
			     }
			}
		} catch (Exception e) {
			res = errorRetry(invokedata);
		}
		return res;
	}
	
	/**
	 * 更新
	 */
	private void updProcRes(String result, String invokedata){
		try {
			String resultcode = "";
			String errorinfo = "";
			String res = valiRes(result, invokedata);
			try{
				Map map = (Map)JSON.parse(result);
				resultcode = map.get("resultcode")+"";
				errorinfo = map.get("errorinfo")+"";
			}catch (Exception e1){
				resultcode = "8000";
				errorinfo = res;
			}
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}
			//String sql = "UPDATE TD_KAFKA_SRCDATA_DOUYIN SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+sdrBean.getId()+"' ";
			//更新处理状态
			//int rows = dao.updSrcDataBySql(sql);
			int rows = dao.updDouyinSrcDataRes(resultcode, errorinfo, sdrBean.getId());
			if("0".equals(resultcode) && rows > 0){
				//删除处理成功的数据
				//rows = dao.delDouyinSrcDataById(sdrBean.getId());
			}
			//logger.info("更新数据完成，影响行："+rows);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+sdrBean.getId());
		} 
	}
	
	/**
	 * 失败重试
	 * @param invokedata
	 * @param count
	 * @return
	 */
	private String errorRetry(String invokedata){
		String result = "";
		try {
			logger.info("重试开始：invokedata=>"+invokedata);
			result = URLUtil.postJson(douyinurl, invokedata, 1000 * 30);
		} catch (Exception e) {
			e.printStackTrace();
			result = "{\"resultcode\":\"8000\",\"errorinfo\":\"重试异常，"+e.getMessage()+"\"}";		
		}
		return result;
	}
}
