package com.chinacreator.kafka.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;

/**
 * 抖音数据处理类
 * @author xu.zhou
 * @date 20201208
 */
@Component
public class DouyinService extends BaseService{
	
	private static Logger logger = Logger.getLogger("douyinDeal");
	
	@Autowired
	SourceDataRecDao dao;
	
	@Value("${douyin.data.page}")
	private int page;
	
	@Value("${douyin.invoke.url}")
	private String douyinurl;
	
	@Value("${douyin.invoke.pwd}")
	private String douyinpwd;
	
	public void process(){
		try {
			SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
			String starttime = f.format(new Date());
			logger.info("开始查询数据 ，" + starttime);
			List<SourceDataRecBean> list = dao.getDouyinSrcData(page);
			logger.info("查询数据结束，开始时间=>"+starttime+"，结束时间=>"+f.format(new Date())+", 获取数据条数=> " + list.size());
			if(list != null && list.size()>0){
				//批量更新为待处理状态
				dao.updBatchDouyinSrcData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(40, 50, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (SourceDataRecBean bean : list) {
					DouyinDeal prothread = new DouyinDeal(bean, list.size(), threadSignal, logger, dao, douyinurl, douyinpwd);
					executorService.execute(prothread);
				}
				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(6000,logger);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("抖音数据处理出现异常，"+e.getMessage());
		}
	}
}
