package com.chinacreator.kafka.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.ReflowBean;
import com.chinacreator.kafka.bean.SpInfo;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.util.IdGenerateUtil;
import com.chinacreator.kafka.util.SendSmsUtil;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * @Author shuiying.ou
 * @Date 2022-06-07 14:06:10
 * @DESCRIPTION
 */

public class FlashSMSDeal implements Runnable {

    private Logger logger;

    private ReflowBean bean;

    private CountDownLatch threadSignal;

    private ActivityReflowDao activityReflowDao;

    public FlashSMSDeal(ReflowBean bean, CountDownLatch threadSignal, Logger logger, ActivityReflowDao activityReflowDao) {
        this.bean = bean;
        this.threadSignal = threadSignal;
        this.logger = logger;
        this.activityReflowDao = activityReflowDao;
    }

    @Override
    public void run() {

        String result;
        String resultinfo = null;
        String resultcode = null;
        boolean hasupd = false;
        // 发送促回流短信成功10s后，下发一条闪信
        String id = IdGenerateUtil.uuidSix();
        try {
            ReflowBean reflowBean = new ReflowBean();
            reflowBean.setId(id);
            reflowBean.setResultcode("1");
            reflowBean.setResultinfo("处理中");
            reflowBean.setActivitycode(bean.getActivitycode());
            reflowBean.setSpid(bean.getSpid());
            reflowBean.setUserid(bean.getUserid());
            //添加到发送记录表
            this.activityReflowDao.insertFlashBean(reflowBean);
            //入库正常
            hasupd = true;
            List<SpInfo> spInfos = activityReflowDao.selectSpNameBySpid(this.bean.getSpid());
            Map<String, String> map = new HashMap<>();
            SpInfo spInfo = spInfos.get(0);
            map.put("aaa", spInfo.getSpname());
            map.put("bbb", "一张5元话费券");
            // 闪信
            result = SendSmsUtil.sendFlashSMS(this.bean.getUserid(), "1033002205171804126512104", map);
            logger.info(this.bean.getUserid() + "=>" + bean.getSpid() + "二三级回流用户下发闪信结果========>" + result);
            if (!StringUtils.isBlank(result)) {
                resultinfo = result;
                JSONObject obj = JSON.parseObject(result);
                resultcode = obj.getString("resultcode");
            } else {
                resultcode = "5000";
                resultinfo = "无返回结果";
            }

        } catch (Exception exception) {
            resultcode = "8000";
            resultinfo = "发送出现异常 " + exception.getMessage();
            exception.printStackTrace();
            logger.error("发送出现异常" + "==>" + bean.getSpid() + "==>" + bean.getUserid() + exception.getMessage() + exception);
        } finally {
            //	添加到发送成功
            if (hasupd) {
                try {
                    //截断数据，防止更新失败
                    if (resultinfo.length() > 1000) {
                        resultinfo = resultinfo.substring(0, 1000);
                    } else {
                        resultinfo = "已处理";
                    }
                    //更新发送结果
                    this.activityReflowDao.updateSendFlashLog(id, resultcode, resultinfo);
                    if ("0".equals(resultcode)) {
                        this.activityReflowDao.updSms(bean.getId(),"1");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error(bean.getUserid() + "=>" + bean.getSpid() + "更新闪信发送结果出现异常" + e.getMessage() + e);
                }
            }
        }
    }
}
