package com.chinacreator.kafka.service;

import java.util.*;
import java.util.concurrent.CountDownLatch;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.util.SendSmsUtil;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.KuaishouBean;
import com.chinacreator.kafka.bean.KuaishouPushBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.KuaishouDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;

/**
 * 抖音数据处理类
 *
 * @author xu.zhou
 * @date 20201208
 */
public class KuaishouDeal implements Runnable {

    private Logger logger;

    private SourceDataRecDao dao;

    private KuaishouDao kuaishouDao;

    private int totalSize;

    private CountDownLatch threadSignal;

    private KuaishouBean bean;

    public KuaishouDeal(KuaishouBean bean, int totalSize, CountDownLatch threadSignal, Logger logger, SourceDataRecDao dao, KuaishouDao kuaishouDao) {
        this.logger = logger;
        this.dao = dao;
        this.kuaishouDao = kuaishouDao;
        this.bean = bean;
        this.totalSize = totalSize;
        this.threadSignal = threadSignal;
    }

    public void run() {
        long start = System.currentTimeMillis();
        String resultcode = "-1";
        String errorinfo = "";
        //是否推送
        boolean result = false;
        try {
            if (!"100".equals(bean.getPERCENT())) {
                throw new VideoException("8001", "非百分之百消耗，不处理");
            }

            String spid = kuaishouDao.getSpidByElementid(bean.getELEMENT_ID());
            if (spid != null && !"".equals(spid) && !"null".equals(spid)) {
                bean.setSPID(spid);
            } else {
                throw new VideoException("8002", "元素ID未匹配到SPID=>" + bean.getELEMENT_ID());
            }
            //数据入库，字段SERIAL_NUMBER, SPID, DEAL_MONTH有惟一约束，一个月一个产品一个手机号码只能入库一条数据
            try {
            	//发送流量耗尽短信，如果可能发的话
            	sendsms();
                kuaishouDao.insert(bean);
            } catch (Exception e) {
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
                    logger.error("添加数据违反唯一约束，" + bean.getID());
                    bean.setLOGINFO("当月已有相同数据，不入库TD_KAFKA_KUAISHOU。");
                } else {
                    logger.error("添加数据出现异常，" + bean.getID() + "， " + e.getMessage());
                    bean.setLOGINFO("出现异常，入库TD_KAFKA_KUAISHOU失败。");
                }
            }

            //当前月份
            String pushmonth = bean.getDEAL_MONTH(); //new SimpleDateFormat("yyyyMM").format(new Date());
            //流量数据月份的每一天第一秒
            String pushmonthFirstDay = bean.getDEAL_TIME().substring(0, 8) + "000001";
            //手机号码
            String serialNumber = bean.getSERIAL_NUMBER();
            //入库表指定月份所有的SPID，多个用#分隔
            String spids = "";

            //查询当月已收到的流量耗尽数据，可能有多个产品
            List<KuaishouBean> dataList = kuaishouDao.query(serialNumber, pushmonth);
            //logger.info(serialNumber+"----size----" + (dataList != null ? dataList.size():0) );
            if (dataList != null && dataList.size() > 0) {
                //数据按月份分组
                for (KuaishouBean bean : dataList) {
                    if ("".equals(spids)) {
                        spids += bean.getSPID();
                    } else {
                        spids += "#" + bean.getSPID();
                    }
                }
                //是否要推送
                result = hasPush(serialNumber, spids, pushmonth, pushmonthFirstDay);
            } else {
                throw new VideoException("1006", "入库表指定月份无数据");
            }

            resultcode = "0";
            errorinfo = result ? "推送" : "不推送";
        } catch (Exception e) {
            if (e instanceof VideoException) {
                resultcode = ((VideoException) e).getErrorcode();
                errorinfo = ((VideoException) e).getMessage();
            } else {
                e.printStackTrace();
                resultcode = "8000";
                errorinfo = "处理数据出现异常，" + e.getMessage();
                logger.error("处理数据出现异常，" + e.getMessage(), e);
            }
        } finally {
            if (bean.getLOGINFO() != null && !"".equals(bean.getLOGINFO())) {
                errorinfo += "， " + bean.getLOGINFO();
            }
            //更新处理结果
            updProcRes(resultcode, errorinfo, bean.getSPID());
            threadSignal.countDown();
            logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize + "，耗时：" + (System.currentTimeMillis() - start) + " ms" + ", "+bean.getSERIAL_NUMBER()+"处理结果：" + resultcode + "=>" + errorinfo + "，id：" + bean.getID() + "，是否推送：" + result);
        }
    }


    /**
     * 更新源数据处理结果
     *
     * @param resultcode
     * @param errorinfo
     */
    private void updProcRes(String resultcode, String errorinfo) {
        try {
            if (bean.getLOGINFO() != null) {
                errorinfo = bean.getLOGINFO() + errorinfo;
            }
            if (errorinfo != null && errorinfo.length() > 300) {
                errorinfo = errorinfo.substring(0, 300);
            }
            //String sql = "UPDATE TD_KAFKA_SRCDATA_KUAISHOU SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+bean.getID()+"' ";
            //更新处理状态
            //int rows = dao.updSrcDataBySql(sql);
            //logger.info("更新数据完成，影响行："+rows);

            int rows = dao.updKuaishouSrcDataRes(resultcode, errorinfo, bean.getID());
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("更新状态出现异常，" + e.getMessage() + "=>" + bean.getID());
        }
    }

    /**
     * 把spid转为List
     *
     * @param spids
     * @return
     */
    private List<String> fullSpid(String spids) {
        List<String> spidsList = new ArrayList<String>();
        if (spids != null && !"".equals(spids)) {
            String[] spidsArray = spids.split("#");
            for (String s : spidsArray) {
                spidsList.add(s);
            }
        }
        return spidsList;
    }

    /**
     * 判断是否要推送“流量已耗尽”
     * 首先获取用户当前所有有效的快手订购关系，再判断要推送 的SPID集合是否与所有订购关系相同，如果是说明用户的所有快手流量已耗尽，要推送（当月未推送过），
     * 再判断当月是否对要推送的SPID已经有过推送，如果有，不再触发推送。
     * @param serialNumber
     * @param spids
     * @param pushmonth
     * @param pushmonthFistDay 流量数据月份的第一天第一秒
     * @return
     * @throws Exception
     */
    private boolean hasPush(String serialNumber, String spids, String pushmonth, String pushmonthFistDay) throws Exception {
        boolean result = true;
        List<String> spidsList = fullSpid(spids);//Arrays.asList(spids.split("#"));使用这种方式构建的List在移除元素时会出现异常java.lang.UnsupportedOperationException

        //按月份获取有效订购数据
        //List<OrderInfo> orderList = kuaishouDao.findByUser(serialNumber,pushmonthFistDay);

        //查询用户快手有效的订购关系的SPID，剔除重复数据，SPID必须包含在TB_SP_AOP_CONFIG表内的订购信息，不要TB_SP_AOP_CONFIG表的信息不处理，因为匹配不到ELEMENT_ID
        List<OrderInfo> orderList = kuaishouDao.findspidByUser(serialNumber, pushmonthFistDay);
        //无有效订购
        if (orderList == null || orderList.size() == 0) {
            throw new VideoException("1003", "不推送，当前用户无有效订购信息");
        }

        //把订购关系的SPID放在LIST里
        List<String> orderSpidsList = new ArrayList<String>();
        for (OrderInfo orderInfo : orderList) {
            orderSpidsList.add(orderInfo.getSpid());
        }

        logger.info("serialNumber=>" + serialNumber + ", 要推送的spids:" + spidsList + "，已订购的spids:" + orderSpidsList);

        //流量消耗SPID比订购关系表的少
        if (orderList.size() > spidsList.size()) {
            throw new VideoException("1005", "不推送，当前有效订购关系spid个数大于要推送的spid个数，有流量未用完，SPIDS:" + spids);
        }


        //判断流量耗尽的SPID是否包含在订购的SPID里，如果流量消耗的spid当前无订购关系，移除
        String tmpspid = "";
        boolean nonflag = false; //是否包含，默认不包含
        for (int i = 0; i < spidsList.size(); i++) {
            nonflag = false;
            tmpspid = spidsList.get(i);
            for (String orderspid : orderSpidsList) {
                if (tmpspid.equals(orderspid)) {
                    nonflag = true;
                    break;
                }
            }
            //SPID无效，移除没在订购关系中的SPID
            if (!nonflag) {
                logger.info("serialNumber: " + serialNumber + ", SPID:" + tmpspid + ", 当前无有效订购关系");
                spidsList.remove(i);
                i--;
            }
        }

        //处理后的SPID为0，代表流量耗尽的SPID都不包含订购的SPID里
        if (spidsList.size() == 0) {
            throw new VideoException("1004", "不推送，要推送的spids当前都无有效订购关系，SPIDS:" + spids);
        }

        //处理后的流量耗尽SPID个数不等于订购关系的SPID个数
        if (spidsList.size() != orderSpidsList.size()) {
            throw new VideoException("1007", "不推送，要推送的spids与当前订购的spid不相同，spidsList:" + spidsList + "，orderSpidsList:" + orderSpidsList);
        }

        //所有SPID都包含，再判断是否当月已经有推送
        List<KuaishouPushBean> pushList = kuaishouDao.queryPush(serialNumber, pushmonth, "0");
        if (pushList != null && pushList.size() > 0) {
            List<String> tmpSpidsList = null;
            for (KuaishouPushBean bean : pushList) {
                tmpSpidsList = Arrays.asList(bean.getSpids().split("#"));
                //关联SPID个数相同
                if (tmpSpidsList.size() == spidsList.size()) {
                    boolean flag = true; //是否SPID都相同，默认相同
                    //判断所有SPID是否相同
                    for (String spid : tmpSpidsList) {
                        //有不相同的
                        if (!spidsList.contains(spid)) {
                            //SPID不一样，
                            flag = false;
                            break;
                        }
                    }
                    //如果已有相同SPID推送，则不再推送
                    if (flag) {
                        result = false;
                        break;
                    }
                }
            }
        }

        //把推送数据添加到推送记录表。
        if (result) {
            String newspids = "";
            //重组SPIDS，（有可能有删减）
            for (String tmp : spidsList) {
                if ("".equals(newspids)) {
                    newspids += tmp;
                } else {
                    newspids += "#" + tmp;
                }
            }

            logger.info("serialNumber=>" + serialNumber + ", 添加到推送表的SPIDS:" + spidsList);
            KuaishouPushBean pushBean = new KuaishouPushBean();
            pushBean.setId(kuaishouDao.getNo());
            pushBean.setPushmonth(pushmonth);
            pushBean.setPushtype("0"); //pushtype推送类型：0为流量已耗尽，1为流量未耗尽，2为月末推送
            pushBean.setResultcode("1"); //推送结果编码， 1待处理，2处理中，，0成功，其他为失败
            pushBean.setResultinfo("待处理");
            pushBean.setSerial_number(serialNumber);
            pushBean.setSpids(newspids);
            kuaishouDao.insertPush(pushBean);
        } else {
            throw new VideoException("1005", "不推送，已推送相同的spids，SPIDS:" + spidsList);
        }

        logger.info("是否要推送result=>" + result + "，serialNumber：" + serialNumber);
        return result;
    }

    /**
     * 发送短信
     */
    private void sendsms() {
        String resultcode = "";
        String resultinfo = "";
        boolean hasupd = false; //是否要更新发送结果
        String result = ""; //调接口返回信息
        String smstxt = null; //短信内容
        String smsVars = null; //短信模板参数

//        HashMap<String, String> smsconf = new HashMap<String, String>();
//        smsconf.put("1022","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手定向流量8元包中的定向流量已耗尽，您可通过https://v.mige.tv/hjts15g 继续获取快手定向流量，有效期到本月底。");
//        smsconf.put("1023","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手定向流量15元包中的定向流量已耗尽，您可通过https://v.mige.tv/kshjts8g 继续获取快手定向流量，有效期到本月底。");
//
//        smsconf.put("292","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手12G定向流量合约包中的定向流量已耗尽，您可通过http://v.mige.tv/8ctdx  继续获取快手定向流量，感谢您的使用。");
//        smsconf.put("293","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手12G定向流量合约包中的定向流量已耗尽，您可通过http://v.mige.tv/8ctdx  继续获取快手定向流量，感谢您的使用。");
//
//        smsconf.put("294","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手25G定向流量合约包中的定向流量已耗尽，您可通过http://v.mige.tv/8ctdx 继续获取快手定向流量，感谢您的使用。");
//        smsconf.put("295","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手25G定向流量合约包中的定向流量已耗尽，您可通过http://v.mige.tv/8ctdx 继续获取快手定向流量，感谢您的使用。");
//
//        smsconf.put("296","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手12G定向流量月包中的定向流量已耗尽，您可通过http://v.mige.tv/vAjMbu 继续获取快手定向流量，感谢您的使用。");
//        smsconf.put("297","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手12G定向流量月包中的定向流量已耗尽，您可通过http://v.mige.tv/vAjMbu 继续获取快手定向流量，感谢您的使用。");
//
//        smsconf.put("298","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手25G定向流量月包中的定向流量已耗尽，您可通过http://v.mige.tv/FzuQFr 继续获取快手定向流量，感谢您的使用。");
//        smsconf.put("299","【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手25G定向流量月包中的定向流量已耗尽，您可通过http://v.mige.tv/FzuQFr 继续获取快手定向流量，感谢您的使用。");

        Map<String, String> _smsVarConf = new HashMap();
        _smsVarConf.put("1022","快手定向流量8元包-https://v.mige.tv/hjts15g");
        _smsVarConf.put("1023","快手定向流量15元包-https://v.mige.tv/kshjts8g");

        _smsVarConf.put("292","快手12G定向流量合约包-http://v.mige.tv/8ctdx");
        _smsVarConf.put("293","快手12G定向流量合约包-http://v.mige.tv/8ctdx");

        _smsVarConf.put("294","快手25G定向流量合约包-http://v.mige.tv/8ctdx");
        _smsVarConf.put("295","快手25G定向流量合约包-http://v.mige.tv/8ctdx");

        _smsVarConf.put("296","快手12G定向流量月包-http://v.mige.tv/vAjMbu");
        _smsVarConf.put("297","快手12G定向流量月包-http://v.mige.tv/vAjMbu");

        _smsVarConf.put("298","快手25G定向流量月包-http://v.mige.tv/FzuQFr");
        _smsVarConf.put("299","快手25G定向流量月包-http://v.mige.tv/FzuQFr");

        //入库记录
        HashMap recmap = new HashMap();
        //发短信开始
        try {
            /***
            String ks_8 = "【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手定向流量8元包中的定向流量已耗尽，您可通过https://v.mige.tv/hjts15g 继续获取快手定向流量，有效期到本月底。";
            String ks_15 = "【快手流量包，限时优惠】 亲爱的用户您好，您订购的快手定向流量15元包中的定向流量已耗尽，您可通过https://v.mige.tv/kshjts8g 继续获取快手定向流量，有效期到本月底。";
            logger.info("快手产品发送短信开始====>" + bean.getSERIAL_NUMBER() + "  ===SPID===>" + bean.getSPID());
            if ("1022".equals(bean.getSPID())) {
                result = SendSmsUtil.send(bean.getSERIAL_NUMBER(), ks_8);
            } else if ("1023".equals(bean.getSPID())) {
                result = SendSmsUtil.send(bean.getSERIAL_NUMBER(), ks_15);
            }
            **/
            //获取短信内容
//            smstxt = smsconf.get(bean.getSPID());
            smsVars = _smsVarConf.get(bean.getSPID());//获取短信模板参数
//            if(!StringUtils.isEmpty(smstxt) && bean.getSERIAL_NUMBER() != null && bean.getSERIAL_NUMBER().length() == 11){//有短信内容匹配
            if(!StringUtils.isEmpty(smsVars) && bean.getSERIAL_NUMBER() != null && bean.getSERIAL_NUMBER().length() == 11){//有短信内容匹配
            	recmap.put("id", kuaishouDao.getNo());
                recmap.put("userid", bean.getSERIAL_NUMBER());
                recmap.put("resultcode", "1");
                recmap.put("resultinfo", "处理中");
                recmap.put("spid", bean.getSPID());
                //查询当月是否已有推送记录
                int reccount = kuaishouDao.qrySmsRec(recmap);
                if(reccount == 0){//没有推送短信
                    //添加到发送记录表，同一号码同一产品同一月份只能发送一次，表有唯一约束
                    kuaishouDao.sendSmsRec(recmap);
                    //入库正常
                    hasupd = true;
                    //发送短信
//                    result = SendSmsUtil.send(bean.getSERIAL_NUMBER(), smstxt);//旧
                    //处理参数map集合中的数据
                    String[] vars = smsVars.split("-");
                    HashMap<String, Object> map = new HashMap<>();
                    map.put("businame",vars[0]);
                    map.put("url",vars[1]);
//                    result = SendSmsUtil.sendByTemplate(bean.getSERIAL_NUMBER(),"202206090912014411",map);
                    String smsid = "2022060614432"; //快手发送短信渠道
                    String pwd = "#?csgj06";
                    String contentid = "202206090912014411";
                    // 优化
                    result = SendSmsUtil.sendSMSWithVars(bean.getSERIAL_NUMBER(),contentid,smsid,pwd,map);
                    logger.info("快手产品发送短信结果====>" + result);
                    if (result != "" && result != null) {
                    	resultinfo = result;
                        JSONObject obj = JSON.parseObject(result);
                        resultcode = obj.getString("resultcode");
                    }else{
                    	resultcode = "5000";
                    	resultinfo = "无返回结果";
                    }
                }else{
                	logger.info(bean.getID()+"=>"+bean.getSERIAL_NUMBER()+"=>"+bean.getSPID()+"当月已推送短信");
                }
            }
        } catch (Exception e) {
            //e.printStackTrace();
            resultcode = "8000";
            resultinfo = "发送出现异常，"+e.getMessage();
            logger.error(bean.getID()+"=>"+bean.getSERIAL_NUMBER()+"=>"+bean.getSPID()+"发送出现异常，"+e.getMessage(),e);
        } finally {
        	//添加到发送成功
        	if(hasupd){
        		try {
	        		//截断数据，防止更新失败
	        		if(resultinfo != null && resultinfo.length()>1000){
	        			resultinfo = resultinfo.substring(0,1000);
	        		}
	        		recmap.put("resultcode", resultcode);
	        		recmap.put("resultinfo", resultinfo);
                	//更新发送结果
                    kuaishouDao.updSmsRec(recmap);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error(bean.getID()+"=>"+bean.getSERIAL_NUMBER()+"=>"+bean.getSPID()+"更新短信发送结果出现异常，"+e.getMessage(),e);
                }
        	}
        }
    }

    /**
     * 更新源数据处理结果
     * @param resultcode
     * @param errorinfo
     * @param spid
     * @param
     */
    private void updProcRes(String resultcode, String errorinfo, String spid) {
        try {
            if (bean.getLOGINFO() != null) {
                errorinfo = bean.getLOGINFO() + errorinfo;
            }
            if (errorinfo != null && errorinfo.length() > 300) {
                errorinfo = errorinfo.substring(0, 300);
            }
            //String sql = "UPDATE TD_KAFKA_SRCDATA_KUAISHOU SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+bean.getID()+"' ";
            //更新处理状态
            //int rows = dao.updSrcDataBySql(sql);
            //logger.info("更新数据完成，影响行："+rows);

            int rows = dao.updKuaishouSrcDataResNew(resultcode, errorinfo, bean.getID(), spid);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("更新状态出现异常，" + e.getMessage() + "=>" + bean.getID());
        }
    }
}
