package com.chinacreator.kafka.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.QueenCardBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.QueenCardDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;

/**
 * 橙卡流量数据扣减处理类
 * @author xu.zhou
 * @date 20201208
 */
public class QueenCardDeal implements Runnable{
	
	private Logger logger;
	
	private SourceDataRecDao dao;
	
	private QueenCardDao qcDao;
	
	private int totalSize;

	private CountDownLatch threadSignal;
	
	private SourceDataRecBean sdrBean;
	
	//#女王卡产品编码，多个用|分隔
	private String qcproductid;
	
	public QueenCardDeal(SourceDataRecBean sdrBean, int totalSize, CountDownLatch threadSignal,Logger logger, SourceDataRecDao dao, String qcproductid, QueenCardDao qcDao) {
		this.logger = logger;
		this.dao = dao;
		this.qcDao = qcDao;
		this.sdrBean = sdrBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.qcproductid = qcproductid;
	}
	
	public void run(){
		long start = System.currentTimeMillis();
		String resultcode = "";
		String errorinfo = "";
		try {
			QueenCardBean qcBean = null;
			String params = sdrBean.getSourcedata();
			//System.out.println(qcproductid);
			//System.out.println(params);
			//判断 是否女王卡，是，则入到对应的库表
			if(!StringUtils.isEmpty(qcproductid)){
			//if(qcproductid != null && !"".equals(qcproductid) && !"null".equals(qcproductid)){
				String [] qcids = qcproductid.split("\\|");
				for(String qcid: qcids){
					qcBean = null;
					if(params.contains("\"productId\":\""+qcid+"\"") 
							|| params.contains("\"NEW_PRODUCT_ID\":\""+qcid+"\"")
							|| params.contains("\"OLD_PRODUCT_ID\":\""+qcid+"\"")
							//|| resstr.contains("\"PRODUCT_ID\":\""+qcid+"\"")
							){
						qcBean =  getQueenCard(qcid);
						if("0".equals(qcBean.getStatus())){
							String valires = valiData(qcBean);
							if("ok".equals(valires)){
						        try {
									int rows = qcDao.order(qcBean);
									qcBean.setErrorinfo("订购成功");
									qcBean.setErrorcode("0");
								} catch (Exception e) {
									logger.error(qcBean.getUserid()+","+qcBean.getSerialNumber()+"=>订购出现异常，"+e.getMessage());
					    			qcBean.setErrorinfo("订购失败，"+e.getMessage());
					    			qcBean.setErrorcode("8000");
								}
							}else{
								qcBean.setErrorinfo(valires);
								qcBean.setErrorcode("8001");
							}
						}else if("1".equals(qcBean.getStatus())){
							String valires = valiData(qcBean);
							if("ok".equals(valires)){
					    		try {
					    			int rows = qcDao.cancel(qcBean);
					    			qcBean.setErrorinfo("退订成功");
					    			qcBean.setErrorcode("0");
								} catch (Exception e) {
									logger.error(qcBean.getUserid()+","+qcBean.getSerialNumber()+"=>退订出现异常，"+e.getMessage());
					    			qcBean.setErrorinfo("退订失败，"+e.getMessage());
					    			qcBean.setErrorcode("8000");
								}
					    	}else{
					    		qcBean.setErrorinfo(valires);
					    		qcBean.setErrorcode("8002");
							}
						}else{
							qcBean.setErrorinfo("未定义类型");
							qcBean.setErrorcode("0");
						}
						//添加到处理日志表
						try{
							qcDao.insertLogData(qcBean);
						}catch(Exception e){
							logger.error("写日志异常，"+e.getMessage()+"=>"+sdrBean.getId(),e);
						}
						break;
					}
				}
				resultcode = (qcBean == null ? "未匹配到产品信息" : qcBean.getErrorcode());
				errorinfo = (qcBean == null ? "未获取到女王卡产品编码配置信息" : qcBean.getErrorinfo());
			}else{
				resultcode = "8003";
				errorinfo = "未获取到女王卡产品编码配置信息";
			}
		} catch (Exception e) {
			//logger.error("推送数据出错：文件名=》"+filename+"，异常信息=》"+e.getMessage()+", 推送数据："+data); 
			e.printStackTrace();	
			logger.error("推送异常，"+e.getMessage()+"=>"+sdrBean.getId(),e);
			resultcode = "8000";
			errorinfo = "出现异常=>"+e.getMessage();
		} finally {
			updProcRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - start)+" ms"+", 推送结果："+errorinfo+" , "+resultcode+"，id："+sdrBean.getId());
		}
	}
	
	/**
	 * 获取女王卡Bean
	 * @param filename
	 * @param str
	 * @param productid
	 * @return
	 */
	private QueenCardBean getQueenCard(String productid){
		QueenCardBean bean = new QueenCardBean();
		bean.setProductId(productid);
		//String resstr = str.substring(str.indexOf("{"), str.indexOf("}") + 1);
		String logname = sdrBean.getOldlogname();
		String jsondata = sdrBean.getSourcedata();
		if (logname.contains("bg")) {
			String topic = "NewProdECApi";
			BGbean bgBean = (BGbean) JsonUtil.jsonToBean(jsondata, BGbean.class);
			if(bgBean != null){
				if(productid.equals(bgBean.getNEW_PRODUCT_ID())){ //变更转入（订购）
					bean = setQueenCardBean(bean,bgBean,topic,"3");
				}else{ //变更转出（退订）
					bean = setQueenCardBean(bean,bgBean,topic,"4");
				}			
			}
		}else if(logname.contains("msglog")){
			bean = (QueenCardBean) JsonUtil.jsonToBean(jsondata, QueenCardBean.class);
			//0：预开户, 1：撤销, 2：拆机, 3下月一号订购，订购时间以operttime为准
			if("0".equals(bean.getActioncode()) || "3".equals(bean.getActioncode())){
				bean.setStatus("0"); //订购
				bean.setOrdertime(bean.getOperaTime());
			}else if("1".equals(bean.getActioncode()) || "2".equals(bean.getActioncode())){
				bean.setStatus("1"); //退订
				bean.setCanceltime(bean.getOperaTime());
			}else{
				//continue; //其他不处理
				//其他数据也在入库
			}
			
			bean.setTopic("NewProdInfo");
		}

		bean.setGettime(sdrBean.getInserttime());
		try {
			bean.setPartition(Integer.parseInt(sdrBean.getPartition()));
			bean.setOffset(Long.parseLong(sdrBean.getOffset()));
		} catch (NumberFormatException e) {

		}
		
		bean.setFilename(sdrBean.getId());
		
		return bean;
	}
	
	
	/**
	 * 
	 * @param bean
	 * @param bgBean
	 * @param topic
	 * @param actionCode  订购关系：0：预开户（订购），1：撤销（退订），2：拆机（退订），3：转入变更（订购），4：转出变更（退订）
	 * @return
	 */
	private QueenCardBean setQueenCardBean(QueenCardBean bean, BGbean bgBean,String topic,String actionCode){
		bean.setTopic(topic);
		bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
		bean.setActionCode(actionCode);
		bean.setUserid(bgBean.getUSER_ID());
		bean.setOperaTime(bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":",""));
		bean.setNewproductid(bgBean.getNEW_PRODUCT_ID());
		bean.setOldproductid(bgBean.getOLD_PRODUCT_ID());
		String oldstartdate = bgBean.getOLD_START_DATE();
		String oldenddate = bgBean.getOLD_END_DATE();
		bean.setOldenddate(parseOldDate(oldenddate));
		bean.setOldstartdate(parseOldDate(oldstartdate));
		bean.setKafkatype(bgBean.getKAFKA_TYPE());
		if("3".equals(actionCode)){ //订购
			bean.setOrdertime(bean.getOperaTime());
			bean.setStatus("0");
		}else{//退订
			bean.setCanceltime(bean.getOperaTime());
			bean.setStatus("1");
		}

		return bean;
	}	
	
	/**
	 * 验证是否可处理订购关系
	 * @param bean
	 * @return
	 */
	public String valiData(QueenCardBean bean){
		String result = "ok";
		try {
			//String sql = " SELECT ID,SERIALNUMBER, TO_CHAR(ORDERTIME, 'yyyymmddhh24miss') ORDERTIME, TO_CHAR(CANCELTIME,  'yyyymmddhh24miss') CANCELTIME, STATUS, VIPMON FROM TD_KAFKA_QUEENCARD_ORDER WHERE SERIALNUMBER = ? AND PRODUCTID = ?";
			List<HashMap> list = qcDao.queryReal(bean.getSerialNumber(),bean.getProductId()); //SQLExecutor.queryListWithDBName(HashMap.class, "net3g", sql, new Object[] {bean.getSerialNumber(),bean.getProductId()});
			if(list != null && list.size()>0){
				HashMap hm = list.get(0);
				//ACTIONCODE IS '订购关系：0：预开户（订购），1：撤销（退订），2：拆机（退订），3：转入变更（订购），4：转出变更（退订）
				if("0".equals(bean.getActioncode()) || "3".equals(bean.getActioncode())){//订购
					if("0".equals(hm.get("STATUS"))){ //当前是订购状态
						if(Long.parseLong(bean.getOperaTime()) <= Long.parseLong(hm.get("ORDERTIME")+"")){
							result = "订购时间小于或等于当前的订购时间";
						}
					}else{//当前是退订状态
						if(Long.parseLong(bean.getOperaTime()) <= Long.parseLong(hm.get("CANCELTIME")+"")){
							result = "订购时间小于或等于当前的退订时间";
						}
					}
				}else{//退订
					if("0".equals(hm.get("STATUS"))){ //当前是订购状态
						if(Long.parseLong(bean.getOperaTime()) <= Long.parseLong(hm.get("ORDERTIME")+"")){
							result = "退订时间小于或等于当前的订购时间";
						}
					}else{//当前是退订状态
						if(Long.parseLong(bean.getOperaTime()) <= Long.parseLong(hm.get("CANCELTIME")+"")){
							result = "退订时间小于或等于当前的退订时间";
						}
					}
				}
				if("ok".equals(result)){
					this.getVipMon(bean, hm);
				}
			}else{//当前无订购数据
				if("1".equals(bean.getActioncode()) || "2".equals(bean.getActioncode()) || "4".equals(bean.getActioncode())){//订购
					result = "当前无订购关系，退订操作不处理";
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("userid=>"+bean.getUserid()+","+bean.getSerialNumber()+"=>验证是否可处理订购关系出现异常，"+e.getMessage());
			//System.out.println(String.valueOf(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date())) + ", "+"userid=>"+bean.getUserid()+","+bean.getSerialNumber()+"=>验证是否可处理订购关系出现异常，"+e.getMessage());
			result = e.getMessage();
		}
		//System.out.println(String.valueOf(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date())) + ", "+"验证结果=>"+", userid=>"+bean.getUserid()+", serialnumber=>"+bean.getSerialNumber()+"，result=>"+result);
		logger.info("验证结果=>"+", userid=>"+bean.getUserid()+", serialnumber=>"+bean.getSerialNumber()+"，result=>"+result);
		return result;
	}
	
	/**
	 * 设置会员已经可以领取的月份（只有退订的时候要设置vipmon）
	 * @param inBean //要校报增的订购关系
	 * @param currBean //当前已经有订购关系
	 * @throws Exception
	 */
	public void getVipMon(QueenCardBean inBean, HashMap hm) throws Exception{
		try {
			if(hm == null){ //当前没有订购关系，不处理
				inBean.setVipmon("");
				return;
			}

			//获取已有月份值
			String vipmon = (String)hm.get("VIPMON");
			if(vipmon == null || "null".equals(vipmon)) vipmon = "";
			if(vipmon.split(",").length > 24){
				//inBean.setVipmon(vipmon);
				//return; //最多保存24个月份值
			}

			if("1".equals(inBean.getStatus())){ //退订，获取从订购时间起到现在的可以领取会员的月份值
				String canceltime = inBean.getCanceltime().substring(0, 6);
				String ordertime = ((String)hm.get("ORDERTIME")).substring(0, 6);
				//退订月份大于订购月份
				if(Integer.parseInt(canceltime)>Integer.parseInt(ordertime)){
					 int months = this.differMonth(canceltime, ordertime);
					 String tmp = "";
					 for(int i = 1; i <= months; i++){
						  tmp += ","+this.addMonth(ordertime, i);
					 }
					 if(!"".equals(tmp)){
						 if("".equals(vipmon)){
							 vipmon = tmp.substring(1);
						 }else{
							 vipmon += tmp;
						 }
					 }
				}
			}else{//订购
				if("0".equals(hm.get("STATUS"))){//当前是订购状态，又来条订购的数据
					//订购关系表的订购时间
					String ordertime = ((String)hm.get("ORDERTIME")).substring(0, 6);
					SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
					//新输入的订购时间
					String ordertime2 = inBean.getOrdertime().substring(0,6);
					int months = this.differMonth(ordertime2, ordertime);
					String tmp = "";
					for(int i = 1; i <= months; i++){
						 tmp += ","+this.addMonth(ordertime, i);
					}
					if(!"".equals(tmp)){
						if("".equals(vipmon)){
							 vipmon = tmp.substring(1);
						}else{
							 vipmon += tmp;
						}
					}
				}
			}
			//排序并去重
			vipmon = this.sortVipmon(vipmon);
			if(vipmon != null && vipmon.length()>167){//超过24个月份
				vipmon = vipmon.substring(0,167);//只保存24个月份
			}
			inBean.setVipmon(vipmon);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("设置会员已经可以领取的月份出现异常，"+e.getMessage()+", userid=>"+inBean.getUserid()+", serialnumber=>"+inBean.getSerialNumber());
			throw new Exception("设置会员已经可以领取的月份出现异常，"+e.getMessage());
		}
	}
	
    /**
     * 获取两个日期相差月份
     * @param canceltime
     * @param ordertime
     * @throws ParseException
     */
	private int differMonth(String canceltime, String ordertime) throws ParseException{
		int months = 0;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
        Calendar bef = Calendar.getInstance();
        Calendar aft = Calendar.getInstance();
        bef.setTime(sdf.parse(ordertime));
        aft.setTime(sdf.parse(canceltime));
        int month = aft.get(Calendar.MONTH) - bef.get(Calendar.MONTH);
        int year = (aft.get(Calendar.YEAR) - bef.get(Calendar.YEAR)) * 12;

        months = year + month;
        logger.info("相差月份：months: " + months + ", year: " + year+", month: "+month);
        return months;
	}

	/**
	 * 获取添加月份后的月份值
	 * @param ordermonth 订购月份
	 * @param amount 要增加的月份值
	 * @return
	 * @throws ParseException
	 */
	private String addMonth(String ordermonth , int amount) throws ParseException{
		String res = "";
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
		Calendar ordertime = Calendar.getInstance();
		ordertime.setTime(sdf.parse(ordermonth));
		ordertime.add(Calendar.MONTH, amount);
		res =  sdf.format(ordertime.getTime());
		logger.info("添加月份后的月份值: "+res+", ordermonth: "+ordermonth+", amount: "+amount);
		return res;
	}
	
	/**
	 * 去重及排序
	 * @param vipmon
	 * @return
	 */
	private String sortVipmon(String vipmon){
		String res = "";
		if(vipmon == null || "".equals(vipmon)) return res;
		HashMap<String,String> hm = new HashMap<String,String>();
		String [] vipmonArray = vipmon.split(",");
		List<Integer> list = new ArrayList<Integer>();
		for(String s : vipmonArray){
			if(!hm.containsKey(s)){
				hm.put(s,"");
				list.add(Integer.parseInt(s));
			}
		}

		Collections.sort(list);
		res = list.toString().replace("[", "").replace("]", "").replace(" ", "");

		return res;
	}
	
	
	/**
	 * 解析日期格式
	 * @param dateStr 2019-12-31:23:59:59
	 * @return 20191231235959
	 */
	private String parseOldDate(String dateStr){
		String olddate = "";
		try {
			String ymd = dateStr.substring(0, 10).replace("-", "");
			String hms = dateStr.substring(10).replace(":", "");
			olddate = ymd+hms;
		} catch (Exception e) {
			e.printStackTrace();
			//System.out.println(String.valueOf(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date())) + ", "+"解析olddate出现异常");
		}
		//System.out.println();
		return olddate;
	}
	
	/**
	 * 更新源数据处理结果
	 * @param resultcode
	 * @param errorinfo
	 */
	private void updProcRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}
			//String sql = "UPDATE TD_KAFKA_SRCDATA_QUEENCARD SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+sdrBean.getId()+"' ";
			//更新处理状态
			//int rows = dao.updSrcDataBySql(sql);
			int rows = dao.updQueencardSrcDataRes(resultcode, errorinfo, sdrBean.getId());
			//logger.info("更新数据完成，影响行："+rows);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+sdrBean.getId());
		} 
	}
	

}
