package com.chinacreator.kafka.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.*;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.util.IdGenerateUtil;
import com.chinacreator.kafka.util.SendSmsUtil;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.jboss.logging.Logger;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;

/**
 * @Author shuiying.ou
 * @Date 2022-03-22 11:47
 * @DESCRIPTION
 */

public class ReflowNoteDeal implements Runnable {

    private Logger logger;

    private CheckData checkData;

    private CountDownLatch threadSignal;

    private ActivityReflowDao activityReflowDao;

    private KafkaInfoDao kafkaInfoDao;


    public ReflowNoteDeal(CheckData checkData, CountDownLatch threadSignal, Logger logger, ActivityReflowDao activityReflowDao, KafkaInfoDao kafkaInfoDao) {
        this.checkData = checkData;
        this.threadSignal = threadSignal;
        this.logger = logger;
        this.activityReflowDao = activityReflowDao;
        this.kafkaInfoDao = kafkaInfoDao;
    }

    @Override
    public void run() {
    	//logger.info("开始处理...");
    	HashMap<String, Object> logmap = new HashMap<String, Object>();
    	String resultcode = "";
    	String resultinfo = "";
        try {
        	logmap.put("SequenceId", checkData.getSequenceId());
        	logmap.put("ServiceId", checkData.getServiceId());
        	logmap.put("ProductId", checkData.getProductId());
        	
            // 将数据状态改为处理中
            checkData.setResultcode("2");
            checkData.setResultinfo("处理中");
            activityReflowDao.updateData(checkData.getResultcode(),checkData.getResultinfo(),checkData.getSequenceId());
            //根据productid查spid
            List<SpInfo> spInfos = activityReflowDao.selectSpInfo(checkData.getProductId());
            if(spInfos != null && spInfos.size() > 0){
	            for (SpInfo spInfo : spInfos) {
	                String spid = spInfo.getSpid();
	                if (!StringUtils.isEmpty(spid)) {
	                    Date date = activityReflowDao.selectEndTime(spid);
	                    if(date == null) break;
	                    long endTime = date.getTime();
	                    long orderTime = checkData.getInserttime().getTime();
	                    if (orderTime < endTime) {
	                        List<String> huaweiOrderids = activityReflowDao.findAsynch(checkData.getServiceId(), spid);
	                        if (huaweiOrderids != null && huaweiOrderids.size() > 0){
	                            logger.info(checkData.getServiceId() + "=>" + spid + "=>" + "属于华为订购异步订购的数据");
	                            resultinfo = checkData.getServiceId() + "=>" + spid + "=>" + "属于华为订购异步订购的数据";
	                        }else {
	                            // 查订购关系
	                            OrderInfo orderInfo = activityReflowDao.findByUserAndSpid(checkData.getServiceId(), spid);
	                            if (orderInfo != null) {
	                                logger.info(checkData.getServiceId() + "=>" + spid + "=>" + "已订购该产品");
	                                resultinfo = checkData.getServiceId() + "=>" + spid + "=>" + "已订购该产品";
	                            } else {
	                                // 查近五分钟订购日志表是否有订购关系
	                                List<OrderLog> orderLogList = activityReflowDao.selectOrderLog(checkData.getServiceId(), spid);
	                                if (orderLogList.size() == 0) {
	                                    //订购表中没有订购关系，订购日志中近五分钟也没有订购记录
	                                    // 判断是否是合约期内退订的用户
	                                    OrderInfo orderInfoCancel = activityReflowDao.findOrderInfo(checkData.getServiceId(), spid);
	                                    if (!StringUtils.isEmpty(orderInfoCancel)) {
	                                        // 有退订记录
	                                        // 是合约包产品并且在合约期间退订，不发回流短信    3 合约产品
	                                        if ("3".equals(spInfo.getDirectype())) {
	                                            String canceltime = orderInfoCancel.getCanceltime();
	                                            String ordertime = orderInfoCancel.getOrdertime();
	
	                                            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
	                                            String currmonth = sdf.format(new Date());
	
	                                            Integer month = activityReflowDao.getMonth(spid);
	                                            if (!StringUtils.isEmpty(month)) {
	                                                // 退订月份在订购日期后的5月内，且当前月与订购月份在6个月内(一般合约期都为5个月)
	                                                if (getDifMonth(canceltime, ordertime) < month && getDifMonth(currmonth, ordertime) < (month + 1)) {//退订月份在订购日期后的5月内，且当前月与订购月份在6个月内
	                                                    logger.info(checkData.getServiceId() + "=>" + spid + "=>" + "该用户在合约期内退订，无需发送回流短信");
	                                                    resultinfo = checkData.getServiceId() + "=>" + spid + "=>" + "该用户在合约期内退订，无需发送回流短信";
	                                                } else {
	                                                    // 已过合约时间，发
	                                                    sendsms(spid, checkData, logmap);
	                                                }
	                                            } else {
	                                                // 没有启动合约期限，发
	                                                sendsms(spid, checkData, logmap);
	                                            }
	                                        } else {
	                                            // 不是合约产品，发
	                                            sendsms(spid, checkData, logmap);
	                                        }
	                                    } else {
	                                        // 无退订记录，发
	                                        sendsms(spid, checkData, logmap);
	                                    }
	                                } else {
	                                    logger.info(checkData.getServiceId() + "=>" + spid + "=>" + "订购日志表中有订购记录");
	                                    resultinfo = checkData.getServiceId() + "=>" + spid + "=>" + "订购日志表中有订购记录";
	                                }
	                            }
	                        }
	                    } else {
	                        logger.info("此次操作不在二级回流活动期" + checkData.getServiceId() + "==> 活动结束时间：" + date);
	                        resultinfo = "此次操作不在二级回流活动期" + checkData.getServiceId() + "==> 活动结束时间：" + date;
	                    }
	                }
	            }
            }else{
            	resultinfo = "无SPINFO信息";
            }
            
            checkData.setResultcode("0");
            checkData.setResultinfo("已处理");
        } catch (Exception e) {
            logger.error(checkData.getSequenceId() + "  == >操作失败" + e.getMessage(), e);
            resultinfo = checkData.getSequenceId() + "  == >操作失败" + e.getMessage();
            checkData.setResultcode("8000");
            checkData.setResultinfo("处理出现异常，"+e.getMessage());
        } finally {
        	try {
        		if(StringUtils.isEmpty(checkData.getResultinfo())){
        			checkData.setResultinfo("无信息");
        		}
                activityReflowDao.updateData(checkData.getResultcode(),checkData.getResultinfo(),checkData.getSequenceId());
                resultinfo += "，更新状态完成";
			} catch (Exception e2) {
				e2.printStackTrace();
				resultinfo += "，更新状态出现异常，"+e2.getMessage();
			}
        	logmap.put("resultinfo", resultinfo);
            threadSignal.countDown();
            logger.info(logmap);
        }
    }

    /**
     * 获取月份差值
     *
     * @param canceltime 格式YYYYMMDDHH24MISS
     * @param ordertime  格式YYYYMMDDHH24MISS
     * @return 月份差值，比如canceltime是202108XXXXXX，ordertime202103XXXXXX，返回5
     * 退订日期在订购日期的5月内，返回小于5，比如2月份订,6月份退，返回4，7月退返回5
     * @throws Exception
     */
    private int getDifMonth(String canceltime, String ordertime) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(sdf.parse(ordertime.substring(0, 6)));
        end.setTime(sdf.parse(canceltime.substring(0, 6)));
        int result = end.get(Calendar.MONTH) - start.get(Calendar.MONTH);
        int month = (end.get(Calendar.YEAR) - start.get(Calendar.YEAR)) * 12;
        return Math.abs(month + result);
    }

    /**
     * 发送短信
     */
    private void sendsms(String spid, CheckData checkData, HashMap<String, Object> logmap) {
        String resultcode = "";
        String resultinfo = "";
        boolean hasupd = false; //是否要更新发送结果
        String result = ""; //调接口返回信息
        ReflowBean reflowBean = new ReflowBean();
        ActivityReflowDao activityReflowDao = SpringContextUtils.getBean(ActivityReflowDao.class);
        String activitycode = "reflowActivity";
        String id = null;

        //发短信开始
        try {
            // 查询是否发送过券码，已发送过的不发
            int res = activityReflowDao.selectCouponLog(checkData.getServiceId(), spid,activitycode);
            if (res < 1) {
                reflowBean.setSpid(spid);
                reflowBean.setUserid(checkData.getServiceId());
                //查询当天是否已有推送记录
                ReflowBean reflow = activityReflowDao.qrySmsRec(spid, checkData.getServiceId());
                if (StringUtils.isEmpty(reflow)) {//没有推送短信
                    id = IdGenerateUtil.uuid6();
                    reflowBean.setId(id);
                    reflowBean.setResultcode("1");
                    reflowBean.setResultinfo("处理中");
                    reflowBean.setActivitycode("reflowActivity");
                    //添加到发送记录表，同一号码同一产品同一天只能发送一次，表有唯一约束
                    activityReflowDao.insertReflowBean(reflowBean);
                    //入库正常
                    hasupd = true;
                    HashMap<String, Object> map = new HashMap<>();
                    //短信模板内容
                    String smstxt = activityReflowDao.findSmstxt(spid);
                    String[] SMSDetails = smstxt.split("\\|");
                    map.put("url",SMSDetails[0]);
                    map.put("url-p1",SMSDetails[1]);
                    map.put("url-p2",SMSDetails[2]);
                    map.put("spname",SMSDetails[3]);
                    map.put("url-p3",SMSDetails[4]);
                    map.put("url-p4",SMSDetails[5]);
                    //发送短信
                    String smsid = kafkaInfoDao.getDictionaryValue("reflowActSmsid");
                    String pwd = kafkaInfoDao.getDictionaryValue("reflowActPwd");
                    String contentid = "202204151711164420";
                    result = SendSmsUtil.sendSMSWithVars(checkData.getServiceId(), contentid, smsid, pwd, map);
//                        result = SendSmsUtil.send(checkData.getServiceId(), smstxt);
                    logger.info(checkData.getServiceId() + "=>" + spid + "回流用户挽留发送短信结果====>" + result);
                    logmap.put("sendsmsres", result);
                    if (result != "" && result != null) {
                        resultinfo = result;
                        JSONObject obj = JSON.parseObject(result);
                        resultcode = obj.getString("resultcode");
                    } else {
                        resultcode = "5000";
                        resultinfo = "无返回结果";
                    }
                } else {
                    logger.info(reflow.getId() + "=>" + reflow.getUserid() + "=>" + reflow.getSpid() + "当天已推送短信");
                    logmap.put("sendsmsres", reflow.getId() + "=>" + reflow.getUserid() + "=>" + reflow.getSpid() + "当天已推送短信");
                }

            } else {
                logger.info(checkData.getServiceId() + "=>" + spid + "已经发送过券码");
                logmap.put("sendsmsres",checkData.getServiceId() + "=>" + spid + "已经发送过券码");
            }
        } catch (Exception e) {
            resultcode = "8000";
            resultinfo = "发送出现异常，" + e.getMessage();
            logger.error(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "发送出现异常，" + e.getMessage(), e);
            logmap.put("sendsmsres",reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "发送出现异常，" + e.getMessage());
        } finally {
            //添加到发送成功
            if (hasupd) {
                try {
                    //截断数据，防止更新失败
                    if (resultinfo.length() > 1000) {
                        resultinfo = resultinfo.substring(0, 1000);
                    } else {
                        resultinfo = "已处理";
                    }
                    reflowBean.setResultcode(resultcode);
                    reflowBean.setResultinfo(resultinfo);
                    //更新发送结果
                    activityReflowDao.updSmsRec(id, resultcode, resultinfo);
                    logmap.put("updSmsRec","成功");
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.error(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "更新短信发送结果出现异常，" + e.getMessage(), e);
                    logmap.put("updSmsRec", reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "更新短信发送结果出现异常，" + e.getMessage());
                }
            }
        }
    }
}
