package com.chinacreator.kafka.service;

import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.commons.lang.time.DateUtils;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.ConfigBean;
import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.bean.ProvinceBean;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.URLUtil;

/**
 * 订购关系同步
 * @author xu.zhou
 * @date 20201208
 */
public class SyncMainDeal implements Runnable{
	
	private Logger logger;
	
	private KafkaInfoDao kidao;
	
	private int totalSize;
	
	private CountDownLatch threadSignal;
	
	private KafkaBean kafkaBean;
	
	//产品ID配置信息
	HashMap<String, String> productmap;
	
	//订购关系同步成功后要发送短信的产品编码，多个用|分隔
	private String sendsmsspid;
	
	public SyncMainDeal(KafkaBean kafkaBean, int totalSize, CountDownLatch threadSignal,Logger logger, 
			KafkaInfoDao kidao, HashMap<String, String> productmap, String sendsmsspid) {
		this.logger = logger;
		this.kidao = kidao;
		this.kafkaBean = kafkaBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.productmap = productmap;
		this.sendsmsspid = sendsmsspid;
	}

	public void run() {
		//把要发短信的SPID转成List
		List<String> smssendList = new ArrayList<String>();
		if(!StringUtils.isEmpty(sendsmsspid)){
			smssendList = Arrays.asList(sendsmsspid.split("\\|"));
		}
		String resultcode = "-1";
		String errorinfo = "";
		long start = System.currentTimeMillis();
		OrderInfo info = new OrderInfo();
		try {
			if (StringUtils.isEmpty(kafkaBean.getSerialNumber()) || kafkaBean.getSerialNumber().length() != 11) {
				throw new VideoException("1000", "手机号码参数错误");
			}
			
			// 北京号码段订购限制
			String numberlimit = kidao.getDictionaryValue("Numberlimit");
			if(numberlimit != null && numberlimit.indexOf(kafkaBean.getSerialNumber().substring(0, 7)) != -1){
			   throw new VideoException("9010", "用户为黑名单用户");
			}
			
			if (StringUtils.isEmpty(kafkaBean.getActionCode())
					|| (kafkaBean.getTopic().equals("NewProdInfo") && (!kafkaBean.getActionCode().equals("0")
							&& !kafkaBean.getActionCode().equals("1") && !kafkaBean.getActionCode().equals("2")&& !kafkaBean.getActionCode().equals("3")))
					|| (kafkaBean.getTopic().equals("ProdOrderInfo") && (!kafkaBean.getActionCode().equals("0")
							&& !kafkaBean.getActionCode().equals("1") && !kafkaBean.getActionCode().equals("2")&& !kafkaBean.getActionCode().equals("3")))
					|| (kafkaBean.getTopic().equals("TencentAppFree") && (!kafkaBean.getActionCode().equals("0")
							&& !kafkaBean.getActionCode().equals("1") && !kafkaBean.getActionCode().equals("2")
							&& !kafkaBean.getActionCode().equals("3") && !kafkaBean.getActionCode().equals("4")))
					|| (kafkaBean.getTopic().equals("AsynTfBhTrade") && (!kafkaBean.getActionCode().equals("0") && !kafkaBean.getActionCode().equals("1")))
					|| (kafkaBean.getTopic().equals("Syngnathus") && !kafkaBean.getActionCode().equals("0") && !kafkaBean.getActionCode().equals("1")) //银铃卡，20210909 xu.zhou
					) {
				throw new VideoException("1000", "操作类型参数错误");
			}
			if (StringUtils.isEmpty(kafkaBean.getOperaTime()) || kafkaBean.getOperaTime().length() != 14) {
				throw new VideoException("1000", "操作时间参数错误");
			}
			if (StringUtils.isEmpty(kafkaBean.getProductId())) {
				throw new VideoException("1000", "产品参数错误");
			}
			if(StringUtils.isEmpty(productmap.get(kafkaBean.getProductId()))){
					throw new VideoException("1000", "产品参数错误,id不存在");
			}
			//获取产品配置
			ConfigBean configBean = (ConfigBean) JsonUtil.jsonToBean(productmap.get(kafkaBean.getProductId()), ConfigBean.class);
			info.setCpid(configBean.getCpid());
			info.setSpid(configBean.getSpid());
			info.setMutex(configBean.getMutex());
			info.setUserid(kafkaBean.getSerialNumber());
			info.setId(kafkaBean.getId());
			
			//updatetype 操作标识：0订购、 1退订
			String updatetype = "";
			
			if (kafkaBean.getTopic() == null) {
				kafkaBean.setTopic("NewProdInfo");
			}
			if ((kafkaBean.getTopic().equals("NewProdInfo") && (kafkaBean.getActionCode().equals("0") || kafkaBean.getActionCode().equals("3")))
					|| (kafkaBean.getTopic().equals("TencentAppFree")
							&& (kafkaBean.getActionCode().equals("0") || kafkaBean.getActionCode().equals("4")))
					|| (kafkaBean.getActionCode().equals("0") && kafkaBean.getTopic().equals("NewProdECApi"))
					|| (kafkaBean.getTopic().equals("ProdOrderInfo") && ( kafkaBean.getActionCode().equals("0") || kafkaBean.getActionCode().equals("3")))
					|| (kafkaBean.getTopic().equals("AsynTfBhTrade") && ( kafkaBean.getActionCode().equals("0")))
					|| ("Syngnathus".equals(kafkaBean.getTopic()) && "0".equals(kafkaBean.getActionCode())) //银铃龙王卡
					) {
				updatetype = "0";
				/****
				errorinfo = order(info, kafkaBean);
				if("ok".equals(errorinfo)){
					//System.out.println(JsonUtil.objectToJson(info));
					kidao.syncOrder(info);
					//指定产品发短信
					insertSmsmq(info, smssendList);
					//写日志
					kidao.writeLogOrder(info);
				}
				*****/
			} else {
				updatetype = "1";
				/*****
				errorinfo = cancel(info, kafkaBean);
				if("ok".equals(errorinfo)){
					kidao.syncCancel(info);
					//指定产品发短信
					insertSmsmq(info, smssendList);
					//写日志
					kidao.writeLogCancel(info);
				}
				****/
			}
			
			//业务订购退订限制验证
			if(this.valiOperLimit(info.getId(),info.getSpid(),updatetype)){
				throw new VideoException("1000", "业务办理限制，不处理");
			}
			//获取配置信息
			List<HashMap> productConf = kidao.getProductConf(kafkaBean.getProductId());
			HashMap<String,String> product = new HashMap<>();
			if (productConf!=null&&productConf.size()>0){
				product = productConf.get(0);
			}
			//订购
			if("0".equals(updatetype)){
				errorinfo = order(info, kafkaBean);
				logger.info("=======第一个地否能订购："+errorinfo+",spid: "+info.getSpid());
				if(!"ok".equals(errorinfo)) {
					kidao.syncOrder(info);
					//指定产品发短信
					insertSmsmq(info, smssendList);
					//写日志
					kidao.writeLogOrder(info);
				}else {
					//TB_KAFKA_PRODUCT表EXT1字段不为空时落EXT1字段的spid订购关系，91022137除外，这个产品允许同时存在新旧spid订购关系
					if (!StringUtils.isEmpty(product.get("EXT1"))&&!"91022137".equals(kafkaBean.getProductId())){
						info.setSpid(product.get("EXT1"));
						info.setCpid(kidao.getCpid(product.get("EXT1")));
						errorinfo = order(info, kafkaBean);
					}
					logger.info("-------第二个地否能订购："+errorinfo+",spid: "+info.getSpid());

					kidao.syncOrder(info);
					//指定产品发短信
					insertSmsmq(info, smssendList);
					//写日志
					kidao.writeLogOrder(info);

				}

			}else{//退订
				try {
					errorinfo = cancel(info, kafkaBean);
					if ("ok".equals(errorinfo)) {
						kidao.syncCancel(info);
						//指定产品发短信
						insertSmsmq(info, smssendList);
						//写日志
						kidao.writeLogCancel(info);
					}
				}catch (Exception e1){
					logger.error(e1.getMessage(), e1);
					//由于星芒卡产品1000 退订时使用91022137productid，故传此productid退订时将关联两个产品均退订，即spid=1000和spid=2000
//					if("91022137".equals(kafkaBean.getProductId())){
					if (!StringUtils.isEmpty(product.get("EXT1"))){
						if (e1 instanceof VideoException) {
							resultcode = ((VideoException) e1).getErrorcode();
							errorinfo = ((VideoException) e1).getMessage();
						} else {
							resultcode = "8000";
							errorinfo = "系统忙，"+e1.getMessage();
							logger.error(kafkaBean.getId()+"，同步订购关系出现异常，"+e1.getMessage(),e1);
						}
						info.setErrorcode(resultcode);
						info.setErrorinfo(errorinfo);
						//写订购日志
						writeLog(info);
					}else{
						throw e1;
					}
				}finally {
					//由于星芒卡产品1000 退订时使用91022137productid，故传此productid退订时将关联两个产品均退订，即spid=1000和spid=2000
//					if("91022137".equals(kafkaBean.getProductId())){
					if (!StringUtils.isEmpty(product.get("EXT1"))){
						logger.info("开始退订星芒卡产品");
//						info.setSpid("99134272");
//						info.setCpid("fhcp");

						info.setSpid(product.get("EXT1"));
						info.setCpid(kidao.getCpid(product.get("EXT1")));

						errorinfo = cancel(info, kafkaBean);
						if("ok".equals(errorinfo)){
							kidao.syncCancel(info);
							//指定产品发短信
							insertSmsmq(info, smssendList);
							//写日志
							kidao.writeLogCancel(info);
						}
					}
				}
			}
			
			resultcode = "0";
			errorinfo = "".equals(errorinfo) ? "ok" : errorinfo;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage();
			} else {
				resultcode = "8000";
				errorinfo = "系统忙，"+e.getMessage();
				logger.error(kafkaBean.getId()+"，同步订购关系出现异常，"+e.getMessage(),e);
			}
			info.setErrorcode(resultcode);
			info.setErrorinfo(errorinfo);
			//写订购日志
			writeLog(info);
		} finally {
			//更新处理结果
			updProcRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info("[" + (totalSize - threadSignal.getCount()) + "/" + totalSize + "]:userid:["
			+ kafkaBean.getSerialNumber() + "，id=>"+kafkaBean.getId()+"]同步结果=>" + (resultcode) + "，花费时间："+ (System.currentTimeMillis() - start) + " ms");
		}
	}
	
	/**
	 * 确认是否能订购
	 * @param orderInfo
	 * @param kafkaBean
	 * @return
	 * @throws Exception
	 */
	public String order(OrderInfo orderInfo, KafkaBean kafkaBean) throws Exception {
		String errorInfo = "";
		orderInfo.setStatus(0);
		orderInfo.setOrderchannel("kafka");
		orderInfo.setOrderstatus(2);
		orderInfo.setOrdertime(kafkaBean.getOperaTime());
		ProvinceBean bean = kidao.getProvInfo(orderInfo.getUserid().substring(0, 7));
		if(bean != null){
			orderInfo.setProvince(bean.getProvince());
			orderInfo.setArea(bean.getArea());
		}
		List<OrderInfo> listOrderInfo = kidao.findByUser(orderInfo.getUserid());
		if (listOrderInfo != null && listOrderInfo.size() > 0) {
			if (!StringUtils.isEmpty(orderInfo.getMutex())) {
				String[] mutexSpids = orderInfo.getMutex().trim().split(",");
				for (String mutexSpid : mutexSpids) {
					for (OrderInfo oldOrderInfo : listOrderInfo) {
						if (oldOrderInfo.getSpid().equals(orderInfo.getSpid()) ) {
							if(oldOrderInfo.getStatus() == 0){
								//接收到的订购请求时间是否大于本地订购时间。若大于本地订购时间，则覆盖更新本地订购时间接收到的订购请求时间是否大于本地订购时间。
								// 若大于本地订购时间，则覆盖更新本地订购时间
								if (NumberUtils.toLong(oldOrderInfo.getOrdertime()) >= NumberUtils.toLong(orderInfo.getOrdertime()))
									throw new VideoException("9010", "您已订购，请不要重复订购！", new String[0]);
								errorInfo = "用户已订购，更新订购时间为最新";
							}else if(oldOrderInfo.getStatus() == 1 && NumberUtils.toLong(oldOrderInfo.getCanceltime()) > NumberUtils.toLong(orderInfo.getOrdertime())){
								throw new VideoException("9010", "订购时间晚于本地退订时间", new String[0]);
							}else if(oldOrderInfo.getStatus() == 1&&(oldOrderInfo.getEndtime().substring(0,6).equals(orderInfo.getOrdertime().substring(0,6)))){
								//退订状态，且是当月退订，更新订购状态
								errorInfo = "当月退订，更新订购状态与订购时间";
							}
						}
						if (oldOrderInfo.getSpid().equals(mutexSpid) && (StringUtils.isEmpty(oldOrderInfo.getEndtime()) || (StringUtils.isNotEmpty(oldOrderInfo.getEndtime()) && NumberUtils.toLong(oldOrderInfo.getEndtime()) > NumberUtils.toLong(kafkaBean.getOperaTime()))))  {
							throw new VideoException("9011", "您已订购同类产品!", new String[0]);
						}
					}
				}
			} else {
				if (listOrderInfo != null && listOrderInfo.size() != 0) {
					for (OrderInfo oldOrderInfo : listOrderInfo) {
						if (oldOrderInfo.getSpid().equals(orderInfo.getSpid()) ) {
							if(oldOrderInfo.getStatus() == 0){
								//大于覆盖
								if (NumberUtils.toLong(oldOrderInfo.getOrdertime()) >= NumberUtils.toLong(orderInfo.getOrdertime()))
									throw new VideoException("9010", "您已订购，请不要重复订购！", new String[0]);
                                errorInfo = "用户已订购，更新订购时间为最新";
							}else if(oldOrderInfo.getStatus() == 1 && NumberUtils.toLong(oldOrderInfo.getCanceltime()) > NumberUtils.toLong(orderInfo.getOrdertime())){
								throw new VideoException("9010", "订购时间晚于本地退订时间", new String[0]);
							}else if(oldOrderInfo.getStatus() == 1&&(oldOrderInfo.getEndtime().substring(0,6).equals(orderInfo.getOrdertime().substring(0,6)))){
								//退订状态，且是当月退订，更新订购状态
								errorInfo = "当月退订，更新订购状态与订购时间";
							}
						}
					}
				}
			}
		}
		orderInfo.setEffecttime(kidao.getCurrentTime());
		orderInfo.setBatchid(DateFormatUtils.format(new Date(), "yyyyMMddHH"));

		if (orderInfo != null && !StringUtils.isEmpty(orderInfo.getUserid())) {
			if (StringUtils.isEmpty(errorInfo)){
				errorInfo = "ok";
			}
		} else {
			throw new VideoException("9010", "手机号码为空", new String[0]);
		}
		return errorInfo;
	}
	
	/**
	 * 确定是否能退订
	 * @param orderInfo
	 * @param kafkaBean
	 * @return
	 * @throws Exception
	 */
	public String cancel(OrderInfo orderInfo, KafkaBean kafkaBean) throws Exception {
		String errorInfo = "";
		orderInfo.setStatus(1);
		orderInfo.setCancelchannel("kafka");
		orderInfo.setOrderstatus(4);
		orderInfo.setCanceltime(kafkaBean.getOperaTime());
		ProvinceBean bean = kidao.getProvInfo(orderInfo.getUserid().substring(0, 7));
		if(bean != null){
			orderInfo.setProvince(bean.getProvince());
			orderInfo.setArea(bean.getArea());
		}
		List<OrderInfo> listOrderInfo = kidao.findByUser(orderInfo.getUserid());
		if (listOrderInfo == null || listOrderInfo.size() == 0) {
			throw new VideoException("9016", "退订失败,无该用户订购关系或者已失效", new String[0]);
		}
		boolean hasrel = false;
		for (OrderInfo orderInfo2 : listOrderInfo) {
			if (orderInfo2.getSpid().equals(orderInfo.getSpid())) {
				if (orderInfo2.getStatus() == 1) {
					if (NumberUtils.toLong(orderInfo2.getCanceltime()) >= NumberUtils.toLong(orderInfo.getCanceltime()))
						throw new VideoException("9016", "退订失败,用户已退订", new String[0]);
                    hasrel = true;
                    errorInfo = "用户已退订，更新退订时间为最新";
                    break;
				} else {
					if(NumberUtils.toLong(orderInfo2.getOrdertime()) >NumberUtils.toLong(orderInfo.getCanceltime())){
						throw new VideoException("9016", "退订失败,退订时间早于订购时间", new String[0]);
					}
					hasrel = true;
					break;
				}
			}
		}
		if (!hasrel) {
			throw new VideoException("9018", "退订失败,该用户没有订购该产品或者已失效", new String[0]);
		}
		orderInfo.setEndtime(kidao.getEndDayOfCurrentMonth(orderInfo.getCanceltime()));
		errorInfo = "ok";
		return errorInfo;
	}
	
	/**
	 * 写订购日志
	 * @param info
	 */
	private void writeLog(OrderInfo info){
		try {
			if(info == null || info.getUserid() == null) return;
			if(info.getStatus() == 0){ //订购
				kidao.writeLogOrder(info);
			}else{
				kidao.writeLogCancel(info);
			}
		} catch (Exception e) {
			logger.error(info.getUserid()+"=>"+info.getSpid()+"，写订购日志出现异常，"+e.getMessage(),e);
		}
	}
	
	  /**
	   * 添加短信发送到mq
	   * @param orderInfo
	   */
	  private void insertSmsmq(OrderInfo orderInfo, List<String> smssendList){
			try {
				//logger.info("可以发短信的SPID=>"+smssendList);
				//发短信的SPID配置为空，或者当前的SPID没有配置
				if(smssendList == null || smssendList.size() == 0 || !smssendList.contains(orderInfo.getSpid())) return;
	
				Map<String, String> map = new HashMap<String, String>();
				map.put("userid", orderInfo.getUserid());
				map.put("cpid", orderInfo.getCpid());
				map.put("spid", orderInfo.getSpid());
				map.put("result", "0");
				map.put("style","0000");
				map.put("times", "");
				map.put("orderType", "");
				map.put("type", "cssms");
				//4G沃派校园卡
				//if(!"99134251".equals(orderInfo.getSpid())) return ;
				if(orderInfo.getStatus() == 0){ //订购
					map.put("channel", orderInfo.getOrderchannel());
					map.put("busiType", "kafka_tran_succ");	//订购成功短信
				}else{
					map.put("channel", orderInfo.getCancelchannel());
					map.put("busiType", "kafka_cancel_succ");//退订成功短信
				}
				String url = kidao.getDictionaryValue("mqReciveUrl");
				URLUtil.post(url, JsonUtil.objectToJson(map),1000 * 30);
				logger.info("订购退订发短信推送队列完成， data："+JsonUtil.objectToJson(orderInfo));
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("订购退订发短信推送队列出现异常，"+e.getMessage()+"， data："+JsonUtil.objectToJson(orderInfo),e);
			}
	  }
	
	/**
	 * 更新源数据处理结果
	 * @param resultcode
	 * @param errorinfo
	 */
	private void updProcRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 200){
				errorinfo = errorinfo.substring(0, 200);
			}		
			//更新处理结果
			kidao.updRequestDataRes(resultcode, errorinfo, kafkaBean.getId());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+kafkaBean.getId(), e);
		} 
	}
	
    /**
     * 验证业务操作限制（是否能订购，是否能退订）
     * @param id
     * @param spid
     * @param updatetype 操作标识：0订购、 1退订
     * @return
     * @throws Exception
     */
    private boolean valiOperLimit(String id, String spid,String updatetype) throws Exception {
    	List<HashMap> dataList = kidao.getSpBusiLimitConf(spid);
		boolean haslimit = false;
        if(dataList != null && dataList.size()>0){   		
        	for(HashMap tm : dataList){
        		if(updatetype.equals(tm.get("OPERTYPE"))){
        			haslimit = true;
        			break;
        		}
        	}     
        	//被限制抛异常
        	if(haslimit){
        		//throw new BusinessException("9016", id+"业务办理限制，不处理", new String[0]);
        	}
        }
        
        return haslimit;
    }
}
