package com.chinacreator.kafka.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

import com.chinacreator.kafka.bean.*;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.commons.lang.time.DateUtils;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.URLUtil;

/**
 * 订购数据入库处理类
 * @author xu.zhou
 * @date 20201208
 */
public class SyncPutDeal implements Runnable{
	
	private Logger logger;
	
	private SourceDataRecDao dao;
	
	private KafkaInfoDao kidao;
	
	private int totalSize;
	
	private CountDownLatch threadSignal;
	
	private SourceDataRecBean sdrBean;
	
	//产品ID配置信息
	HashMap<String, String> productmap;
	
	public SyncPutDeal(SourceDataRecBean sdrBean, int totalSize, CountDownLatch threadSignal,Logger logger, SourceDataRecDao dao,
			HashMap<String, String> productmap, KafkaInfoDao kidao) {
		this.logger = logger;
		this.dao = dao;
		this.kidao = kidao;
		this.sdrBean = sdrBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
		this.productmap = productmap;
	}
	
	public void run(){
		long start = System.currentTimeMillis();
		String resultcode = "-1";
		String errorinfo = "";
		//是否推送
		boolean result = false;
		try {			
			List<KafkaBean> listBeans = truncBean();
			if(listBeans != null && listBeans.size()>0){
				for(KafkaBean tmp : listBeans){
					kidao.addSyncPutData(tmp);
					resultcode = "0";
					errorinfo = "入库到同步表成功，入库数据条数为"+listBeans.size();
				}
			}else{
				resultcode = "1005";
				errorinfo = "无数据入库同步表";
			}
		} catch (Exception e) {
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage();
			} else {
				e.printStackTrace();
				resultcode = "8000";
				errorinfo = "处理数据出现异常，"+e.getMessage();
				logger.error("处理数据出现异常，"+e.getMessage(),e);
			}
		} finally {
			updProcRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - start)+" ms，id："+sdrBean.getId()+", 处理结果："+resultcode+"=>"+errorinfo);
		}
	}
	
	private List<KafkaBean> truncBean() throws Exception{
		List<KafkaBean> listBean = new ArrayList<KafkaBean>();
		KafkaBean bean = new KafkaBean();
		String filename = sdrBean.getOldlogname();
		String sourcedata = sdrBean.getSourcedata();
		
		bean.setGettime(sdrBean.getInserttime());
		bean.setPartition(Integer.parseInt(sdrBean.getPartition()));
		bean.setOffset(Long.parseLong(sdrBean.getOffset()));
		bean.setFilename(sdrBean.getId());
		
		if (filename.contains("wk")) {
			if (sourcedata.contains("CityCode")) {
				throw new VideoException("1006","包含CityCode，数据不合法，不入库");
			}
			WCbean wcBean = (WCbean) JsonUtil.jsonToBean(sourcedata, WCbean.class);
			bean.setSerialNumber(wcBean.getPhoneNum());
			bean.setActionCode(wcBean.getStateTag());
			bean.setOperaTime(
					wcBean.getStateChgTime().replaceAll(" ", "").replaceAll("-", "").replaceAll(":", ""));
			bean.setProductId(wcBean.getProductNo());
			bean.setTopic("TencentAppFree");
		} else if (filename.contains("bg") || filename.contains("chg")) {
			String topic = "NewProdECApi";
			if (filename.contains("chg")) topic = "ProdOrderInfo";
			BGbean bgBean = (BGbean) JsonUtil.jsonToBean(sourcedata, BGbean.class);
			//System.out.println("bgBean to json: "+JsonUtil.objectToJson(bgBean));
			if (bgBean != null && bgBean.getKAFKA_TYPE().equals("2") && StringUtils.isNotEmpty(bgBean.getOLD_PRODUCT_ID()) && productmap.containsKey(bgBean.getNEW_PRODUCT_ID())) {
				// 旧产品ID和新产品ID不相等，表示用户变更2I产品，需要将变更数据转换成两条数据，一条退订，一条订购
				if (!bgBean.getNEW_PRODUCT_ID().equals(bgBean.getOLD_PRODUCT_ID())) {
					if (productmap.containsKey(bgBean.getOLD_PRODUCT_ID())) {
						//System.out.println("-------------------------------------");
						bean = setBean(bean,bgBean,topic,"1");

						//bean.setGettime(str.substring(str.indexOf("[") + 1, str.indexOf(",")));
						//String newstr = str.substring(str.indexOf("]") + 1, str.indexOf("{")).replace("-", "").trim();
						//bean.setPartition(Integer.parseInt(newstr.split(",")[1]));
						//bean.setOffset(Long.parseLong(newstr.split(",")[0]));
						//bean.setFilename(filename);
						
						if (bean.getOperaTime().length() <= 14) {
							listBean.add(bean); //先添加一条
						}
						bean = new KafkaBean();
						bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
						bean.setActionCode("0");
						//System.out.println("OPERA_TIME=>"+bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":",""));
						bean.setOperaTime(getTime(bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":","")));
						bean.setProductId(bgBean.getNEW_PRODUCT_ID() );
						bean.setTopic(topic);
					}else{
						bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
						bean.setActionCode("0");
						bean.setOperaTime(getTime(bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":","")));
						bean.setProductId(bgBean.getNEW_PRODUCT_ID() );
						bean.setTopic(topic);
					}
				}else{
					bean = setBean(bean,bgBean,topic,"0");
				}
			} else if(bgBean != null && bgBean.getKAFKA_TYPE().equals("2") && StringUtils.isNotEmpty(bgBean.getOLD_PRODUCT_ID()) && productmap.containsKey(bgBean.getOLD_PRODUCT_ID())){
				bean = setBean(bean,bgBean,topic,"1");
			} else if(bgBean != null && bgBean.getKAFKA_TYPE().equals("2") && StringUtils.isNotEmpty(bgBean.getNEW_PRODUCT_ID()) && productmap.containsKey(bgBean.getNEW_PRODUCT_ID())){
				bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
				bean.setActionCode("0");
				bean.setOperaTime(getTime(bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":","")));
				bean.setProductId(bgBean.getNEW_PRODUCT_ID() );
				bean.setTopic(topic);
			}
		} else if(filename.contains("old")){
			OldBean oldBean =(OldBean)JsonUtil.jsonToBean(sourcedata, OldBean.class);
			bean.setSerialNumber(oldBean.getOLD_SERIAL_NUMBER());
			bean.setActionCode("1");
			bean.setOperaTime(oldBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":",""));
			bean.setProductId(oldBean.getPRODUCT_ID());
			bean.setTopic("NewProdECApi");
			
			//bean.setGettime(str.substring(str.indexOf("[") + 1, str.indexOf(",")));
			//String newstr = str.substring(str.indexOf("]") + 1, str.indexOf("{")).replace("-", "").trim();
			//bean.setPartition(Integer.parseInt(newstr.split(",")[1]));
			//bean.setOffset(Long.parseLong(newstr.split(",")[0]));
					
			bean.setKafkatype("o");
			//bean.setFilename(filename);
			if (bean.getOperaTime().length() <= 14) {
				listBean.add(bean);
			}
			bean = new KafkaBean();
			bean.setSerialNumber(oldBean.getNEW_SERIAL_NUMBER());
			bean.setActionCode("0");
			bean.setOperaTime(oldBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":",""));
			bean.setProductId(oldBean.getPRODUCT_ID());
			bean.setTopic("NewProdECApi");
			insertMq(oldBean.getNEW_SERIAL_NUMBER(), oldBean.getOLD_SERIAL_NUMBER());
		} else if(filename.contains("vf")){
			VFbean vfbean = (VFbean) JsonUtil.jsonToBean(sourcedata, VFbean.class);
			bean.setSerialNumber(vfbean.getSerialNumber());
			bean.setActionCode(vfbean.getActionCode());
			bean.setOperaTime(vfbean.getOperaTime());
			bean.setProductId(vfbean.getProductId());
			bean.setTopic("ProdOrderInfo");
		} else if(filename.contains("wopai")){//99134251 fhcp 4G沃派校园卡，三个产品对应一个SPID
			String topic = "NewProdECApi";
			BGbean bgBean = (BGbean) JsonUtil.jsonToBean(sourcedata, BGbean.class);
			bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
			bean.setActionCode("0".equals(bgBean.getSTATE_TAG()) ? "0" : "1"); //STATE_TAG为0时是订购，其余为退订
			bean.setOperaTime(bgBean.getACCEPT_DATE());
			bean.setOperaTime(bean.getOperaTime().replaceAll("-", "").replaceAll(" ", "").replaceAll(":", ""));
			bean.setProductId(bgBean.getDISCNT_CODE());
			bean.setTopic(topic);
		} else if(filename.contains("bkPro") || filename.contains("ltCard") || filename.contains("xrpaCard") || filename.contains("xrfCard")){
			String topic = "AsynTfBhTrade";
			AysnTfBhbean asynbean = (AysnTfBhbean)JsonUtil.jsonToBean(sourcedata, AysnTfBhbean.class);
			bean.setSerialNumber(asynbean.getSERIAL_NUMBER());
			bean.setActionCode("592".equals(asynbean.getTRADE_TYPE_CODE()) ? "0" : "1"); //592是订购，594是退订
			bean.setOperaTime(asynbean.getACCEPT_DATE());
			bean.setOperaTime(bean.getOperaTime().replaceAll("-", "").replaceAll(" ", "").replaceAll(":", ""));
			bean.setProductId(asynbean.getPRODUCT_ID());
			bean.setTopic(topic);
        /**
         * 银铃卡配置 xu.zhou 20210909
         * kuaishou	1068	银龄天龙卡
		 * kuaishou	1069	银龄小龙卡
		 * kuaishou	1070	银龄大龙卡			
         */
		}else if ("yltlc".equals(filename)){//银铃龙王卡
			String topic = "Syngnathus";
			AysnTfBhbean asynbean = (AysnTfBhbean)JsonUtil.jsonToBean(sourcedata, AysnTfBhbean.class);
			bean.setSerialNumber(asynbean.getSERIAL_NUMBER());
			bean.setActionCode("0".equals(asynbean.getKAFKA_TYPE()) ? "0" : "1"); //0是订购，4是退订
			bean.setOperaTime(asynbean.getOPERA_TIME());
			bean.setOperaTime(bean.getOperaTime().replaceAll("-", "").replaceAll(" ", "").replaceAll(":", ""));
			bean.setProductId(asynbean.getPRODUCT_ID());
			bean.setTopic(topic);
			//只处理为0或为4的数据
			if(!"0".equals(asynbean.getKAFKA_TYPE()) && !"4".equals(asynbean.getKAFKA_TYPE())){
				throw new VideoException("1004","KAFKA_TYPE非0或4，不处理");
			}
		}else {
			bean = (KafkaBean) JsonUtil.jsonToBean(sourcedata, KafkaBean.class);
			//20220105
			//此前我们支撑的电商卡“宝卡乐享版”（产品ID=91022137），因业务调整，将衍生出别名卡产品，
			// 后续“产品ID=91022137”通过kafka（topic：NewProdInfo）同步时，
			// 若同步数据中有auditTag参数且auditTag=“J"或者“K”时，
			// 则我们本地落“星芒卡Young”产品订购关系，“星芒卡Young”产品在原有“宝卡乐享版”（产品ID=91022137）
			// 处理逻辑基础上新增30GB/月的“芒果TV定向流量”（同步给芒果侧15元订购关系），且同步成功后不发平台短信通知。
			if("J".equals(bean.getAuditTag())|| "K".equals(bean.getAuditTag())){
				bean.setProductId("91022137_xmk");
			}
			bean.setTopic("NewProdInfo");
		}
		bean.setGettime(sdrBean.getInserttime());
		bean.setPartition(Integer.parseInt(sdrBean.getPartition()));
		bean.setOffset(Long.parseLong(sdrBean.getOffset()));
		bean.setFilename(sdrBean.getId());
		//bean.setTopic(sdrBean.getTopic());
		
		if (bean.getOperaTime() != null && bean.getOperaTime().length() <= 14) {
			listBean.add(bean);
		}else{
			//System.out.println(String.valueOf(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date()))+"operatime: "+bean.getOperaTime()+"，数据不合法，不入库："+sourcedata);
			logger.info(sdrBean.getId()+"，operatime: "+bean.getOperaTime()+"，数据不合法，不入库："+sourcedata);
			throw new VideoException("1006","operatime数据不合法，不入库");
		}
		
		return listBean;
	}

	private KafkaBean setBean(KafkaBean bean, BGbean bgBean,String topic,String actionCode){
		bean.setSerialNumber(bgBean.getSERIAL_NUMBER());
		bean.setActionCode(actionCode);
		bean.setOperaTime(bgBean.getOPERA_TIME().replaceAll("-","").replaceAll(" ", "").replaceAll(":",""));
		bean.setProductId("0".equals(actionCode) ? bgBean.getNEW_PRODUCT_ID() : bgBean.getOLD_PRODUCT_ID());
		bean.setTopic(topic);
		return bean;
	}
	
	/**
	 * kafka换号处理
	 * com.chinacreator.process.job.KafkaTransJob
	 * @param userid
	 * @param oldUserid
	 * @throws Exception
	 */
	public void insertMq(String userid,String oldUserid) throws Exception {
		try{ 
			Map<String,String> map = new HashMap<String, String>();
			map.put("userid", userid);
			map.put("oldUserid", oldUserid);
			map.put("type", "kfkchange");
			String url = kidao.getDictionaryValue("mqReciveUrl");
			URLUtil.post(url, JsonUtil.objectToJson(map),1000 * 30);
		}catch(Exception e){
	    	e.printStackTrace();
	    }
	}
	
	public String getTime(String timeStr) throws ParseException{
		Calendar c = DateUtils.toCalendar(DateUtils.parseDate(timeStr, new String[]{"yyyyMMddHHmmss"}));
		c.set(Calendar.MONTH, c.get(Calendar.MONTH)+1);
		c.set(Calendar.DAY_OF_MONTH, 1);
		c.set(Calendar.HOUR_OF_DAY, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.SECOND, 0);
		return DateFormatUtils.format(c, "yyyyMMddHHmmss");
	}
	
	/**
	 * 更新源数据处理结果
	 * @param resultcode
	 * @param errorinfo
	 */
	private void updProcRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}
			//String sql = "UPDATE TD_KAFKA_SRCDATA SET RESULTCODE = '"+resultcode+"', RESULTINFO = '"+errorinfo+"', UPDTIME = SYSDATE WHERE RESULTCODE = '2' AND ID = '"+sdrBean.getId()+"' ";
			//更新处理状态
			//int rows = dao.updSrcDataBySql(sql);
			//logger.info("更新数据完成，影响行："+rows);
			
			int rows = dao.updCommSrcDataRes(resultcode, errorinfo, sdrBean.getId());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新状态出现异常，"+e.getMessage()+"=>"+sdrBean.getId());
		} 
	}
}
