package com.chinacreator.kafka.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.AcproductConfigBean;
import com.chinacreator.kafka.bean.OrderInfo;
import com.chinacreator.kafka.bean.ReflowBean;
import com.chinacreator.kafka.bean.SpInfo;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.util.*;
import kafka.utils.Json;
import org.jboss.logging.Logger;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * @Author ShuiYing_Ou
 * @Description 三级回流挽回
 * @Date 2022-07-26
 **/
public class ThreeReflowBackDeal implements Runnable {
    private ReflowBean reflowBean;
    private CountDownLatch threadSignal;
    private Logger logger;
    private ActivityReflowDao activityReflowDao;
    private KafkaInfoDao kafkaInfoDao;

    public ThreeReflowBackDeal(ReflowBean reflowBean, CountDownLatch threadSignal, Logger logger, ActivityReflowDao activityReflowDao, KafkaInfoDao kafkaInfoDao) {
        this.activityReflowDao = activityReflowDao;
        this.logger = logger;
        this.threadSignal = threadSignal;
        this.reflowBean = reflowBean;
        this.kafkaInfoDao = kafkaInfoDao;
    }

    @Override
    public void run() {
        HashMap<String, Object> logMap = new HashMap<>();
        logMap.put("reflowBean", reflowBean);
        try {
            ReflowBean reflowBean = activityReflowDao.qrySmsBack(this.reflowBean.getSpid(), this.reflowBean.getUserid());
            if (reflowBean == null) {
                List<OrderInfo> orderInfoList = activityReflowDao.findOrderRel(this.reflowBean.getUserid(), this.reflowBean.getSpid());
                if (orderInfoList != null && orderInfoList.size() > 0) {
                    OrderInfo orderInfo = orderInfoList.get(0);
                    SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
                    long ordertime = format.parse(orderInfo.getOrdertime()).getTime();
                    long inserttime = format.parse(this.reflowBean.getInserttime()).getTime();
                    long curreTime = new Date().getTime();
                    // 订购时间是否在三级回流表入库之间到当前时间
                    if (ordertime < inserttime || ordertime > curreTime) {
                        if (!"0".equals(Integer.toString(orderInfo.getStatus()))) {
                            sendSMS(logMap);
                        }else {
                            logger.info(this.reflowBean.getUserid() + "==>" + this.reflowBean.getSpid() + "====>" + orderInfo.getStatus() + "===> 从触发三级回流至今曾订购过该产品");
                        }
                    } else {
                        logger.info(this.reflowBean.getUserid() + "==>" + this.reflowBean.getSpid() + "====>" + orderInfo.getOrdertime() + "===> 从触发三级回流至今曾订购过该产品");
                    }
                } else {
                    sendSMS(logMap);
                }
            }else {
                logger.info(this.reflowBean.getUserid() + "==>" + this.reflowBean.getSpid() + "====>" + "该用户该产品本月已发月挽回短信");
            }
        } catch (Exception e) {
            e.printStackTrace();
            reflowBean.setBackcode("3");
            // 将数据状态改为失败
            activityReflowDao.updateBackData(reflowBean.getBackcode(), reflowBean.getId());
            logger.error(reflowBean.getUserid() + "====>三级回流用户再挽回执行失败" + e.getMessage(), e);
        } finally {
            logger.info(JSON.toJSONString(logMap));
            threadSignal.countDown();
        }
    }

    private void sendSMS(HashMap<String, Object> logMap) {
        String result;
        String backcode = null;
        String backresinfo = null;
        String taskid = null;
        ReflowBean bean = new ReflowBean();
        // 调信用额度
        result = getEcredit(logMap);
        if (!StringUtils.isEmpty(result)) {
            Object data = JSON.parseObject(result).get("data");
            if (!ObjectUtils.isEmpty(data)) {
                List<HashMap> hashMaps = JSONArray.parseArray((String) data, HashMap.class);
                if (hashMaps.size() > 0) {
                    int availableCreditValue = Integer.parseInt(hashMaps.get(0).get("AVAILABLE_CREDIT_VALUE").toString());
                    logMap.put("availableCreditValue: ", availableCreditValue);
                    if (availableCreditValue >= 2500) {
                        // 再挽回信息入库
                        String id = IdGenerateUtil.uuid6();
                        bean.setId(id);
                        bean.setSpid(reflowBean.getSpid());
                        bean.setUserid(reflowBean.getUserid());
                        bean.setResultcode("1");
                        bean.setResultinfo("处理中");
                        bean.setActivitycode("threeReflowBack");
                        activityReflowDao.insertBackBean(bean);
                        // 短信模板内容
                        AcproductConfigBean configBean = activityReflowDao.selectAcproduct(reflowBean.getSpid());
                        SpInfo spInfo = activityReflowDao.getDirectype(reflowBean.getSpid());
                        String extend2 = configBean.getExtend2();
                        logMap.put("extend2: ", extend2);
                        String[] config = extend2.split("\\|");
                        String SMSDetail = activityReflowDao.getSMSDetails("threeReflowBackSMS");
                        String[] SMSDetails = SMSDetail.split("\\|");
                        Map<String, Object> map = new HashMap<>();
                        // 纯流量为0
                        if ("0".equals(config[2])) {
                            // 短信模板，必须传值
                            map.put("appname",config[1]);
                        } else {
                            map.put("appname", config[1] + SMSDetails[0]);
                        }
                        map.put("flow", config[3]);
                        if ("3".equals(spInfo.getDirectype())) {
                            map.put("price", SMSDetails[1]);
                        } else {
                            map.put("price", " ");
                        }
                        map.put("url", config[0]);
                        map.put("url2", SMSDetails[2]);
                        try {
                            String smsid = kafkaInfoDao.getDictionaryValue("threeReflowBackSmsid");
                            String pwd = kafkaInfoDao.getDictionaryValue("threeReflowBackPwd");
                            String smsResult = SendSmsUtil.sendSMSWithVars(reflowBean.getUserid(), SMSDetails[3], smsid, pwd, map);
                            logMap.put("smsResult: ", smsResult);
                            logger.info(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "==> 三级回流再挽留发送营销短信结果" + smsResult);
                            if (!org.apache.commons.lang.StringUtils.isBlank(smsResult)) {
                                backresinfo = smsResult;
                                JSONObject obj = JSON.parseObject(smsResult);
                                backcode = obj.getString("resultcode");
                                taskid = obj.getString("taskid");
                            } else {
                                backcode = "5000";
                                backresinfo = "无返回结果";
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                            backcode = "8000";
                            backresinfo = "发送出现异常 " + e.getMessage();
                            logger.info(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "====> 三级回流再挽回短信发送异常" + e.getMessage() + e);
                        } finally {
                            try {
                                //截断数据，防止更新失败
                                if (!StringUtils.isEmpty(backresinfo) && backresinfo.length() > 1000) {
                                    backresinfo = backresinfo.substring(0, 1000);
                                } else {
                                    backresinfo = "已处理";
                                }
                                ReflowBean reflowBean = new ReflowBean();
                                reflowBean.setId(id);
                                reflowBean.setResultcode(backcode);
                                reflowBean.setResultinfo(backresinfo);
                                reflowBean.setTaskid(taskid);
                                //更新发送结果
                                this.activityReflowDao.updBackSms(reflowBean);
                            } catch (Exception e) {
                                e.printStackTrace();
                                logger.error(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "==> 更新短信发送结果出现异常" + e.getMessage() + e);
                            }
                        }
                    }
                }
            }
        } else {
            logger.info(reflowBean.getUserid() + "==>" + reflowBean.getSpid() + "信用额度接口无返回" );
        }
    }

    private String getEcredit(HashMap<String, Object> logMap) {
        String result = null;
        try {
            String url = activityReflowDao.getEcredit("getEcredit");
            logMap.put("url :", url);
            String channel = "kc";
            String pwd = activityReflowDao.getPwd(channel);
            logMap.put("pwd :", pwd);

            //                String pwd = "315b5de2";
            String userid = reflowBean.getUserid();
            userid = DesUtil.encode(userid, pwd);
            String timestamp = System.currentTimeMillis() / 1000 + "";
            String signature = MD5.MD5Encode(userid + channel + timestamp + pwd);
            userid = URLEncoder.encode(userid, "utf-8");
            StringBuilder sb = new StringBuilder(url);
            url = sb.append("?userid=").append(userid).append("&channel=").append(channel).append("&timestamp=").append(timestamp).append("&signature=").append(signature).toString();
            result = URLUtil.get(url);
            System.out.println(result);
            logMap.put("newurl :", url);
            logMap.put("result: ", result);
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("调用户信用额度查询接口失败");
        }
        return result;
    }
}
