package com.chinacreator.kafka.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.*;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.util.*;
import org.jboss.logging.Logger;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.net.URLEncoder;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CountDownLatch;

/**
 * @Author shuiying.ou
 * @Date 2022-03-22 11:47
 * @DESCRIPTION
 */

public class ThreeReflowDeal implements Runnable {

    private Logger logger;

    private ReflowBean reflowBean;

    private CountDownLatch threadSignal;

    private ActivityReflowDao activityReflowDao;

    private KafkaInfoDao kafkaInfoDao;


    public ThreeReflowDeal(ReflowBean reflowBean, CountDownLatch threadSignal, Logger logger, ActivityReflowDao activityReflowDao, KafkaInfoDao kafkaInfoDao) {
        this.reflowBean = reflowBean;
        this.threadSignal = threadSignal;
        this.logger = logger;
        this.activityReflowDao = activityReflowDao;
        this.kafkaInfoDao = kafkaInfoDao;
    }

    @Override
    public void run() {
        HashMap<String, Object> logMap = new HashMap<>();
        try {
            String spid = reflowBean.getSpid();
            String userid = reflowBean.getUserid();
            String resultcode = "";
            String resultinfo = "";
            boolean hasupd = false; //是否要更新发送结果
            String activitycode = "reflowActivityLevel3";
            String id = null;
            SpInfo spInfo = activityReflowDao.qrySpInfo(reflowBean.getSpid(), activitycode);
            if (spInfo != null) {
                // 活动时间判断
                Date endTime = spInfo.getEndtime();
                String orderTime = reflowBean.getInserttime();
                SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
                long ordertime = format.parse(orderTime).getTime();
                long time = endTime.getTime();  // 活动结束时间
                logMap.put("spInfo", spInfo);
                if (ordertime <= time) {
                    OrderInfo info = this.activityReflowDao.findByUserAndSpid(spid, userid);
                    if (info != null) {
                        logger.info(userid + "=>" + spid + "=>" + "已订购该产品");
                    } else {
                        int rowCoupon = this.activityReflowDao.selectCouponLog(userid, spid, activitycode);
                        if (rowCoupon > 0) {
                            logger.info(userid + "=>" + spid + "=>" + "已领过订购该产品下发的话费券");
                        } else {
                            ReflowBean reflow = this.activityReflowDao.selectSendLog(userid, spid, activitycode);
                            logMap.put("reflow", reflow);
                            if (reflow != null) {
                                logger.info(userid + "=>" + spid + "=>" + "当天已发送过三级回流短信");
                            } else {
                                Integer availableCreditValue = getEcredit(logMap);
                                if (availableCreditValue <= 0) {
                                    // 三级促回流
                                    try {
                                        ReflowBean bean = new ReflowBean();
                                        id = IdGenerateUtil.uuid6();
                                        bean.setId(id);
                                        bean.setUserid(userid);
                                        bean.setSpid(spid);
                                        bean.setResultcode("1");
                                        bean.setResultinfo("处理中");
                                        bean.setActivitycode("reflowActivityLevel3");
                                        //添加到发送记录表，同一号码同一产品同一天只能发送一次，表有唯一约束
                                        this.activityReflowDao.insertReflowBean(bean);
                                        //入库正常
                                        hasupd = true;
                                        // 分转元
                                        BigDecimal availableValue = new BigDecimal(Math.abs(availableCreditValue)).divide(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_UP);
                                        String extend2 = spInfo.getExtend2();
                                        String[] config = extend2.split("\\|");
                                        Map<String, Object> map = new HashMap<>();
                                        map.put("spname", spInfo.getSpname());
                                        map.put("price", availableValue);  //欠费金额（基于信用额度接口）
                                        map.put("order", "充值后继续订购，将送您5元话费券，即领即到账");
                                        map.put("url1", "https://cs.mige.tv/H/recharge");
                                        map.put("url2", config[0]);

                                        if ("0".equals(config[2])) {
                                            map.put("spname2", config[1]);
                                            map.put("spname3", config[3]);
                                        } else {
                                            map.put("spname2", config[1] + "视频月会员+");
                                            map.put("spname3", config[3]);
                                        }
                                        map.put("cashbask", "成功订购立返5元话费，即领即到账");
                                        logMap.put("vars", map);
                                        // 三级回流发短信
                                        String contentid = "202204151711164421";
                                        String smsid = kafkaInfoDao.getDictionaryValue("reflowAct3Smsid");
                                        String pwd = kafkaInfoDao.getDictionaryValue("reflowAct3Pwd");
                                        String smsResult = SendSmsUtil.sendSMSWithVars(userid, contentid, smsid, pwd, map);
                                        logger.info(userid + "=>" + spid + "三级回流用户挽留发送短信结果" + smsResult);
                                        if (!org.apache.commons.lang.StringUtils.isBlank(smsResult)) {
                                            resultinfo = smsResult;
                                            JSONObject obj = JSON.parseObject(smsResult);
                                            resultcode = obj.getString("resultcode");
                                        } else {
                                            resultcode = "5000";
                                            resultinfo = "无返回结果";
                                        }
                                    } catch (Exception exception) {
                                        resultcode = "8000";
                                        resultinfo = "发送出现异常 " + exception.getMessage();
                                        exception.printStackTrace();
                                        logger.error("发送出现异常" + "==>" + spid + "==>" + userid + exception.getMessage() + exception);
                                    } finally {
                                        //	添加到发送成功
                                        if (hasupd) {
                                            try {
                                                //截断数据，防止更新失败
                                                if (resultinfo.length() > 1000) {
                                                    resultinfo = resultinfo.substring(0, 1000);
                                                } else {
                                                    resultinfo = "已处理";
                                                }
                                                //更新发送结果
                                                this.activityReflowDao.updSmsRec(id, resultcode, resultinfo);
                                            } catch (Exception e) {
                                                e.printStackTrace();
                                                logger.error(reflowBean.getUserid() + "=>" + reflowBean.getSpid() + "更新短信发送结果出现异常" + e.getMessage() + e);
                                            }
                                        }
                                    }
                                } else {
                                    logger.info(userid + "====>" + spid +"AVAILABLE_CREDIT_VALUE>0，不发送促回流短信 ==>" + availableCreditValue);
                                }
                            }
                        }
                    }

                } else {
                    logger.info("此次订购不在三级回流活动期" + userid +  "====>" + spid +"活动结束时间" + time);
                }
            }else {
                logger.info("该产品不在三级回流活动范围" + userid + "====>" + spid);
            }
            this.reflowBean.setResultcode("0");
            this.reflowBean.setResultinfo("已处理");
        } catch (Exception e) {
            e.printStackTrace();
            reflowBean.setResultcode("3");
            reflowBean.setResultinfo("失败");
            logger.error(reflowBean.getUserid() + "====>" + reflowBean.getSpid() +"发短信业务出错" + e.getMessage() + e);
        } finally {
            String log = JSON.toJSONString(logMap);
            logger.info(log);
            // 将数据状态改为已处理
            activityReflowDao.updateThreeReflowData(reflowBean.getResultcode(), reflowBean.getResultinfo(), reflowBean.getId());
            threadSignal.countDown();
        }
    }


    private Integer getEcredit(HashMap<String, Object> logMap) throws Exception {
        String result = null;
        String feeResult = null;
        String url = null;
        String feeUrl = null;
        Integer availableCreditValue = null;
        Integer allNewBalance = null;
        String userid = reflowBean.getUserid();
        try {
            // 调用户信用额度查询接口
            url = kafkaInfoDao.getDictionaryValue("getEcredit");
            String channel = "kc";
            String pwd = activityReflowDao.getPwd(channel);
            userid = DesUtil.encode(userid, pwd);
            String timestamp = System.currentTimeMillis() / 1000 + "";
            String signature = MD5.MD5Encode(userid + channel + timestamp + pwd);
            userid = URLEncoder.encode(userid, "utf-8");
            StringBuilder sb = new StringBuilder(url);
            url = sb.append("?userid=").append(userid).append("&channel=").append(channel).append("&timestamp=").append(timestamp).append("&signature=").append(signature).toString();
            result = URLUtil.get(url);
            logMap.put("url", url);
            logMap.put("ecreditResult", result);
            if (!StringUtils.isEmpty(result)) {
                JSONObject json = JSON.parseObject(result);
                if ("0".equals(json.getString("resultcode"))) {
                    String data = json.getString("data");
                    if (!StringUtils.isEmpty(data)) {
                        availableCreditValue = Integer.parseInt(JSONArray.parseArray(data, HashMap.class).get(0).get("AVAILABLE_CREDIT_VALUE").toString());
                    } else {
                        logger.info("用户信用额度查询接口返回的data值为空" + result);
                    }
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("调用户信用额度查询接口失败==>" + url + e.getMessage() + e);
            throw new Exception();
        }
        try {
            // 调用户余额查询接口
            feeUrl = kafkaInfoDao.getDictionaryValue("shareselectfeeinfoUrl");
            feeUrl = feeUrl + "?userid=" + reflowBean.getUserid();
            feeResult = URLUtil.get(feeUrl);
            logMap.put("feeUrl", feeUrl);
            logMap.put("feeResult", feeResult);
            if (!StringUtils.isEmpty(feeResult)) {
                JSONObject feeJson = JSON.parseObject(feeResult);
                if ("0".equals(feeJson.getString("resultcode"))) {
                    String data = feeJson.getString("data");
                    if (!StringUtils.isEmpty(data)) {
                        allNewBalance = Integer.parseInt(JSON.parseObject(JSONObject.toJSONString(JSONArray.parseArray(data, HashMap.class).get(0).get("ACCT_BALANCE_INFO"))).getString("ALL_NEW_BALANCE"));
                    } else {
                        logger.info("用户余额查询接口返回的data值为空" + result);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("调用户余额查询接口失败 ==>" +feeUrl + e.getMessage() + e);
            throw new Exception();
        }
        ReflowInvokeLog reflowInvokeLog = new ReflowInvokeLog();
        reflowInvokeLog.setUserid(reflowBean.getUserid());
        reflowInvokeLog.setAllnewbalance(allNewBalance);
        reflowInvokeLog.setEcreditresult(result);
        reflowInvokeLog.setAvailablecreditvalue(availableCreditValue);
        reflowInvokeLog.setFeeresult(feeResult);
        activityReflowDao.insertInvokeLog(reflowInvokeLog);

        return availableCreditValue;
    }
}
