package com.chinacreator.kafka.sync.handle.impl;

import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SendSmsUtil;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 【退订再挽回下发券码】活动
 *
 * @author shuiying.ou
 * @date 20220817
 * <p>
 * <p>
 * <p>
 * 1、限制每位用户、每个产品，半年内只允许领取一次5元话费券。半年的计算依据以订购时间作判断
 * 2、话费券有效期：1个月，按照31天计算，如8月11日下发，9月11日失效。
 * 3、通过特定渠道订购成功,生成5元话费券领取资格，则下发话费券待领取短信
 */
@Component("cancelActivity")
public class CancelActivityHandleService implements IHandleService {

    private static Logger logger = Logger.getLogger("cancelActivity");

    @Autowired
    ActivityHandleDao handleDao;
    @Autowired
    KafkaInfoDao kafkaInfoDao;


    @Override
    public HashMap<String, String> handle(HashMap<String, String> _data, HashMap<String, String> _invokeConf) throws Exception {
        HashMap<String, String> reData = new HashMap<String, String>();
        HashMap<String, Object> logmap = new HashMap<String, Object>();
        long starttime = System.currentTimeMillis();
        logmap.put("data", _data);
        String resultcode = "5000";
        String errorinfo = "无处理结果";
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
            String currtime = df.format(new Date());//当前时间
            String invokestarttime = _invokeConf.get("EXTEND1");

            //判断当前时间是否在活动时间内
            if (!StringUtils.isEmpty(invokestarttime) && invokestarttime.length() == 14
                    && (Long.parseLong(currtime) < Long.parseLong(invokestarttime)
            )) {
                throw new VideoException("6003", "当前时间活动还未开启", new String[0]);
            }
            //券码有效期：发放日后的1个月，按照31天计算
            /***
             STARTTIME 取ORDERTIME，只要年月日，格式为YYYY-MM-DD
             ENDTIME 按ORDERTIME，计算发放后的一个月，格式为YYYY-MM-DD
             ORDERTIME必须在活动配置表的STARTTIME和ENDTIME之间
             COUPONMONTH取ORDERTIME的年月值，格式为YYYYMM
             COUPONID取tb_acproduct_config的ID
             */
            HashMap<String, String> params = new HashMap<String, String>();
            params.put("CHSYNCWAITID", _data.get("ID"));
            params.put("ORDERID", _data.get("RELID"));
            params.put("USERID", _data.get("USERID"));
            params.put("SPID", _data.get("SPID"));
            String ordertime = _data.get("ORDERTIME");
            String st = ordertime.substring(0, 4) + "-" + ordertime.substring(4, 6) + "-" + ordertime.substring(6, 8);
            params.put("STARTTIME", st);
            String oneMonthDay = getOneMonthDay(st);

            params.put("ENDTIME", oneMonthDay);
            params.put("COUPONMONTH", ordertime.substring(0, 6));

            String couponid = "";
            String channellist = "";
            HashMap confMap = null;
            //获取活动配置
            List<HashMap> confData = handleDao.qryAcproductConf(_invokeConf.get("URL"));
            if (confData != null && confData.size() > 0) {
                for (HashMap hm : confData) {
                    //获取配置
                    if (hm.get("SPID").equals(_data.get("SPID")) && hm.get("ACTIVITYCODE").equals(_invokeConf.get("URL"))) {
                        confMap = hm;
                        break;
                    }
                }
            }
            //有配置数据
            if (confMap != null) {
                couponid = confMap.get("ID").toString();
           /*     String activitystarttime = confMap.get("STARTTIME").toString();
                String activityendtime = confMap.get("ENDTIME").toString();
                //订购时间小于活动开始时间，或者订购时间大于活动结束时间
                if (Long.parseLong(ordertime) < Long.parseLong(activitystarttime)
                        || Long.parseLong(ordertime) > Long.parseLong(activityendtime)) {
                    throw new VideoException("6001", "订购时间未在活动期内", new String[0]);
                }*/
                //获得channel（子渠道）
                channellist = confMap.get("CHANNEL") == null ? "" : confMap.get("CHANNEL").toString();
            } else {
                throw new VideoException("99999", "TB_ACPRODUCT_CONFIG无配置", new String[0]);
            }

            String spid = _data.get("SPID");
            String relid = _data.get("RELID");
            HashMap relmap = handleDao.getOrderData(relid);
            String orderchannel2 = "";
            boolean flag = false;//标记是否是该渠道能否生成券码
            if (relmap != null && relmap.size() > 0 && !"".equals(channellist)) {
                orderchannel2 = relmap.get("ORDERCHANNEL2") == null ? "" : relmap.get("ORDERCHANNEL2").toString();
                //将从配置表里获取的限制渠道subchannel通过','分割
                String[] limitchannel = channellist.split(",");
                for (String s : limitchannel) {
                    if (orderchannel2.equals(s)) {
                        flag = true;
                        break;
                    }
                }
            }
            logger.info(flag);
            if (!flag) {
                throw new VideoException("6004", "此spid=" + spid + " 渠道=" + orderchannel2 + " 不参与【千人千面】券码活动", new String[0]);
            }

            params.put("COUPONTYPE", confMap.get("COUPONTYPE") + "");
            params.put("COUPONID", couponid);
            params.put("ACTIVITYCODE", confMap.get("ACTIVITYCODE") + "");

            logmap.put("params", params);
            // 半年内只允许领取一次5元话费券。半年的计算依据以订购时间作判断
            // 查询半年内是否有话费券记录，只取最近的一条数据
            HashMap<String, Object> hashMap = handleDao.qryActiveDataByCancel(params);
            if (hashMap == null) {
                // 没有券码记录
                handleDao.addActiveDataByCancel(params);
            } else {
                // 判断是否领取
                long currentTime = new Date().getTime();   // 当前时间
                // 1 已兑换
                if ("1".equals(hashMap.get("STATUS"))) {
                    String couponmonth = hashMap.get("COUPONMONTH").toString();
                    String afterMonth = getAfterMonth(couponmonth, 5);   // 券码领取时间的半年后
                    Integer orderTime = Integer.parseInt(ordertime.substring(0, 6));
                    // 判断订购时间与券码领取时间是否相差半年
                    if (orderTime > Integer.parseInt(afterMonth)) {
                        handleDao.addActiveDataByCancel(params);
                    } else {
                        throw new VideoException("6002", "此spid=" + spid + "号码=" + _data.get("USERID") + "半年内已领取过券码，不再下发");
                    }

                } else {
                    // 判断该券码目前是否还在有效期内，不在则下发
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    String endtime = hashMap.get("ENDTIME").toString();
                    long time = sdf.parse(endtime).getTime();   // 有效期
                    SimpleDateFormat f = new SimpleDateFormat("yyyyMMddHHmmss");
                    long orderTime = f.parse(ordertime).getTime();
                    if (orderTime > time) {
                        // 不在
                        handleDao.addActiveDataByCancel(params);
                    } else {
                        // 在有效期内，原券码设置为作废   5作废
                        HashMap<String, Object> map = new HashMap<>();
                        map.put("ID", hashMap.get("ID"));
                        map.put("STATUS", "5");
                        handleDao.updateActiveDataCancel(map);
                        // 重新下发一条
                        handleDao.addActiveDataByCancel(params);
                    }
                }
            }


            resultcode = "0";
            errorinfo = "ok";
            String smssendres = this.sendsms(_data.get("USERID"), oneMonthDay, confMap);
            errorinfo += "，smssend:" + smssendres;
            logmap.put("smssendres", smssendres);
        } catch (Exception e) {
            if (e instanceof VideoException) {
                resultcode = ((VideoException) e).getErrorcode();
                errorinfo = e.getMessage() + errorinfo;
            } else {
                e.printStackTrace();
                logger.error("处理数据出现异常，" + e.getMessage(), e);
                resultcode = "8000";
                errorinfo = "处理数据出现异常，" + e.getMessage() + errorinfo;
            }
        } finally {
            logmap.put("resultcode", resultcode);
            logmap.put("errorinfo", errorinfo);
            logmap.put("times", System.currentTimeMillis() - starttime);
            logger.info(logmap);
        }

        reData.put("resultcode", resultcode);
        reData.put("errorinfo", errorinfo);

        return reData;
    }

    /**
     * 获取指定日期的后一个月
     *
     * @param day YYYY-MM-DD
     * @return 返回格式：YYYY-MM-DD
     */
    private String getOneMonthDay(String day) {
        try {
            Calendar cal = Calendar.getInstance();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            cal.setTime(sdf.parse(day));
            cal.add(Calendar.DATE, 31); // 向后31天，用正数即可，向前用负数
            return sdf.format(cal.getTime());
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }


    public static String getAfterMonth(String inputDate, int number) {
        Calendar c = Calendar.getInstance();//获得一个日历的实例
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
        Date date = null;//初始日期
        try {
            date = sdf.parse(inputDate);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info("日期转换出错" + e.getMessage() + e);
        }
        c.setTime(date);//设置日历时间
        c.add(Calendar.MONTH, number);//在日历的月份上增加6个月
        return sdf.format(c.getTime());
    }


    /**
     * 发送短信
     * <p>
     * 202208170956014412   mm,dd,spname,value,expdate
     * 【联通畅视】您于{$mm}月{$dd}日订购的{$spname}已办理成功。
     * 送您{$value}元话费，请点击 https://v.mige.tv/yzhl318 领取，
     * 或者前往微信小程序“联通畅视”-“我的”-“畅视礼券”领取，有效期至{$expdate}。
     */
    private String sendsms(String userid, String oneMonthDay, HashMap confMap) {
        String result = ""; //调接口返回信息
        Calendar now = Calendar.getInstance();
        String month = now.get(Calendar.MONTH) + 1 + "";    // 当前
        String day = now.get(Calendar.DAY_OF_MONTH) + "";
        SimpleDateFormat sdf = new SimpleDateFormat("M月d日");
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd");
        String expdate = null;
        try {
            expdate = sdf.format(f.parse(oneMonthDay));
        } catch (ParseException e) {
            e.printStackTrace();
            expdate = " ";
        }
        try {
            Map<String, Object> map = new HashMap<>();
            map.put("mm", month);
            map.put("dd", day);
            map.put("spname", confMap.get("SPNAME"));
            Integer value = Double.valueOf(confMap.get("AMOUNT").toString()).intValue();
            map.put("value", value);
            map.put("expdate", expdate);
            String contentid = "202208170956014412";
            String smsid = kafkaInfoDao.getDictionaryValue("cancelBackSmsid");
            String pwd = kafkaInfoDao.getDictionaryValue("cancelBackPwd");
            result = SendSmsUtil.sendSMSWithVars(userid, contentid, smsid, pwd, map);
        } catch (Exception e) {
            result = "短信发送出现异常，" + e.getMessage();
        }
        return result;
    }
}
