package com.chinacreator.kafka.sync.handle.impl;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.druid.util.StringUtils;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.mapper.PrevAutoCancelDao;
import com.chinacreator.kafka.sync.handle.IHandleService;

/**
 * 前向产品到期自动退订
 * 河南版随心选
 * @author xu.zhou
 * @date 20220722
 业务处理逻辑：
1. 通过渠道同步获取数据（订购和退订）
1.1 订购：
1.1.1 获取渠道同步的订购数据，用RELID（订购关系表的ID）判断记录表是否已有数据：
1.1.1.1 无，则直接添加，AUTOCANCELTIME设置为ORDERTIME+365天(365是配置)，RESULTCODE设置为1（待处理）。
1.1.1.2 有，则判断记录表的ENDTIME是否为空，
					ENDTIME为空代表未退订，AUTOCANCELTIME设置为AUTOCANCELTIME+365天（累加），更新ORDERTIME，ORDERCOUNT+1，RESULTCODE设置为1（待处理）。
					ENDTIME不为空则代表已退订，AUTOCANCELTIME设置为ENDTIME+365天，更新ORDERTIME，清空CANCELTIME和ENDTIME，ORDERCOUNT+1，RESULTCODE设置为1（待处理）。
1.2 退订：
1.2.1 获取渠道同步的订购数据，用RELID（订购关系表的ID）判断记录表是否已有数据：
1.2.1.1 无，则直接添加，设置ORDERTIME, CANCELTIME, ENDTIME, AUTOCANCELTIME设置为空，RESULTCODE设置为3（不处理）。
1.2.1.2	有，则判断记录表的ENDTIME是否为空
					ENDTIME不为空，抛出异常（重复退订）
					ENDTIME为空，设置CANCELTIME, ENDTIME		 
 */
@Component("prevAutoCancelHandle")
public class PrevAutoCancelHandleService  implements IHandleService {
	
    private static Logger logger = Logger.getLogger("prevAutoCancel");
    
    @Autowired
    PrevAutoCancelDao prevAutoCancelDao;

    /**
业务处理逻辑：
1. 通过渠道同步获取数据（订购和退订）
1.1 订购：
1.1.1 获取渠道同步的订购数据，用RELID（订购关系表的ID）判断记录表是否已有数据：
1.1.1.1 无，则直接添加，AUTOCANCELTIME设置为ORDERTIME+365天(365是配置)，RESULTCODE设置为1（待处理）。
1.1.1.2 有，则判断记录表的ENDTIME是否为空，
					ENDTIME为空代表未退订，判断AUTOCANCELTIME是否大于当前时间，
						是：AUTOCANCELTIME未过期，AUTOCANCELTIME设置为AUTOCANCELTIME+365天（累加），更新ORDERTIME，ORDERCOUNT+1，RESULTCODE设置为1（待处理）。
						否：AUTOCANCELTIME已过期，AUTOCANCELTIME设置为ORDERTIME+365天（重置），更新ORDERTIME，ORDERCOUNT+1，RESULTCODE设置为1（待处理）。
					ENDTIME不为空则代表已退订，判断ENDTIME是否大于当前时间，
						是，则代表已失效，AUTOCANCELTIME设置为ORDERTIME+365天，
						否，则代表未失败，AUTOCANCELTIME设置为ENDTIME+365天，
						最后更新ORDERTIME，清空CANCELTIME和ENDTIME，ORDERCOUNT+1，RESULTCODE设置为1（待处理）。
1.2 退订：
1.2.1 获取渠道同步的订购数据，用RELID（订购关系表的ID）判断记录表是否已有数据：
1.2.1.1 无，则直接添加，设置ORDERTIME, CANCELTIME, ENDTIME, 不设置AUTOCANCELTIME、RESULTCODE
1.2.1.2	有，则判断记录表的ENDTIME是否为空
					ENDTIME不为空，抛出异常（重复退订）
					ENDTIME为空，设置CANCELTIME, ENDTIME, 不设置AUTOCANCELTIME、RESULTCODE	
     */
	@Override
	public HashMap<String, String> handle(HashMap<String, String> _data,
			HashMap<String, String> _invokeConf) throws Exception {
        HashMap<String, String> reData = new HashMap<String, String>();
        HashMap<String, Object> logmap = new HashMap<String, Object>();
        long starttime = System.currentTimeMillis();
        logmap.put("data", _data);
        String resultcode = "5000";
        String errorinfo = "";
        HashMap<String, String> params = new HashMap<String, String>();
        try {
        	//RELID, USERID, CPID, SPID, ORDERTIME, CANCELTIME, ENDTIME
        	params.put("WAITID", _data.get("ID"));
        	params.put("RELID", _data.get("RELID"));
        	params.put("USERID", _data.get("USERID"));
        	params.put("CPID", _data.get("CPID"));
        	params.put("SPID", _data.get("SPID"));
        	params.put("ORDERTIME", _data.get("ORDERTIME"));
        	params.put("CANCELTIME", _data.get("CANCELTIME"));
        	params.put("ENDTIME", _data.get("ENDTIME"));
        	
        	//自动退订时间
        	String authCancelTime = "";
        	String spid = _data.get("SPID");
        	String relid = _data.get("RELID");
        	String ordertype = _data.get("ORDERTYPE");
        	//业务处理逻辑
        	HashMap confMap = prevAutoCancelDao.qryAutocancelConf(spid);
        	if(confMap == null){
        		throw new VideoException("99999", "TB_PREV_AUTOCANCEL_CONF无配置", new String[0]);
        	}
       	
      		//先添加数据到日志表，一个WAITID只允许添加一条数据，添加成功表示可以执行
      		prevAutoCancelDao.addRecLog(params);
        	
        	//获取已有记录表数据
        	HashMap recMap = prevAutoCancelDao.qryRecByRelid(relid);
        	if("0".equals(ordertype)){//订购
        		authCancelTime = this.getAutoCancelTime(confMap, recMap, _data);
          		params.put("AUTOCANCELTIME", authCancelTime);
          		
            	if(recMap == null){//没记录    
            		prevAutoCancelDao.addRecOrder(params);
            	}else{//已有记录
            		params.put("ID", recMap.get("ID").toString());
            		params.put("RESULTCODE", "1");
            		prevAutoCancelDao.updRecOrder(params);
            	}   
        	}else{//退订
            	if(recMap == null){//没记录
            		prevAutoCancelDao.addRecCancel(params);
            	}else{//已有记录
            		String endtimeRec = (String)recMap.get("ENDTIME");
            		String ordertimeRec = (String)recMap.get("ORDERTIME");
            		String canceltime = _data.get("CANCELTIME");
            		//记录表退订时间不为空，已退订
            		if(!StringUtils.isEmpty(endtimeRec)){
            			throw new VideoException("9001", "重复退订", new String[0]);
            		}
            		//此次退订时间小于记录表的订购时间
            		if(Long.parseLong(canceltime) < Long.parseLong(ordertimeRec)){
            			throw new VideoException("9002", "退订时间小于记录表的订购时间", new String[0]);
            		}
            		
            		params.put("ID", recMap.get("ID").toString());
                	//更新记录表为退订	
            		prevAutoCancelDao.updRecCancel(params);
            	}   
        	}
        	
        	resultcode = "0";
        	errorinfo = "OK";
        	
      		//更新执行结果
        	params.put("RESULTCODE", resultcode);
      		prevAutoCancelDao.updRecLog(params);
        } catch (Exception e) {
            if (e instanceof VideoException) {
                resultcode = ((VideoException) e).getErrorcode();
                errorinfo = ((VideoException) e).getMessage() + errorinfo;
            } else {
                e.printStackTrace();
                logger.error("处理数据出现异常，" + e.getMessage(), e);
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
                    resultcode = "6002";
                    errorinfo = "重复数据，添加失败";
                } else {
                    resultcode = "8000";
                    errorinfo = "处理数据出现异常，" + e.getMessage() + errorinfo;
                }
            }
        } finally {
        	logmap.put("params", params);
            logmap.put("resultcode", resultcode);
            logmap.put("errorinfo", errorinfo);
            logmap.put("times", System.currentTimeMillis() - starttime);
            logger.info(logmap);
        }

        reData.put("resultcode", resultcode);
        reData.put("errorinfo", errorinfo);

        return reData;
	}
	
	/**		 
	 * 
	 * 获取自动退订时间
	 * @param confMap
	 * @param recMap
	 * @param _data
	 * @return
	 * @throws Exception
	 */
	private String getAutoCancelTime(HashMap confMap,  HashMap recMap, HashMap<String, String> _data) throws Exception{
		//自动退订时间，格式yyyyMMddHHmmss
		String authCancelTime = "";
    	//有效期值，对应periodtype
    	int periodval = Integer.parseInt(confMap.get("PERIODVAL").toString());
    	//有效期类型， 有效期类型，MONTH月，DAY日
    	String periodtype = (String)confMap.get("PERIODTYPE");
    	//是否包含当前天/当前月
    	String hascurr = (String)confMap.get("HASCURR");
    	//是否自然日/自然月，0是，1否
    	String hasture = (String)confMap.get("HASNATURE");
    	//订购时间
    	String ordertime = _data.get("ORDERTIME"); 
    	//0订购，1退订
    	String ordertype = _data.get("ORDERTYPE"); 
		SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
		//当前时间
		String currtime = format.format(new Date());
    	if("0".equals(ordertype)){//订购
        	if(recMap == null){//新数据，自动退订记录表无数据，AUTOCANCELTIME设置为ORDERTIME+365天(365是配置)
        		authCancelTime = this.calcuAutoCancelTime(periodtype, periodval, ordertime, hascurr, hasture);
        	}else{//有数据
        		String endtime = (String)recMap.get("ENDTIME");
        		String act = (String)recMap.get("AUTOCANCELTIME");
        		if(StringUtils.isEmpty(endtime)){//ENDTIME为空代表未退订，判断AUTOCANCELTIME是否大于当前时间，
        			if(Long.parseLong(act) >= Long.parseLong(currtime)){//AUTOCANCELTIME未过期，AUTOCANCELTIME设置为AUTOCANCELTIME+365天（累加），
            			authCancelTime = this.calcuAutoCancelTime(periodtype, periodval, act, hascurr, hasture);
        			}else{//AUTOCANCELTIME已过期，AUTOCANCELTIME设置为ORDERTIME+365天（重置）
            			authCancelTime = this.calcuAutoCancelTime(periodtype, periodval, ordertime, hascurr, hasture);
        			}
        		}else{
        			//判断ENDTIME是否大于当前时间
        			if(Long.parseLong(endtime) >= Long.parseLong(currtime)){//未失效，AUTOCANCELTIME设置为ENDTIME+365天，
        				authCancelTime = this.calcuAutoCancelTime(periodtype, periodval, endtime, hascurr, hasture);
        			}else{//已失效，AUTOCANCELTIME设置为ORDERTIME+365天，
        				authCancelTime = this.calcuAutoCancelTime(periodtype, periodval, ordertime, hascurr, hasture);
        			}
        		}
        	}
    	}
    	
		return authCancelTime;
	}
	
	/**
	 * 
字段：
HASCURR 计算自动退订时间（AUTOCANCELTIME）是否包含当天或当月，包含0，不包含1， 当HASNATURE为0时有效
HASNATURE 是否自然日或自然月，0是，1否。有效期类型（PERIODTYPE）为MONTH月时，是否为自然月（从1号到月末最后一天），为DAY，是否为自然日（算从0点到23点为一天）
不为自然日（从当前时间加24小时算一天），不为自然月（从当前时间加几个月）
示例：
PERIODVAL: 1
PERIODTYPE: DAY
HASCURR：0 （包含当前日）
HASNATURE：0 （自然日）
当前时间：2022-07-26 09:43:25
以上配置说明：有效期为1，单位是天，包含当天时间，按自然天算，
结果：自动退订时间AUTOCANCELTIME，为当天的23时59分59秒（2022-07-26 23:59:59）

配置改为HASCURR：1 （不包含当前日）
结果：自动退订时间AUTOCANCELTIME，为第二天的23时59分59秒（2022-07-27 23:59:59）

配置改为HASNATURE：1 （非自然日，相当于当前时间加24小时）
不用管HASCURR的配置
结果：自动退订时间AUTOCANCELTIME，为第二天的23时59分59秒（2022-07-27 09:43:25）	 
	 * 
	 * 计算自动退订时间
	 * @param periodtype 类型，MONTH月，DAY日
	 * @param starttime 起始时间
	 * @param periodval 值，对应periodtype
	 * @param hascurr，是否包含当天/当月，0是，1否
	 * @param hasture 是否自然日/自然月，0是，1否
	 * @return
	 * @throws Exception
	 */
	public String calcuAutoCancelTime(String periodtype, int periodval, String starttime , String hascurr, String hasture) throws Exception {
	    if(periodval <= 0){
	    	return starttime;
	    }
		SimpleDateFormat showsdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
	    if(starttime == null) starttime = sdf.format(new Date());
	    Date currdate = sdf.parse(starttime);
	    Calendar cal = Calendar.getInstance();
	    cal.setTime(currdate);
	    if("DAY".equals(periodtype)){
	    	if("0".equals(hascurr) && "0".equals(hasture)){//包含当天，按自然日算，结束时间为当天的最后一秒
	    		if(periodval == 1){
	    			cal.add(Calendar.DATE, 0);
	    		}else{
	    			cal.add(Calendar.DATE, periodval - 1); 
	    		}
	    		cal.set(Calendar.HOUR_OF_DAY, 23);
				// 分
	    		cal.set(Calendar.MINUTE, 59);
				// 秒
	    		cal.set(Calendar.SECOND, 59);
	    	}else if("1".equals(hascurr) && "0".equals(hasture)){//不包含当前，按自然日算，加1天，为第二天23时59分59秒
	    		cal.add(Calendar.DATE, periodval);
	    		cal.set(Calendar.HOUR_OF_DAY, 23);
				// 分
	    		cal.set(Calendar.MINUTE, 59);
				// 秒
	    		cal.set(Calendar.SECOND, 59);
	    	}else if("1".equals(hasture)){//不按自然日算，不用管是否包含当前天
	    		cal.add(Calendar.DATE, periodval);
	    	}
	    }else{
	    	if("0".equals(hascurr) && "0".equals(hasture)){//包含当月，按自然月算，结束时间为当月最后一天的最后一秒
	    		if(periodval == 1){
	    			cal.add(Calendar.MONTH, 0);
	    		}else{
	    			cal.add(Calendar.MONTH, periodval - 1);
	    		}
	    		cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
	    		cal.set(Calendar.HOUR_OF_DAY, 23);
				// 分
	    		cal.set(Calendar.MINUTE, 59);
				// 秒
	    		cal.set(Calendar.SECOND, 59);
	    	}else if("1".equals(hascurr) && "0".equals(hasture)){//不包含当前月，按自然月算，加1月，为第二月最后一天23时59分59秒
	    		cal.add(Calendar.MONTH, periodval);
	    		cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
	    		cal.set(Calendar.HOUR_OF_DAY, 23);
				// 分
	    		cal.set(Calendar.MINUTE, 59);
				// 秒
	    		cal.set(Calendar.SECOND, 59);
	    	}else if("1".equals(hasture)){//不按自然月算，不用管是否包含当前月
	    		cal.add(Calendar.MONTH, periodval);
	    	} 
	    }

	    Date enddate = cal.getTime();
	    String endtime = sdf.format(enddate);
	    
	    logger.info("初始的时间是：" + showsdf.format(currdate)+"，hascurr=>"+hascurr+"，hasture=>"+hasture+"，增加["+periodval+"]"+("MONTH".equals(periodtype)?"月":"日")+"以后的时间：" + showsdf.format(enddate));
	    
	    return endtime;
	}
}
