package com.chinacreator.kafka.sync.handle.impl;

import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.mapper.ActivityReflowDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SendSmsUtil;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 【回流渠道赠送券码】活动
 *
 * @author zhengrong.yan
 * @date 20220322
 */
@Component("reflowActivityLevel3")
public class ReFlowActivityLevel3HandleService implements IHandleService {

    private static Logger logger = Logger.getLogger("reflowActivityLevel3Rec");

    @Autowired
    ActivityHandleDao handleDao;
    @Autowired
    ActivityReflowDao activityReflowDao;

    @Override
    public HashMap<String, String> handle(HashMap<String, String> _data, HashMap<String, String> _invokeConf) throws Exception {
        HashMap<String, String> reData = new HashMap<String, String>();
        HashMap<String, Object> logmap = new HashMap<String, Object>();
        long starttime = System.currentTimeMillis();
        logmap.put("data", _data);
        String resultcode = "5000";
        String errorinfo = "无处理结果";
        try {
            //20220427 判断是否已发过回流短信，从回流短信渠道来的，防止盗刷
            int reccount = activityReflowDao.qrySmsRecHis(_data.get("SPID"),_data.get("USERID"),_invokeConf.get("URL"));
            //20220809 版本回退 取消此限制
            //int reccountWH = activityReflowDao.qrySmsRecHisWH(_data.get("SPID"),_data.get("USERID"),"threeReflowBack");
            if(reccount == 0) {//没有推送短信
                throw new VideoException("6005", "此号码未发送过三级促回流短信，不属于活动用户", new String[0]);
            }
            SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
            String currtime = df.format(new Date());//当前时间
            String invokestarttime = _invokeConf.get("EXTEND1");
            String invokeendtime = _invokeConf.get("EXTEND2");
            //判断当前时间是否在活动时间内
            if (!StringUtils.isEmpty(invokestarttime) && !StringUtils.isEmpty(invokeendtime)
                    && invokestarttime.length() == 14 && invokeendtime.length() == 14
                    && (Long.parseLong(currtime) < Long.parseLong(invokestarttime)
                    || Long.parseLong(currtime) > Long.parseLong(invokeendtime))) {
                throw new VideoException("6003", "当前时间未在活动期内", new String[0]);
            }
            //券码有效期：发放日后的两周
            /***
             STARTTIME 取ORDERTIME，只要年月日，格式为YYYY-MM-DD
             ENDTIME 按ORDERTIME，计算发放后的两周，格式为YYYY-MM-DD
             ORDERTIME必须在活动配置表的STARTTIME和ENDTIME之间
             COUPONMONTH取ORDERTIME的年月值，格式为YYYYMM
             COUPONID取tb_acproduct_config的ID
             */
            HashMap<String, String> params = new HashMap<String, String>();
            params.put("CHSYNCWAITID", _data.get("ID"));
            params.put("ORDERID", _data.get("RELID"));
            params.put("USERID", _data.get("USERID"));
            params.put("SPID", _data.get("SPID"));
            String ordertime = _data.get("ORDERTIME");
            String st = ordertime.substring(0, 4) + "-" + ordertime.substring(4, 6) + "-" + ordertime.substring(6, 8);
            params.put("STARTTIME", st);
            params.put("ENDTIME", getTwoWeekDay(st));
            params.put("COUPONMONTH", ordertime.substring(0, 6));

            String couponid = "";
            String channellist = "";
            String smscontent = "";
            HashMap confMap = null;
            //获取活动配置
            List<HashMap> confData = handleDao.qryAcproductConf(_invokeConf.get("URL"));
            if (confData != null && confData.size() > 0) {
                for (HashMap hm : confData) {
                    //获取配置
                    if (hm.get("SPID").equals(_data.get("SPID")) && hm.get("ACTIVITYCODE").equals(_invokeConf.get("URL"))) {
                        confMap = hm;
                        break;
                    }
                }
            }
            //有配置数据
            if (confMap != null) {
                couponid = confMap.get("ID").toString();
                String activitystarttime = confMap.get("STARTTIME").toString();
                String activityendtime = confMap.get("ENDTIME").toString();
                //订购时间小于活动开始时间，或者订购时间大于活动结束时间
                if (Long.parseLong(ordertime) < Long.parseLong(activitystarttime)
                        || Long.parseLong(ordertime) > Long.parseLong(activityendtime)) {
                    throw new VideoException("6001", "订购时间未在活动期内", new String[0]);
                }
                //获得channel（子渠道）
                channellist = confMap.get("CHANNEL") == null ? "" : confMap.get("CHANNEL").toString();
                //获得短信内容
                smscontent = confMap.get("EXTEND1") == null ? "" : confMap.get("EXTEND1").toString();
            } else {
                throw new VideoException("99999", "TB_ACPRODUCT_CONFIG无配置", new String[0]);
            }

            String spid = _data.get("SPID");
            String relid = _data.get("RELID");
            HashMap relmap = handleDao.getOrderData(relid);
            String orderchannel2 = "";
            boolean flag = false;//标记是否是该渠道能否生成券码
            if (relmap != null && relmap.size() > 0 && !"".equals(channellist)) {
                orderchannel2 = relmap.get("ORDERCHANNEL2") == null ? "" : relmap.get("ORDERCHANNEL2").toString();
                //将从配置表里获取的限制渠道subchannel通过','分割
                String[] limitchannel = channellist.split(",");
                for (String s : limitchannel) {
                    if (orderchannel2.equals(s)) {
                        flag = true;
                        break;
                    }
                }
            }
            logger.info(flag);
            if (!flag) {
                throw new VideoException("6004", "此spid=" + spid + " 渠道=" + orderchannel2 + " 不参与【三级渠道促回流】券码活动", new String[0]);
            }

            params.put("COUPONTYPE", confMap.get("COUPONTYPE") + "");
            params.put("COUPONID", couponid);
            params.put("ACTIVITYCODE", confMap.get("ACTIVITYCODE") + "");

            logmap.put("params", params);
            //同一号码、同一产品、只能添加一条数据
            handleDao.addActiveDataByReFlow(params);

            resultcode = "0";
            errorinfo = "ok";
            String smssendres = this.sendsms(_data, smscontent,st);
            errorinfo += "，smssend:" + smssendres;
            logmap.put("smssendres", smssendres);
        } catch (Exception e) {
            if (e instanceof VideoException) {
                resultcode = ((VideoException) e).getErrorcode();
                errorinfo = ((VideoException) e).getMessage() + errorinfo;
            } else {
                e.printStackTrace();
                logger.error("处理数据出现异常，" + e.getMessage(), e);
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
                    resultcode = "6002";
                    errorinfo = "此号码和产品已参与过活动，数据已添加过表";
                } else {
                    resultcode = "8000";
                    errorinfo = "处理数据出现异常，" + e.getMessage() + errorinfo;
                }
            }
        } finally {
            logmap.put("resultcode", resultcode);
            logmap.put("errorinfo", errorinfo);
            logmap.put("times", System.currentTimeMillis() - starttime);
            logger.info(logmap);
        }

        reData.put("resultcode", resultcode);
        reData.put("errorinfo", errorinfo);

        return reData;
    }

    /**
     * 获取指定日期的后两周
     *
     * @param day YYYY-MM-DD
     * @return 返回格式：YYYY-MM-DD
     */
    private String getTwoWeekDay(String day) {
        try {
            Calendar cal = Calendar.getInstance();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            cal.setTime(sdf.parse(day));
            cal.add(Calendar.DATE, 14); // 向前一周；如果需要向后一周，用正数即可
            return sdf.format(cal.getTime());
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 发送短信
     */
    private String sendsms(HashMap<String, String> _data, String smscontent, String starttime) {
        String result = ""; //调接口返回信息
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar now = Calendar.getInstance();
        String month = now.get(Calendar.MONTH) + 1 + "";
        String day = now.get(Calendar.DAY_OF_MONTH) + "";
        try {
            now.setTime(sdf.parse(starttime));
            now.add(Calendar.DATE, 14); // 向前一周；如果需要向后一周，用正数即可
            String endmonth = now.get(Calendar.MONTH) + 1 + "";
            String endday = now.get(Calendar.DAY_OF_MONTH) + "";

            String userid = _data.get("USERID");
            if (!StringUtils.isEmpty(smscontent) && userid != null && userid.length() == 11) {//有短信内容匹配
                smscontent = smscontent.replace("#USERID#", userid);
                smscontent = smscontent.replace("#MONTH#", month);
                smscontent = smscontent.replace("#DAY#", day);
                smscontent = smscontent.replace("#ENDMONTH#", endmonth);
                smscontent = smscontent.replace("#ENDDAY#", endday);
                result = SendSmsUtil.send(userid, smscontent);
            } else {
                result = "无短信配置";
            }
        } catch (Exception e) {
            result = "短信发送出现异常，" + e.getMessage();
        } finally {

        }

        return result;
    }
}
