package com.chinacreator.kafka.sync.handle.impl;

import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SendSmsUtil;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 【517电信券码】活动
 * @author zhengrong.yan
 * @date 20220505
 */
/**
科创规则：
		1、快手12G流量月包省分spid= 297和畅视多视频共享15G流量月包省分 spid= 1218，订购当月未退订者，用户将会在次月可以领取相应的话费。比如：5月10号订购，20号退订，次月不能返话费。5月20号订购，6月3号退订，次月能返话费。
		6月订购的7月份可以领取话费
		5月份订购，6月份续订 ， 7月份续订 一共可以领1次 话费。注：活动期间话费一个产品用户只返一次
		补充：用户在活动时间内退订再订购的用户可以参加。如果5月份订购，6月份退订了领取话费，6月份又订购了，7月份就不能领取了
		话费领取时间为次月1号到月底最后一天，订购次月领取当月有效。
 **/
@Component("telecomActivity")
public class TelecomActivityHandleService implements IHandleService{
	
	private static Logger logger = Logger.getLogger("telecomActivityrec");
	
	@Autowired
	ActivityHandleDao handleDao;

	@Override
	public HashMap<String, String> handle(HashMap<String, String> _data, HashMap<String, String> _invokeConf) throws Exception {
		HashMap<String, String> reData = new HashMap<String, String>();
		HashMap<String, Object> logmap = new HashMap<String, Object>();
		long starttime = System.currentTimeMillis();
		logmap.put("data", _data);
		String resultcode = "5000";
		String errorinfo = "无处理结果";
		String couponmonth = "";
		String spid = "";
		HashMap<String, String> params = new HashMap<String, String>();
		try {
			SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
			String currtime = df.format(new Date());//当前时间
			String invokestarttime = _invokeConf.get("EXTEND1");
			String invokeendtime = _invokeConf.get("EXTEND2");
			//判断当前时间是否在活动时间内
			if(!StringUtils.isEmpty(invokestarttime) && !StringUtils.isEmpty(invokeendtime)
					&& invokestarttime.length() == 14 && invokeendtime.length() == 14
					&& (Long.parseLong(currtime) < Long.parseLong(invokestarttime)
							|| Long.parseLong(currtime) > Long.parseLong(invokeendtime))){
				throw new VideoException("6003", "当前时间未在活动期内", new String[0]);
			}
			/***
			STARTTIME 取ORDERTIME，只要年月日，格式为YYYY-MM-DD
			ENDTIME 按ORDERTIME，取当月最后一天，也是年月日，格式为YYYY-MM-DD
			ORDERTIME必须在活动配置表的STARTTIME和ENDTIME之间
			COUPONMONTH取ORDERTIME的年月值，格式为YYYYMM
			COUPONID取tb_acproduct_config的ID
			 */
			params.put("CHSYNCWAITID", _data.get("ID"));
			params.put("ORDERID", _data.get("RELID"));
			params.put("USERID", _data.get("USERID"));
			params.put("SPID", _data.get("SPID"));
			String ordertime = _data.get("ORDERTIME");
			//此活动是订购后发放下月的券码，且时间为下月1号到最后一天
			couponmonth = ordertime.substring(0,6);
			couponmonth = getNextMonth(couponmonth);
			params.put("COUPONMONTH",couponmonth);
			String st = couponmonth.substring(0,4)+"-"+couponmonth.substring(4,6)+"-01";
			params.put("STARTTIME", st);
			params.put("ENDTIME", getLastDayOfMonth(couponmonth.substring(0,4)+"-"+couponmonth.substring(4,6)));

			String couponid = "";
			String channellist = "";
			HashMap confMap = null;
			//获取活动配置
			List<HashMap> confData = handleDao.qryAcproductConf(_invokeConf.get("URL"));
			if(confData != null && confData.size()>0){
				for(HashMap hm : confData){
					//获取配置
					if(hm.get("SPID").equals(_data.get("SPID")) && hm.get("ACTIVITYCODE").equals(_invokeConf.get("URL"))){
						//20220831 修改 当订购来的时间为9.1日前，则均属于五元话费活动，9.1（含）之后的均为第二批次
						String activitystarttime = hm.get("STARTTIME").toString();
						String activityendtime = hm.get("ENDTIME").toString();
						//订购时间小于活动开始时间，或者订购时间大于活动结束时间
						logger.info("ordertiem==>"+ordertime);
						if(Long.parseLong(ordertime) >= Long.parseLong(activitystarttime)
								&& Long.parseLong(ordertime) <= Long.parseLong(activityendtime)){
							confMap = hm;
							logger.info("使用的券码配置>>>>"+hm.get("ID"));
							break;
						}
					}
				}			
			}
			//有配置数据	
			if(confMap != null){
				couponid = confMap.get("ID").toString();
				String activitystarttime = confMap.get("STARTTIME").toString();
				String activityendtime = confMap.get("ENDTIME").toString();
				//订购时间小于活动开始时间，或者订购时间大于活动结束时间
				if(Long.parseLong(ordertime) < Long.parseLong(activitystarttime)
						|| Long.parseLong(ordertime) > Long.parseLong(activityendtime)){
					throw new VideoException("6001", "订购时间未在活动期内", new String[0]);
				}
				//获得channel（子渠道）
				channellist = confMap.get("CHANNEL") == null ? "" : confMap.get("CHANNEL").toString();
			}else{
				throw new VideoException("99999", "TB_ACPRODUCT_CONFIG无配置", new String[0]);
			}

			spid = _data.get("SPID");
			String relid = _data.get("RELID");
			HashMap relmap = handleDao.getOrderData(relid);
			String orderchannel2 = "";
			/**
			 *  活动范围，只针对伯源的两个二级渠道 5月10日-6月30日，
			 *  快手12G流量月包省分           spid= 297 	渠道编码：192908129979
			 *  畅视多视频共享15G流量月包省分 spid= 1218  渠道编码：192908845733
			 */
			boolean flag = false;//标记是否是该渠道能否生成券码
			if (relmap != null && relmap.size() > 0 && !"".equals(channellist)) {
				orderchannel2 = relmap.get("ORDERCHANNEL2") == null ? "" : relmap.get("ORDERCHANNEL2").toString();
				//将从配置表里获取的限制渠道subchannel通过','分割
				String[] limitchannel = channellist.split(",");
				for (String s : limitchannel) {
					if (orderchannel2.equals(s)) {
						flag = true;
						break;
					}
				}
			}else{
				flag = true;
			}
			logger.info(flag);
			if (!flag) {
				throw new VideoException("6004", "此spid=" + spid + " 渠道=" + orderchannel2 + " 不参与【517电信】券码活动", new String[0]);
			}

			params.put("COUPONTYPE", confMap.get("COUPONTYPE")+"");
			params.put("COUPONID", couponid);
			params.put("ACTIVITYCODE", confMap.get("ACTIVITYCODE")+"");

			//20220627 修改 当订购来的时间为7.1日前，则均属于517活动第一批次，7.1（含）之后的均为第二批次
			String limittime = "20220701000000";
			logger.info("ordertiem==>"+ordertime);
			if(Long.parseLong(ordertime) < Long.parseLong(limittime)){
				params.put("BATCH", "1");
				logger.info("小于限制时间7.1，则属于第一批次数据");
			}else {
				params.put("BATCH", "2");
				logger.info("等于或大于限制时间7.1，则属于第二批次数据");
			}

			logmap.put("params", params);
			//同一号码、同一产品、只能添加一条数据
			if("0".equals(_data.get("ORDERTYPE"))) {
				handleDao.addActiveDataTelecom(params);
			}else{
				//退订状态进入此类后，先查是否有券码数据 没有的不处理 有则进行状态更新
				HashMap map = handleDao.queryActiveDataTelecom(params);
				String canceltime = _data.get("CANCELTIME");
				String cancelmonth = canceltime.substring(0,6);
				String ordermonth = ordertime.substring(0,6);
				//0628 修改 当退订月份和订购月份在同一个月内触发时，需要更新表数据为无效状态，否则不进行修改
				if(map!=null && "0".equals(map.get("STATUS")) && cancelmonth.equals(ordermonth)){
					map.put("COUPONMONTH",couponmonth);
					map.put("STARTTIME",params.get("STARTTIME"));
					map.put("ENDTIME",params.get("ENDTIME"));
					map.put("STATUS","5");
					handleDao.updateActiveDataTelecom(map);
					throw new VideoException("6002","已添加过数据，已进行更新，更新月份"+couponmonth,new String[0]);
				}else{
					throw new VideoException("6006","spid："+spid+"无数据，退订状态不进行处理或退订月份与订购月份不一致，订购月份:"+ordermonth+" 退订月份："+cancelmonth,new String[0]);
				}
			}

			resultcode = "0";
			errorinfo = "ok";
			//data添加短信内容 20220426
//			_data.put("CONTENT",confMap.get("EXTEND1")==null?"":confMap.get("EXTEND1").toString());
//			String smssendres = this.sendsms(_data);
//			errorinfo += "，smssend:"+smssendres;
//			logmap.put("smssendres", smssendres);
		} catch (Exception e) {
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage()+errorinfo;
			} else {
				e.printStackTrace();
				logger.error("处理数据出现异常，"+e.getMessage(),e);	
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
//    				resultcode = "6002";
//    				errorinfo = couponmonth+"已添加";
    				logger.info(spid+"已添加数据，由于触发订购，找到表内唯一数据，如果是无效状态修改为有效状态");
    				//如果用户上月订购，上月退订，上月再订购，维持到下月再退订再订购 此时表内数据是下月有效，但触发下下月的数据添加
					//此时数据不进行处理，因为是有效状态，不进行修改，如果是无效状态 则对月份进行更新 更改为下下月的数据
					HashMap map = handleDao.queryActiveDataTelecom(params);
					if(map!=null && "5".equals(map.get("STATUS"))){
						map.put("COUPONMONTH",couponmonth);
						map.put("STARTTIME",params.get("STARTTIME"));
						map.put("ENDTIME",params.get("ENDTIME"));
						map.put("STATUS","0");
						handleDao.updateActiveDataTelecom(map);
						resultcode = "6002";
						errorinfo = "已添加过数据，已进行更新，更新月份"+couponmonth;
					}else{
						resultcode = "6005";
						errorinfo = "spid："+spid+"无数据，或状态不是无效状态";
					}
                }else{
    				resultcode = "8000";
    				errorinfo = "处理数据出现异常，"+e.getMessage()+errorinfo;
                }
			}
		} finally{
			logmap.put("resultcode", resultcode);
			logmap.put("errorinfo", errorinfo);
			logmap.put("times", System.currentTimeMillis() - starttime);
			logger.info(logmap);
		}
		
		reData.put("resultcode", resultcode);
		reData.put("errorinfo", errorinfo);
		
		return reData;
	}

	/**
	 * 获取指定月份的最后一天
	 * @param yearMonth YYYY-MM
	 * @return 返回格式：YYYY-MM-DD
	 */
	private String getLastDayOfMonth(String yearMonth) {
		int year = Integer.parseInt(yearMonth.split("-")[0]);  //年
		int month = Integer.parseInt(yearMonth.split("-")[1]); //月
		Calendar cal = Calendar.getInstance();
		// 设置年份
		cal.set(Calendar.YEAR, year);
		// 设置月份
		// cal.set(Calendar.MONTH, month - 1);
		cal.set(Calendar.MONTH, month); //设置当前月的上一个月
		// 获取某月最大天数
		//int lastDay = cal.getActualMaximum(Calendar.DATE);
		int lastDay = cal.getMinimum(Calendar.DATE); //获取月份中的最小值，即第一天
		// 设置日历中月份的最大天数
		//cal.set(Calendar.DAY_OF_MONTH, lastDay);
		cal.set(Calendar.DAY_OF_MONTH, lastDay - 1); //上月的第一天减去1就是当月的最后一天
		// 格式化日期
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		return sdf.format(cal.getTime());
	}

	private static String getNextMonth(String startmonth) throws Exception {
		String month = startmonth;
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");//注意月份是MM
		Date date = simpleDateFormat.parse(month);
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);//设置起时间
		cal.add(Calendar.MONTH, 1);//增加一个月
		date = cal.getTime();
		month = simpleDateFormat.format(date);
		return month;
	}
	
    /**
     * 发送短信
     */
    private String sendsms(HashMap<String, String> _data) {
        String result = ""; //调接口返回信息
        String smstxt = null; //短信内容
		HashMap recmap = new HashMap();
        //发短信开始
        try {             

        	String spid = _data.get("SPID");
        	String userid = _data.get("USERID");
//        	smstxt = smsconf.get(spid);
			smstxt = _data.get("CONTENT");
            if(!StringUtils.isEmpty(smstxt) && userid != null && userid.length() == 11){//有短信内容匹配
            	smstxt = smstxt.replace("#USERID#", userid);
            	result = SendSmsUtil.send(userid, smstxt);
            }else{
            	result = "无短信配置";
            }
        } catch (Exception e) {
            result = "短信发送出现异常，"+e.getMessage();
        } finally {

        }
        
        return result;
    }
}
