package com.chinacreator.kafka.sync.handle.impl;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SendSmsUtil;

/**
 * 【虎年开门红】活动
 * @author xu.zhou
 * @date 20220215
 */
@Component("tigerOpenRed")
public class TigerOpenRedHandleService implements IHandleService{
	
	private static Logger logger = Logger.getLogger("activityrec");
	
	@Autowired
	ActivityHandleDao handleDao;

	@Override
	public HashMap<String, String> handle(HashMap<String, String> _data, HashMap<String, String> _invokeConf) throws Exception {
		HashMap<String, String> reData = new HashMap<String, String>();
		HashMap<String, Object> logmap = new HashMap<String, Object>();
		long starttime = System.currentTimeMillis();
		logmap.put("data", _data);
		String resultcode = "5000";
		String errorinfo = "无处理结果";
		try {
			SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
			String currtime = df.format(new Date());//当前时间
			String invokestarttime = _invokeConf.get("EXTEND1");
			String invokeendtime = _invokeConf.get("EXTEND2");
			//判断当前时间是否在活动时间内
			if(!StringUtils.isEmpty(invokestarttime) && !StringUtils.isEmpty(invokeendtime)
					&& invokestarttime.length() == 14 && invokeendtime.length() == 14
					&& (Long.parseLong(currtime) < Long.parseLong(invokestarttime)
							|| Long.parseLong(currtime) > Long.parseLong(invokeendtime))){
				throw new VideoException("6003", "当前时间未在活动期内", new String[0]);
			}
			/***
			STARTTIME 取ORDERTIME，只要年月日，格式为YYYY-MM-DD
			ENDTIME 按ORDERTIME，取当月最后一天，也是年月日，格式为YYYY-MM-DD
			ORDERTIME必须在活动配置表的STARTTIME和ENDTIME之间
			COUPONMONTH取ORDERTIME的年月值，格式为YYYYMM
			COUPONID取tb_acproduct_config的ID
			 */
			HashMap<String, String> params = new HashMap<String, String>();
			params.put("CHSYNCWAITID", _data.get("ID"));
			params.put("ORDERID", _data.get("RELID"));
			params.put("USERID", _data.get("USERID"));
			params.put("SPID", _data.get("SPID"));
			String ordertime = _data.get("ORDERTIME");
			String st = ordertime.substring(0,4)+"-"+ordertime.substring(4,6)+"-"+ordertime.substring(6,8);
			params.put("STARTTIME", st);
			params.put("ENDTIME", getLastDayOfMonth(ordertime.substring(0,4)+"-"+ordertime.substring(4,6)));
			params.put("COUPONMONTH", ordertime.substring(0,6));
			
			String couponid = "";
			HashMap confMap = null;
			//获取活动配置
			List<HashMap> confData = handleDao.qryAcproductConf(_invokeConf.get("URL"));
			if(confData != null && confData.size()>0){
				for(HashMap hm : confData){
					//获取配置
					if(hm.get("SPID").equals(_data.get("SPID")) && hm.get("ACTIVITYCODE").equals(_invokeConf.get("URL"))){
						confMap = hm;
						break;
					}
				}			
			}
			//有配置数据	
			if(confMap != null){
				couponid = confMap.get("ID").toString();
				String activitystarttime = confMap.get("STARTTIME").toString();
				String activityendtime = confMap.get("ENDTIME").toString();
				//订购时间小于活动开始时间，或者订购时间大于活动结束时间
				if(Long.parseLong(ordertime) < Long.parseLong(activitystarttime)
						|| Long.parseLong(ordertime) > Long.parseLong(activityendtime)){
					throw new VideoException("6001", "订购时间未在活动期内", new String[0]);
				}
			}else{
				throw new VideoException("99999", "TB_ACPRODUCT_CONFIG无配置", new String[0]);
			}

			//20220308 邮件：107653877411新增渠道，不参加返卷码和续送  该渠道为子渠道orderchannel2
			String relid = _data.get("RELID");
			HashMap relmap = handleDao.getOrderData(relid);
			String orderchannel2 = "";
			if(relmap!=null && relmap.size()>0) {
				orderchannel2 = relmap.get("ORDERCHANNEL2") == null ? "" : relmap.get("ORDERCHANNEL2").toString();
			}
			//20220322 每个产品的回流渠道均不生成虎年券码
			/* 二级渠道 主渠道为boyuan，子渠道为以下编码
				159144738363 	1059
				159144144203 	979
				110101285056 	6
				110101169876 	1056
				133681752577	1005
				133681154854	1060
			*
			* */
			if("1056".equals(_data.get("SPID")) && ("107653877411".equals(orderchannel2) || "110101169876".equals(orderchannel2))){
				throw new VideoException("6004", "此渠道不参加产品1056返券码和续送", new String[0]);
			}else if("1059".equals(_data.get("SPID")) && "159144738363".equals(orderchannel2)){
				throw new VideoException("6004", "此渠道不参加产品1059返券码和续送", new String[0]);
			}else if("979".equals(_data.get("SPID")) && "159144144203".equals(orderchannel2)){
				throw new VideoException("6004", "此渠道不参加产品979返券码和续送", new String[0]);
			}else if("6".equals(_data.get("SPID")) && "110101285056".equals(orderchannel2)){
				throw new VideoException("6004", "此渠道不参加产品6返券码和续送", new String[0]);
			}else if("1005".equals(_data.get("SPID")) && "133681752577".equals(orderchannel2)){
				throw new VideoException("6004", "此渠道不参加产品1005返券码和续送", new String[0]);
			}else if("1060".equals(_data.get("SPID")) && "133681154854".equals(orderchannel2)){
				throw new VideoException("6004", "此渠道不参加产品1060返券码和续送", new String[0]);
			}

			params.put("COUPONTYPE", confMap.get("COUPONTYPE")+"");
			params.put("COUPONID", couponid);
			params.put("ACTIVITYCODE", confMap.get("ACTIVITYCODE")+"");

			logmap.put("params", params);
			//同一号码、同一产品、同一月份，每月只能添加一条数据
			handleDao.addActiveDataByTiger(params);

			resultcode = "0";
			errorinfo = "ok";
			//data添加短信内容 20220426
			_data.put("CONTENT",confMap.get("EXTEND1")==null?"":confMap.get("EXTEND1").toString());
			String smssendres = this.sendsms(_data);
			errorinfo += "，smssend:"+smssendres;
			logmap.put("smssendres", smssendres);
		} catch (Exception e) {
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage()+errorinfo;
			} else {
				e.printStackTrace();
				logger.error("处理数据出现异常，"+e.getMessage(),e);	
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
    				resultcode = "6002";
    				errorinfo = "当月已添加";
                }else{
    				resultcode = "8000";
    				errorinfo = "处理数据出现异常，"+e.getMessage()+errorinfo;
                }
			}
		} finally{
			logmap.put("resultcode", resultcode);
			logmap.put("errorinfo", errorinfo);
			logmap.put("times", System.currentTimeMillis() - starttime);
			logger.info(logmap);
		}
		
		reData.put("resultcode", resultcode);
		reData.put("errorinfo", errorinfo);
		
		return reData;
	}

	/**
	 * 获取指定月份的最后一天
	 * @param yearMonth YYYY-MM
	 * @return 返回格式：YYYY-MM-DD
	 */
	private String getLastDayOfMonth(String yearMonth) {
		int year = Integer.parseInt(yearMonth.split("-")[0]);  //年
		int month = Integer.parseInt(yearMonth.split("-")[1]); //月
		Calendar cal = Calendar.getInstance();
		// 设置年份
		cal.set(Calendar.YEAR, year);
		// 设置月份
		// cal.set(Calendar.MONTH, month - 1);
		cal.set(Calendar.MONTH, month); //设置当前月的上一个月
		// 获取某月最大天数
		//int lastDay = cal.getActualMaximum(Calendar.DATE);
		int lastDay = cal.getMinimum(Calendar.DATE); //获取月份中的最小值，即第一天
		// 设置日历中月份的最大天数
		//cal.set(Calendar.DAY_OF_MONTH, lastDay);
		cal.set(Calendar.DAY_OF_MONTH, lastDay - 1); //上月的第一天减去1就是当月的最后一天
		// 格式化日期
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		return sdf.format(cal.getTime());
	}
	
    /**
     * 发送短信
     */
    private String sendsms(HashMap<String, String> _data) {
        String result = ""; //调接口返回信息
        String smstxt = null; //短信内容
        
        /*
 l  对于参与腾讯15元及25元产品活动的，当用户获得话费券后会追加一条短信（与订购成功通知无关），短信内容：
【联通畅视】186xxxx9658，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。
兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 
兑换路径二：微信公众号“联通畅视“—限·福利—领话费券
更多优惠：1分钱抢购腾讯视频会员月卡＋10GB专属流量请戳：https://operation.mige.tv/a/#/4e46f15d5a2b 
 
l  对于参与B站15元及25元产品活动的，当用户获得话费券后会追加一条短信（与订购成功通知无关），短信内容：
【联通畅视】186xxxx9658，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。
兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 
兑换路径二：微信公众号“联通畅视“—限·福利—领话费券
更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v
 
l  对于参与芒果15元及19元产品活动的，当用户获得话费券后会追加一条短信（与订购成功通知无关），短信内容：
【联通畅视】186xxxx9658，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。
兑换路径一：微信小程序“联通畅视菌“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H  
兑换路径二：微信公众号“联通畅视菌“—限·福利—领话费券
更多优惠：1分钱抢购芒果TV会员月卡＋10GB专属流量请戳：http://v.mige.tv/Mfm2Ef
         */
        HashMap<String, String> smsconf = new HashMap<String, String>();
        smsconf.put("6","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 兑换路径二：微信公众号“联通畅视“—限·福利—领话费券更多优惠：1分钱抢购腾讯视频会员月卡＋10GB专属流量请戳：https://operation.mige.tv/a/#/4e46f15d5a2b");
        smsconf.put("1056","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 兑换路径二：微信公众号“联通畅视“—限·福利—领话费券更多优惠：1分钱抢购腾讯视频会员月卡＋10GB专属流量请戳：https://operation.mige.tv/a/#/4e46f15d5a2b");
        
        smsconf.put("979","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 兑换路径二：微信公众号“联通畅视“—限·福利—领话费券更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v");
        smsconf.put("1059","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H 兑换路径二：微信公众号“联通畅视“—限·福利—领话费券更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v");
        
        smsconf.put("1005","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视菌“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H  兑换路径二：微信公众号“联通畅视菌“—限·福利—领话费券更多优惠：1分钱抢购芒果TV会员月卡＋10GB专属流量请戳：http://v.mige.tv/Mfm2Ef");
        smsconf.put("1060","【联通畅视】#USERID#，您参与“虎年开门红”活动获得5元话费，请及时兑换，有效期至本月月底。兑换路径一：微信小程序“联通畅视菌“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H  兑换路径二：微信公众号“联通畅视菌“—限·福利—领话费券更多优惠：1分钱抢购芒果TV会员月卡＋10GB专属流量请戳：http://v.mige.tv/Mfm2Ef");

        //20220426邮件 5-8月份央视频“看央视，领话费”活动需求
		/**券码：
		券码名称：10元话费券
		起始时间（订购即发券）：订购时间
		失效时间：券码起始时间的当月月底，例如用户5月2日订购，失效时间为5月31日

		额外短信：
		订购后，满足获得卷码要求的用户触发一条短信，短信内容：
		l  对于参与央视频25元及30元产品活动的，当用户获得话费券后会追加一条短信（与订购成功通知无关），短信内容：
【联通畅视】186xxxx9658，您参与“看央视，领话费”活动获得10元话费，请及时兑换，有效期至本月月底。
		兑换路径一：微信公众号“联通畅视“—限·福利—领话费券
		兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H
		更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v
		 **/

		//获取短信模板，从配置表里拿


		smsconf.put("10214","【联通畅视】#USERID#，您参与“看央视，领话费”活动获得10元话费，请及时兑换，有效期至本月月底。\n" +
				"兑换路径一：微信公众号“联通畅视“—限·福利—领话费券\n " +
				"兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H\n" +
				"更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v\n");

		smsconf.put("10216","【联通畅视】#USERID#，您参与“看央视，领话费”活动获得10元话费，请及时兑换，有效期至本月月底。\n" +
				"兑换路径一：微信公众号“联通畅视“—限·福利—领话费券\n " +
				"兑换路径一：微信小程序“联通畅视“—我的—畅视礼券，也可通过链接打开小程序：https://cs.mige.tv/H\n" +
				"更多优惠：1分钱抢购月度B站大会员＋15GB专属流量请戳：http://v.mige.tv/rMR77v\n");



		HashMap recmap = new HashMap();
        //发短信开始
        try {             

        	String spid = _data.get("SPID");
        	String userid = _data.get("USERID");
//        	smstxt = smsconf.get(spid);
			smstxt = _data.get("CONTENT");
            if(!StringUtils.isEmpty(smstxt) && userid != null && userid.length() == 11){//有短信内容匹配
            	smstxt = smstxt.replace("#USERID#", userid);
            	result = SendSmsUtil.send(userid, smstxt);
            }else{
            	result = "无短信配置";
            }
        } catch (Exception e) {
            result = "短信发送出现异常，"+e.getMessage();
        } finally {

        }
        
        return result;
    }
}
