package com.chinacreator.kafka.sync.handle.impl;

import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ActivityHandleDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SendSmsUtil;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 【优酷订购用户送定向流量活动需求】活动
 *
 * @author can.he
 * @date 20220817
 */

/**
 * 规则：
 * 【活动名称】优酷订购用户送定向流量活动需求
 *       【活动时间】
 *               1.活动时间：2022-08-23 00:00:00   至  2022-08-25  23:59:59
 *                 2.生成资格时间：订购成功时间 至 2022-08-25 23:59:59
 *        【活动产品】
 *                 1.优酷合约包（spid： 1048）
 *               2.优酷专属15GB流量（spid：21156）
 *               3.优酷VIP+专属10GB（spid：1058）
 *       【订购范围】全渠道
 *       【活动规则】
 *               1.必须在活动期间订购
 *                 2.必须为新用户，此前对于订购的产品没有订购记录的用户
 *                 3.订购3款产品中的任意一种
 *                 4.活动期间用户只能领取一次，同时订购3款产品也只能领取一次
 *                 5.活动期间订购-退订，依然能领取
 */
@Component("youtuTwoActivity")
public class YoukuTwoActivityHandleService implements IHandleService {

    private static Logger logger = Logger.getLogger("activityrec");

    @Autowired
    ActivityHandleDao handleDao;

    @Override
    public HashMap<String, String> handle(HashMap<String, String> _data, HashMap<String, String> _invokeConf) throws Exception {
        HashMap<String, String> reData = new HashMap<String, String>();
        HashMap<String, Object> logmap = new HashMap<String, Object>();
        long starttime = System.currentTimeMillis();
        logmap.put("data", _data);
        String resultcode = "5000";
        String errorinfo = "无处理结果";
        String spid = "";
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
            String currtime = df.format(new Date());//当前时间
            String invokestarttime = _invokeConf.get("EXTEND1");
            String invokeendtime = _invokeConf.get("EXTEND2");
            //判断当前时间是否在活动时间内
            if (!StringUtils.isEmpty(invokestarttime) && !StringUtils.isEmpty(invokeendtime)
                    && invokestarttime.length() == 14 && invokeendtime.length() == 14
                    && (Long.parseLong(currtime) < Long.parseLong(invokestarttime)
                    || Long.parseLong(currtime) > Long.parseLong(invokeendtime))) {
                throw new VideoException("6003", "当前时间未在活动期内", new String[0]);
            }
            /***
             STARTTIME 取ORDERTIME，只要年月日，格式为YYYY-MM-DD
             ENDTIME 按ORDERTIME，取当月最后一天，也是年月日，格式为YYYY-MM-DD
             ORDERTIME必须在活动配置表的STARTTIME和ENDTIME之间
             COUPONMONTH取ORDERTIME的年月值，格式为YYYYMM
             COUPONID取tb_acproduct_config的ID
             */
            HashMap<String, String> params = new HashMap<String, String>();
            params.put("CHSYNCWAITID", _data.get("ID"));
            params.put("ORDERID", _data.get("RELID"));
            params.put("USERID", _data.get("USERID"));
            params.put("SPID", _data.get("SPID"));
            String ordertime = _data.get("ORDERTIME");
            String st = ordertime.substring(0, 4) + "-" + ordertime.substring(4, 6) + "-" + ordertime.substring(6, 8);
            params.put("STARTTIME", st);
            params.put("ENDTIME", getLastDayOfMonth(ordertime.substring(0, 4) + "-" + ordertime.substring(4, 6)));
            params.put("COUPONMONTH", ordertime.substring(0, 6));

            String couponid = "";
            HashMap confMap = null;
            //获取活动配置
            List<HashMap> confData = handleDao.qryAcproductConf(_invokeConf.get("URL"));
            if (confData != null && confData.size() > 0) {
                for (HashMap hm : confData) {
                    //获取配置
                    if (hm.get("SPID").equals(_data.get("SPID")) && hm.get("ACTIVITYCODE").equals(_invokeConf.get("URL"))) {
                        confMap = hm;
                        break;
                    }
                }
            }
            //有配置数据
            if (confMap != null) {
                couponid = confMap.get("ID").toString();
                String activitystarttime = confMap.get("STARTTIME").toString();
                String activityendtime = confMap.get("ENDTIME").toString();
                //订购时间小于活动开始时间，或者订购时间大于活动结束时间
                if (Long.parseLong(ordertime) < Long.parseLong(activitystarttime)
                        || Long.parseLong(ordertime) > Long.parseLong(activityendtime)) {
                    throw new VideoException("6001", "订购时间未在活动期内", new String[0]);
                }
            } else {
                throw new VideoException("99999", "TB_ACPRODUCT_CONFIG无配置", new String[0]);
            }


            String relid = _data.get("RELID");
            String oldtime  = relid.substring(0,8);

            String newtime  = st.replace("-","");
            if(!oldtime.equals(newtime)){
                throw new VideoException("6007", "此活动仅限新用户", new String[0]);
            }

            params.put("COUPONTYPE", confMap.get("COUPONTYPE") + "");
            params.put("COUPONID", couponid);
            params.put("ACTIVITYCODE", confMap.get("ACTIVITYCODE") + "");

            logmap.put("params", params);
            //同一号码只能添加一条数据
            handleDao.addActiveDataByYoutu(params);

            resultcode = "0";
            errorinfo = "ok";
        } catch (Exception e) {
            if (e instanceof VideoException) {
                resultcode = ((VideoException) e).getErrorcode();
                errorinfo = ((VideoException) e).getMessage() + errorinfo;
            } else {
                e.printStackTrace();
                logger.error("处理数据出现异常，" + e.getMessage(), e);
                //如果违反唯一约束，
                if (e.getMessage() != null && e.getMessage().contains("ORA-00001")) {
                    resultcode = "6002";
                    errorinfo = "当月已添加";
                } else {
                    resultcode = "8000";
                    errorinfo = "处理数据出现异常，" + e.getMessage() + errorinfo;
                }
            }
        } finally {
            logmap.put("resultcode", resultcode);
            logmap.put("errorinfo", errorinfo);
            logmap.put("times", System.currentTimeMillis() - starttime);
            logger.info(logmap);
        }

        reData.put("resultcode", resultcode);
        reData.put("errorinfo", errorinfo);

        return reData;
    }

    /**
     * 获取指定月份的最后一天
     * @param yearMonth YYYY-MM
     * @return 返回格式：YYYY-MM-DD
     */
    private String getLastDayOfMonth(String yearMonth) {
        int year = Integer.parseInt(yearMonth.split("-")[0]);  //年
        int month = Integer.parseInt(yearMonth.split("-")[1]); //月
        Calendar cal = Calendar.getInstance();
        // 设置年份
        cal.set(Calendar.YEAR, year);
        // 设置月份
        // cal.set(Calendar.MONTH, month - 1);
        cal.set(Calendar.MONTH, month); //设置当前月的上一个月
        // 获取某月最大天数
        //int lastDay = cal.getActualMaximum(Calendar.DATE);
        int lastDay = cal.getMinimum(Calendar.DATE); //获取月份中的最小值，即第一天
        // 设置日历中月份的最大天数
        //cal.set(Calendar.DAY_OF_MONTH, lastDay);
        cal.set(Calendar.DAY_OF_MONTH, lastDay - 1); //上月的第一天减去1就是当月的最后一天
        // 格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(cal.getTime());
    }
}
