package com.chinacreator.kafka.sync.quartz;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ChannelSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.SyncMainDeal;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 内部活动数据同步定时任务
 * @author xu.zhou
 * @date 20220217
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ActivityHandleSyncQuartz extends BaseService implements Job {
	
	private static Logger logger = Logger.getLogger("activityrec");

	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			long starttime = System.currentTimeMillis();
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			int pages = jobConf.getPAGES();
			String extend1 = jobConf.getEXTEND1();
			
			if(extend1 == null || "".equals(extend1.trim()) || "null".equals(extend1)){
				logger.info("extend1=>"+extend1+"，无条件配置，流程中止");
				return;
			}
			
			String sel = ""; //选择条件配置，可以按invoke表的channel来配置，不同的channel，使用单独的JOB，以提高数据的处理能力
			String selcondStr = "";
			if(!"*".equals(extend1)){
				List<String> resultList= new ArrayList<String>(Arrays.asList(extend1.split(",")));
				sel = resultList.toString().replaceAll(", ", "','").replace("[", "('").replace("]", "') ");
				selcondStr = " AND B.CHANNEL IN "+sel+" ";
			}
			//List<String> resultList= new ArrayList<String>(Arrays.asList(extend1.split(",")));
			//spids = resultList.toString().replaceAll(", ", "','").replace("[", "('").replace("]", "')");
			logger.info("======================="+selcondStr+"同步开始=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
			//查询数据SQL，只查询SYNCTYPE为2的数据		
			String sql = "SELECT * FROM ( "+
							" SELECT A.ID, RECID, RELID, SPID, CPID, USERID, A.ORDERTYPE, "+
							" TO_CHAR(ORDERTIME,'YYYYMMDDHH24MISS') ORDERTIME, "+
							" TO_CHAR(CANCELTIME,'YYYYMMDDHH24MISS')  CANCELTIME, "+
							" TO_CHAR(ENDTIME,'YYYYMMDDHH24MISS') ENDTIME, "+
							" ORDERCHANNEL,CANCELCHANNEL, "+
							" PROVINCE, AREA, INVOKECHANNEL, TO_CHAR(SYNCCOUNT) SYNCCOUNT"+ 
							//" FROM TD_CHSYNC_WAIT WHERE SPID IN "+spids+" AND RESULTCODE NOT IN ('0','2','99999') AND SYNCCOUNT < 5 ORDER BY INSERTTIME "+
							" FROM TD_CHSYNC_WAIT A, TB_CHSYNC_INVOKE_CONF B WHERE "+
							" A.INVOKECHANNEL = B.CHANNEL AND (A.ORDERTYPE = B.ORDERTYPE OR B.ORDERTYPE = '*') "+
							" AND B.STATUS = '0' AND B.SYNCTYPE = '2' "+
							selcondStr+" AND RESULTCODE NOT IN ('0','2','99999') AND SYNCCOUNT < 5 ORDER BY A.INSERTTIME "+
							" ) WHERE ROWNUM <= "+pages;
			logger.info("查询SQL: "+ sql);

			logger.info(Thread.currentThread().getName()+"============"+sel+"同步开始=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
			
			ChannelSyncDao dao = (ChannelSyncDao) SpringContextUtils.getBean(ChannelSyncDao.class);
			//根据SPID获取在待同步表中的数据
			List<HashMap<String, String>> list = dao.execQryRelWaitSql(sql);
			//去除操作冲突的数据，同一手机号码同一SPID，只保留同一操作的数据，防止后面的数据被提前处理，比如有订购 和退订，防止先处理退订再处理订购
			delRepeat(list);
			//logger.info("获取数据条数=>"+(list == null ? "0" : list.size()));
			logger.info(extend1+"获取数据条数=>"+(list == null ? "0" : list.size()));
			if(list != null && list.size()>0){
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (HashMap<String, String> data : list) {
					ActivityHandleSyncDeal deal = new ActivityHandleSyncDeal(data, list.size(), threadSignal, logger, dao);
					executorService.execute(deal);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			
			logger.info(Thread.currentThread().getName()+"============"+sel+"同步结束=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date())+"，总耗时=>"+((System.currentTimeMillis()-starttime)/1000)+"秒");
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("处理出现异常，"+e.getMessage(),e);
		}
	}

	/**
	 *根据手机号码去重，同一手机号码如果最前的数据是1，则移除为0的数据，如果最前数据是0，则移除为1的数据
	 * 保证同一批数据里不会出现手机号码、SPID、OPERTYPE相同的数据处理
	 * 20220725更新，USERID+SPID+INVOKECHANNEL不能重复
	 * @param list
	 */
	private void delRepeat(List<HashMap<String, String>> list){
		if(list == null || list.size() == 0){
			return;
		}	
		HashMap<String, String> tmpmap = new HashMap<String, String>();
		if(list != null && list.size()>0){
			logger.info("去重复前数据个数："+list.size());
			Iterator<HashMap<String, String>> it = list.iterator();;
			HashMap<String, String> hm = null;
			while(it.hasNext()){
				hm = it.next();
				if(tmpmap.containsKey(hm.get("USERID")+"##"+hm.get("SPID")+"##"+hm.get("INVOKECHANNEL"))){
					/****
					if(hm.get("ORDERTYPE").equals(tmpmap.get(hm.get("USERID")+"##"+hm.get("SPID")))){
						//手机号码+SPID+OPERTYPE相同
					}else{
						logger.info(hm.get("USERID")+"##"+hm.get("SPID")+tmpmap.get(hm.get("USERID")+"##"+hm.get("SPID"))+"手机号码和操作冲突，移除数据，下次处理，" + hm);
						it.remove(); //移除对象
						continue;
					}
					****/
					logger.info("USERID+SPID+INVOKECHANNEL有重复数据，移除数据，下次处理，" + hm);
					it.remove(); //移除对象
					continue;
				}else{
					tmpmap.put(hm.get("USERID")+"##"+hm.get("SPID")+"##"+hm.get("INVOKECHANNEL"),hm.get("ID"));
				}
			}
			logger.info("去重复后数据个数："+list.size());
		}
	}
}
