package com.chinacreator.kafka.sync.quartz;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionStatus;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ChannelSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 处理更新订购同步记录表出现锁表而未成功的数据
 * 迁移REC表和WAIT表的数据
 * @author xu.zhou
 * @date 20210622
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ChannelSyncUpdQuartz extends BaseService implements Job {
	
	private static Logger logger = Logger.getLogger("channelsyncupd");

	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		
		logger.info("=======================执行开始=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
		//获取配置信息
		TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
		int pages = jobConf.getPAGES();
		ChannelSyncDao dao = (ChannelSyncDao) SpringContextUtils.getBean(ChannelSyncDao.class);
		
		//处理REC表状态一直为待处理的数据（WAIT数据同步已完成，但未更新REC表的数据，REC表同步状态为2，处理中）
		recupdprocess(dao);
		
		//迁移表TD_CHSYNC_WAIT数据（完成时间超过10分钟的数据）
		//moveWaitDone(dao);
		
		//迁移表TD_CHSYNC_ORDER_REC数据（完成时间超过3天的数据）
		//moveRecDone(dao);
		
		//其他数据更新
		//updOtherData(dao);
		
		logger.info("=========================执行完成=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
	}
	
	/**
	 * 其他数据更新
	 * @param dao
	 */
	private void updOtherData(ChannelSyncDao dao){
		try {
			//更新WAIT表状态为处理中超过5分钟的数据为待处理
			int rows = dao.updWaitSyncError();
			logger.info("更新WAIT表状态为处理中超过5分钟的数据为待处理数据为=>"+rows);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("更新其他数据出现异常，"+e.getMessage(),e);
		}
		
	}
	
	/**
	 * 迁移表TD_CHSYNC_ORDER_REC数据
	 * 迁移三天前的数据
	 * @param dao
	 */
	private void moveRecDone(ChannelSyncDao dao){
		try {
			List<String> list = dao.getRecDoneData();
			if(list != null && list.size()>0){
				String execsql = "";
				//使用事务控制
				PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
				TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
				TransactionStatus transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
				try {
					String ids = list.toString().replaceAll(", ", "','").replace("[", "('").replace("]", "')");
					execsql = "INSERT INTO TD_CHSYNC_ORDER_REC_HIS SELECT * FROM TD_CHSYNC_ORDER_REC WHERE ID IN "+ids;	
					logger.info("迁移执行SQL=>"+execsql);
					int rows = dao.execAddSql(execsql);
					logger.info("迁移数据行数=>"+rows);
					execsql = " DELETE FROM TD_CHSYNC_ORDER_REC WHERE ID IN "+ids;
					logger.info("删除执行SQL=>"+execsql);
					rows = dao.execDelSql(execsql);
					logger.info("删除数据行数=>"+rows);
					dataSourceTransactionManager.commit(transactionStatus);//提交
				} catch (Exception e) {
					e.printStackTrace();
					dataSourceTransactionManager.rollback(transactionStatus);
					logger.error("迁移历史数据出现异常，"+e.getMessage()+"，执行SQL=>"+execsql,e);
				}		
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("迁移数据出现异常，"+e.getMessage(),e);
		}
	}
	
	/**
	 * 迁移表TD_CHSYNC_WAIT数据
	 * @param dao
	 */
	private void moveWaitDone(ChannelSyncDao dao){
		try {
			List<String> list = dao.getWaitDoneData();
			if(list != null && list.size()>0){
				String execsql = "";
				//使用事务控制
				PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
				TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
				TransactionStatus transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
				try {
					String ids = list.toString().replaceAll(", ", "','").replace("[", "('").replace("]", "')");
					execsql = "INSERT INTO TD_CHSYNC_WAIT_HIS SELECT * FROM TD_CHSYNC_WAIT WHERE ID IN "+ids;	
					logger.info("迁移执行SQL=>"+execsql);
					int rows = dao.execAddSql(execsql);
					logger.info("迁移数据行数=>"+rows);
					execsql = " DELETE FROM TD_CHSYNC_WAIT WHERE ID IN "+ids;
					logger.info("删除执行SQL=>"+execsql);
					rows = dao.execDelSql(execsql);
					logger.info("删除数据行数=>"+rows);
					dataSourceTransactionManager.commit(transactionStatus);//提交
				} catch (Exception e) {
					e.printStackTrace();
					dataSourceTransactionManager.rollback(transactionStatus);
					logger.error("迁移历史数据出现异常，"+e.getMessage()+"，执行SQL=>"+execsql,e);
				}		
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("迁移数据出现异常，"+e.getMessage(),e);
		}
	}
	
	/**
	 * 处理REC表状态一直为待处理的数据
	 * TD_CHSYNC_WAIT
	 * @param dao
	 */
	private void recupdprocess(ChannelSyncDao dao){	
		try {
			List<HashMap<String, String>> list = dao.getRecUpdData();
			String recid = null;
			//使用事务控制
			PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
			TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
			TransactionStatus transactionStatus = null;
			for(HashMap<String, String> updhm : list){
				recid = updhm.get("ID");
				transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
				try {		
					//是否全部同步完成
					boolean hasDone = true;
					//是否同步成功，1待处理，2处理中，3处理失败，0成功
					String syncsucceed = "0";
					String resultcode = "";
					String synccount = "";
					//掊品返回非0，但也是成功的编码，比如华盛接口返回{"code":7018,"data":null,"message":"该用户不存在或已销户","success":false}
					//7018这些调用应该记录为同步成功
					String succcode = null;
					
					//接口配置信息
					List<HashMap<String, String>> confList = null;
					HashMap<String, String> invokeConf = null;
					
					List<HashMap<String, String>> dataList = dao.getWaitDataByRecid(recid);
					for(HashMap<String, String> hm : dataList){
						//获取接口配置信息
						confList = dao.getInvokeConfByOrdertype(hm.get("INVOKECHANNEL"), hm.get("ORDERTYPE"));
						if(confList != null && confList.size()>0){
							//获取配置信息
							invokeConf = dao.getInvokeConfByOrdertype(hm.get("INVOKECHANNEL"), hm.get("ORDERTYPE")).get(0);
							succcode = invokeConf.get("SUCCCODE");
						}

						if(succcode == null) succcode = "";
						
						resultcode = hm.get("RESULTCODE");
						synccount = hm.get("SYNCCOUNT");
						//同步未成功，99999代表无接口配置
						//1 待处理，2处理中，99999无接口配置，0成功, SYNCCOUNT为5代表已重试到最大次数
						if(!"1".equals(resultcode) 
								&& !"2".equals(resultcode)
								&& !"99999".equals(resultcode)
								&& !"0".equals(resultcode)
								&& succcode.indexOf(resultcode) == -1
								&& "5".equals(synccount)){
							syncsucceed = "3";
						}
						
						//同步未完成
						//1待处理，2处理中, 不等于99999且重试次数未达到最大次数
						if("1".equals(resultcode) 
								//待处理
								|| "2".equals(resultcode)
								//同步失败，且未达到最大次数
								|| (!"99999".equals(resultcode) && !"0".equals(resultcode) && !"5".equals(synccount))){
							hasDone = false;
						}
					}
					logger.info("recid=>"+recid+"，syncsucceed=>"+syncsucceed+"，hasDone=>"+hasDone);
					//同步完成，更新同步记录表状态，并更新待处理表的是否更新同步记录表状态（用于数据迁移）
					if(hasDone){
						//更新同步记录表状态为成功或失败
						dao.updOrderRecRes(recid, syncsucceed);
						//根据SYNCID更新对应同步数据
						dao.updWaitUpdrelByRecid(recid);
					}
					//提交事务
					dataSourceTransactionManager.commit(transactionStatus);
				} catch (Exception e) {
					e.printStackTrace();
					//回滚事务
					dataSourceTransactionManager.rollback(transactionStatus);
					logger.error("updOderRecRes更新出现异常，"+e.getMessage()+"=>"+recid,e);
				} 
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("更新REC待处理状态出现异常，"+e.getMessage(),e);
		}
	}
}
