package com.chinacreator.kafka.sync.quartz;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import com.alibaba.fastjson.JSON;
import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.ChannelSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 获取渠道同步表TD_CHSYNC_ORDER_REC的待处理的数据，通过CHANNEL与同步业务配置表TB_CHSYNC_BUSI_CONF关联，
 * 把要同步的数据添加到同步待处理表TD_CHSYNC_WAIT，再更新渠道同步表TD_CHSYNC_ORDER_REC的同步状态标识为处理中，
 * 使用事务方式
 * @author xu.zhou
 * @date 20210618
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ChannelSyncWaitQuartz extends BaseService implements Job {
	
	private static Logger logger = Logger.getLogger("channelsyncwait");

	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		
		logger.info("=======================执行开始=>"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
		//获取配置信息
		TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
		int pages = jobConf.getPAGES();
		ChannelSyncDao dao = (ChannelSyncDao) SpringContextUtils.getBean(ChannelSyncDao.class);
		//获取要同步记录表的数据
		List<HashMap<String, String>> dataList = dao.getSyncRecData(pages);
		//按手机号码去重
		delRepeat(dataList);
		
		String spid = null;
		String opertype = null;
		String orderchannel = null;
		String cancelchannel = null;
		String invokechannel = null;
		List<HashMap<String, String>> confList = null;
		
		//使用事务控制
		PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
		TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
		TransactionStatus transactionStatus = null;
		for(HashMap<String, String> hm : dataList){
			transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
			try {
				spid = null;
				opertype = null;
				orderchannel = null;
				cancelchannel = null;
				invokechannel = null;
				spid = hm.get("SPID");
				opertype = hm.get("ORDERTYPE");
				orderchannel = hm.get("ORDERCHANNEL");
				cancelchannel = hm.get("CANCELCHANNEL");
				if(StringUtils.isEmpty(orderchannel)){
					orderchannel = "*";
				}
				if(StringUtils.isEmpty(cancelchannel)){
					cancelchannel = "*";
				}
				
				if("0".equals(opertype)){
					confList = dao.getOrderSyncConf(spid, orderchannel);
				}else{
					confList = dao.getCancelSyncConf(spid, orderchannel, cancelchannel);
				}
				//同一数据，同一渠道，同一订购类型限制添加多条数据
				HashMap<String, Object> icMap = new HashMap<String, Object>();
				if(confList != null && confList.size() > 0){
					icMap.clear();
					for(HashMap<String, String> confHm : confList){
						invokechannel = confHm.get("INVOKECHANNEL");
						hm.put("INVOKECHANNEL", invokechannel);
						if(icMap.containsKey(invokechannel)){
							logger.info("同一渠道同一类型配置了多条数据，且渠道为"+invokechannel+"已添加了一条数据，无需再添加。用户数据=>"+hm+"，配置数据=>"+confHm);
						}else{
							//添加数据到同步待处理表
							dao.addWaitData(hm);
							icMap.put(invokechannel, confHm);
						}
					}
					icMap.clear();
				}
				
				//根据ID更新TD_CHSYNC_ORDER_REC记录为处理中
				dao.updOrderRecSyncing(hm.get("RECID"));
				//提交事务
				dataSourceTransactionManager.commit(transactionStatus);
			} catch (Exception e) {
				e.printStackTrace();
				//回滚事务
				dataSourceTransactionManager.rollback(transactionStatus);
				logger.error("添加同步数据出现异常，hm=>"+hm+"，错误信息=>"+e.getMessage(),e);
			}
		}
		
		logger.info("=========================执行完成=>"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
	}
	
	/**
	 * 根据手机号码去重，防止同一号码短时间进行订购退订操作，造成同步状态错乱
	 * @param list
	 */
	private void delRepeat(List<HashMap<String, String>> list){
		if(list == null || list.size() == 0){
			return;
		}	
		List<String> userids = new ArrayList<String>();
		if(list != null && list.size()>0){
			logger.info("去重复前数据个数："+list.size());
			Iterator<HashMap<String, String>> it = list.iterator();;
			HashMap<String, String> hm = null;
			while(it.hasNext()){
				hm = it.next();
				if(userids.contains(hm.get("USERID"))){
					logger.info("手机号码重复，移除数据，下次处理，" + hm);
					it.remove(); //移除对象
					continue;
				}else{
					userids.add(hm.get("USERID"));
				}
			}
			logger.info("去重复后数据个数："+list.size());
		}
	}
}
