package com.chinacreator.kafka.sync.quartz;

import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import com.chinacreator.kafka.util.*;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ChannelSyncDao;
import com.chinacreator.kafka.mapper.HuaShengSyncDao;
import com.chinacreator.kafka.sync.bean.BoYuanBean;
import com.chinacreator.kafka.sync.bean.BoYuanLogBean;
import com.chinacreator.kafka.sync.bean.BoYuanPushBean;
import com.chinacreator.kafka.sync.handle.IHandleService;

/**
 *华胜前向业务办理同步处理类
 * @author 周旭
 * @date 20220621
 */
public class HuaShengSyncDeal implements Runnable{
	
	private Logger logger;
	
	private HuaShengSyncDao dao;
	
	private int totalSize;
	
	private CountDownLatch threadSignal;
	
	//TD_CHSYNC_WAIT表数据
	private HashMap<String, String> data;
	
	public HuaShengSyncDeal(HashMap<String, String> _data, int _totalSize, CountDownLatch _threadSignal,Logger _logger, HuaShengSyncDao _dao) {
		this.data = _data;
		this.logger = _logger;
		this.dao = _dao;
		this.totalSize = _totalSize;
		this.threadSignal = _threadSignal;
	}
	
	@SuppressWarnings("unchecked")
	public void run(){
		HashMap<String, Object> logmap = new HashMap<String, Object>();
		long starttime = System.currentTimeMillis();
		logmap.put("data", data);
		String resultcode = "5000";
		String errorinfo = "无返回信息";
		String result = ""; //调接口返回的报文
		String inputparams = "";
		try {		
			//更新数据为处理中
			dao.updWaitSyncing(data.get("ID"));
			//获取接口配置信息		
			List<HashMap<String, String>> dataList = dao.getProjinvokeConf("kafkaorder", "huashengPush");
			if(dataList == null || dataList.size() == 0){
				throw new VideoException("99999", "TB_PROJINVOKEURL_CONF无配置", new String[0]);
			}
			HashMap<String, String> invokeConf = dataList.get(0);
			inputparams = getParams(invokeConf);
			result = invokeface(invokeConf,inputparams);
			errorinfo = result;
            if (!StringUtils.isEmpty(result)) {
            	JSONObject jsonobj =  JSONObject.parseObject(result);
                resultcode = jsonobj.get("code").toString();
            }	
		} catch (Exception e) {
			if("无返回信息".equals(errorinfo)){
				errorinfo = "";
			}
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage()+errorinfo;
			} else {
				e.printStackTrace();
				logger.error("处理数据出现异常，"+e.getMessage(),e);	
				resultcode = "8000";
				errorinfo = "处理数据出现异常，"+e.getMessage()+errorinfo;
			}	
		} finally {
			//当前返回信息非接口文档定义格式时，resultcode为空
			if(StringUtils.isEmpty(resultcode)){
				resultcode = "5001";
			}
            logmap.put("调接口返回报文", result);
			logmap.put("resultcode", resultcode);
			logmap.put("errorinfo", errorinfo);
			logmap.put("inputparams", inputparams);
			logmap.put("times", System.currentTimeMillis() - starttime);
			//更新待同步数据处理结果
			updWaitSyncRes(resultcode, errorinfo, inputparams);
			logger.info(logmap);
			threadSignal.countDown();
		}
	}	
	
	/**
	 * 拼接参数
	 * @return
	 * @throws Exception
	 */
	private String getParams(HashMap<String, String> invokeConf) throws Exception {
		ChannelSyncDao syncdao = (ChannelSyncDao) SpringContextUtils.getBean(ChannelSyncDao.class);
		JSONObject params = new JSONObject();
		JSONObject vars = new JSONObject();
		String ordertype = data.get("ORDERTYPE"); //0订购，1退订
		String hassucc = data.get("HASSUCC"); //办理是否成功，0成功，1失败
		String timestamp = System.currentTimeMillis()+"";
		String templateId = "";//模板ID
		String cpid = data.get("CPID");
		String spid = data.get("SPID");
		String orderchannel = data.get("ORDERCHANNEL"); //订购渠道
		String cancelchannel = data.get("CANCELCHANNEL"); //退订渠道
		//获取SP信息
		HashMap<String, String> spInfo = syncdao.getTbSpInfo(spid).get(0);
		String directype = spInfo.get("DIRECTYPE"); //产品属性
		if(!"1".equals(directype) && !"3".equals(directype)){
			throw new VideoException("7777", "非前向产品，不推送", new String[0]);
		}
		HashMap<String, String> channelInfo = dao.getChannelInfo("huasheng").get(0);
		String mobile = DesUtil.encode(data.get("USERID"), channelInfo.get("PASSWORD"));
		params.put("mobile", mobile);
		params.put("timestamp",timestamp);
		
		//不要推送的订购失败错误编码
		List<String> npfailcode = new ArrayList<String>();
		//不要推送的CPID
		List<String> npcpid = new ArrayList<String>();
		//不要推送的SPID
		List<String> npspid = new ArrayList<String>();
		//不要推送的订购渠道
		List<String> nporderchannel = new ArrayList<String>();
		//不要推送的退订渠道
		List<String> npcancelchannel = new ArrayList<String>();
		//不要处理的错误编码、CPID、SPID
		//{"NPFAILCODE":"8000|333333","NPCPID":"wyj","NPSPID":"111111|222222"}
		String param2 = invokeConf.get("PARAM2"); 
		/**
		 * 验证是否推送
		 */
		if(!StringUtils.isEmpty(param2)){
			JSONObject npjsonObj = JSONObject.parseObject(param2.trim());
			if(npjsonObj.getString("NPFAILCODE") != null){
				npfailcode = Arrays.asList((npjsonObj.getString("NPFAILCODE").split("\\|")));
			}
			if(npjsonObj.getString("NPCPID") != null){
				npcpid = Arrays.asList((npjsonObj.getString("NPCPID").split("\\|")));
				if(npcpid != null && npcpid.contains(cpid)){
					throw new VideoException("7777", "指定CPID不推送", new String[0]);
				}
			}
			if(npjsonObj.getString("NPSPID") != null){
				npspid = Arrays.asList((npjsonObj.getString("NPSPID").split("\\|")));
				if(npspid != null && npspid.contains(spid)){
					throw new VideoException("7777", "指定SPID不推送", new String[0]);
				}
			}			
			if(npjsonObj.getString("NPORDERCHANNEL") != null && "0".equals(ordertype)){
				nporderchannel = Arrays.asList((npjsonObj.getString("NPORDERCHANNEL").split("\\|")));
				if(!StringUtils.isEmpty(orderchannel) && nporderchannel != null && nporderchannel.contains(orderchannel)){
					throw new VideoException("7777", "指定订购渠道不推送", new String[0]);
				}
			}			
			if(npjsonObj.getString("NPCANCELCHANNEL") != null && "1".equals(ordertype)){
				npcancelchannel = Arrays.asList((npjsonObj.getString("NPCANCELCHANNEL").split("\\|")));
				if(!StringUtils.isEmpty(cancelchannel) && npcancelchannel != null && npcancelchannel.contains(cancelchannel)){
					throw new VideoException("7777", "指定退订渠道不推送", new String[0]);
				}
			}
		}
		
		if("0".equals(ordertype)){//订购
			if("0".equals(hassucc)){//成功
				templateId = "15";
				//产品名称：
				vars.put("keyword1",spInfo.get("SPNAME"));
				//订购状态：
				vars.put("keyword2","成功");
				//订购费用：
				vars.put("keyword3",spInfo.get("PRICE"));
				//生效时间：
				vars.put("keyword4",data.get("ORDERTIME"));
			}else{//失败
				String failcode = data.get("FAILCODE");
				if(npfailcode != null && npfailcode.contains(failcode)){
					throw new VideoException("7777", "指定错误编码不推送", new String[0]);
				}
				
				templateId = "14";
				//订购号码：
				vars.put("keyword1",data.get("USERID"));
				//订购时间：
				vars.put("keyword2",data.get("ORDERTIME"));
				//订购业务：
				vars.put("keyword3",spInfo.get("SPNAME"));
				//订购状态：
				vars.put("keyword4","失败");
				//备注：
				vars.put("keyword5",data.get("FAILINFO"));
			}
		}else{//退订
			templateId = "16";
			//本机号码：
			vars.put("tel",data.get("USERID"));
			//业务名称：
			vars.put("type",spInfo.get("SPNAME"));
			//失效时间：
			vars.put("date",data.get("ENDTIME"));
		}
		
		/***
		if("0".equals(ordertype)){//订购
			if("0".equals(hassucc)){//成功
				templateId = "15";
				//产品名称：
				vars += "keyword1:"+spInfo.get("SPNAME");
				//订购状态：
				vars += "|keyword2:成功";
				//订购费用：
				vars += "|keyword3:"+spInfo.get("PRICE");
				//生效时间：
				vars += "|keyword4:"+data.get("ORDERTIME");
			}else{//失败
				templateId = "14";
				//订购号码：
				vars += "keyword1:"+data.get("USERID");
				//订购时间：
				vars += "|keyword2:"+data.get("ORDERTIME");
				//订购业务：
				vars += "|keyword3:"+spInfo.get("SPNAME");
				//订购状态：
				vars += "|keyword4:失败";
				//备注：
				vars += "|keyword5:"+data.get("FAILINFO");
			}
		}else{//退订
			templateId = "16";
			//本机号码：
			vars += "tel:"+data.get("USERID");
			//业务名称：
			vars += "|type:"+spInfo.get("SPNAME");
			//失效时间：
			vars += "|date:"+data.get("ENDTIME");
		}
		***/
		
		params.put("templateId",templateId);	
		params.put("signature",MD5.MD5Encode(mobile+templateId+timestamp+channelInfo.get("PASSWORD")));
		params.put("vars", vars);
		
		return params.toJSONString();
	}
	

	
	/**
	 * 更新待同步数据处理结果
	 * @param resultcode
	 * @param errorinfo
	 */
	private void updWaitSyncRes(String resultcode, String errorinfo, String inputparams){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}		
		    dao.updWaitSyncRes(data.get("ID"), resultcode, errorinfo, inputparams);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("updWaitSyncRes更新出现异常，"+e.getMessage()+"=>"+data.get("ID"));
		} 
	}
	
	
    /**
     * 调接口
     * @param invokeConf
     * @param params
     * @return
     * @throws Exception
     */
    private String invokeface(HashMap<String,String> invokeConf, String params) throws Exception{
    	String result = ""; //调接口返回报文
    	String invokeurl = invokeConf.get("INVOKEURL");
    	//{"METHOD":"POST","TLS":"TLSv1.1","CONTENTTYPE":"application/json","TIMEOUT":"30"}
    	String params1 = invokeConf.get("PARAM1");
		int timeout = 10;
		//TLS协议版本
		String tls = "";
		String method = "POST";
		//数据类型
		String contenttype = "application/json";
		//属性
		Map reqProperty = new HashMap();
    	if(!StringUtils.isEmpty(params1)){
    		JSONObject jsonobj =  JSONObject.parseObject(params1);
    		if(jsonobj.getString("METHOD") != null){
    			method = jsonobj.getString("METHOD");
    		}
    		
    		if(jsonobj.getString("TLS") != null){
    			tls = jsonobj.getString("TLS");
    		}
    		
    		if(jsonobj.getString("CONTENTTYPE") != null){
    			contenttype = jsonobj.getString("CONTENTTYPE");
    		}
    		
    		if(jsonobj.getInteger("TIMEOUT") != null){
    			timeout = jsonobj.getInteger("TIMEOUT");
    		}
    	}
    	
		reqProperty.put("Content-type", contenttype);	
    		
		if(invokeurl.startsWith("https")){
			result = HttpInvoke.sendhttpsReq(method, invokeurl, params, reqProperty, timeout*1000, tls);
		}else{
			result = HttpInvoke.sendhttpReq(method, invokeurl, params, reqProperty, timeout*1000);
		}
    	return result;
    }
    

    //获取系统时间
    private String getDateStr(Date date) {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
        String dateString = formatter.format(date);
        return dateString;
    }

    //将金额转换成2位小数
    private String getPrice(String param){
        String price = "0.00";
        if(param!= null && !param.equals("")) {
            BigDecimal b = new BigDecimal(param);
            price = b.setScale(2,BigDecimal.ROUND_HALF_UP).toString();
        }
        return price;
    }
    
    /**
     * 输出调接口日志
     * @param invokeurl
     * @param inputparams
     * @param outputparams
     */
    private void printLog(HashMap<String, String> invokeConf, String inputparams, String outputparams){
    	logger.info(data.get("ID")+"<=>"+data.get("USERID")+"=>接口配置："+invokeConf+"，请求报文:【"+inputparams+"】响应报文：【"+outputparams+"】");
    }
}
