package com.chinacreator.kafka.sync.quartz;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.HuaShengSyncDao;
import com.chinacreator.kafka.mapper.ProductSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.sync.utils.printLogControl;
import com.chinacreator.kafka.util.HttpInvoke;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.*;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


/**
 * 华胜前向产品办理同步
 * @Author: 周旭
 * @Date: 20220621
 *
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class HuaShengSyncQuartz extends BaseService implements Job {

    private static Logger logger = Logger.getLogger("huashengsync");
    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {

    	String execStarttime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        try {
            //获取配置信息
            TbKafkaJobConf jobConf = (TbKafkaJobConf) context.getJobDetail().getJobDataMap().get("data");
            HuaShengSyncDao dao = (HuaShengSyncDao) SpringContextUtils.getBean(HuaShengSyncDao.class);
            
            //获取要处理的数据
            List<HashMap<String, String>> list = dao.getSyncWaitData(jobConf.getPAGES());
			if(list != null && list.size()>0){
				delRepeat(list);
				logger.info("待处理数据条数："+list.size());
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (HashMap<String, String> data : list) {
					HuaShengSyncDeal deal = new HuaShengSyncDeal(data, list.size(), threadSignal, logger, dao);
					executorService.execute(deal);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			//把状态为正在处理的且时间超过5分钟的异常数据重新设置为待处理。
			dao.updWaitSyncError();
        }catch (Exception e){
            e.printStackTrace();
            logger.error("处理出现异常，"+e.getMessage(),e);
        }
        
        printLogControl.writeLong("执行记录=>开始时间："+execStarttime+"，结束时间："+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()), logger, "HuaShengSyncQuartz");
    }
    
	/**
	 * 根据手机号码去重，同一手机号码如果最前的数据是1，则移除为0的数据，如果最前数据是0，则移除为1的数据
	 * @param list
	 */
	private void delRepeat(List<HashMap<String, String>> list){
		if(list == null || list.size() == 0){
			return;
		}	
		HashMap<String, String> tmpmap = new HashMap<String, String>();
		if(list != null && list.size()>0){
			logger.info("去重复前数据个数："+list.size());
			Iterator<HashMap<String, String>> it = list.iterator();;
			HashMap<String, String> hm = null;
			while(it.hasNext()){
				hm = it.next();
				if(tmpmap.containsKey(hm.get("USERID")+"##"+hm.get("SPID"))){
					logger.info(hm.get("USERID")+"##"+hm.get("SPID")+"##"+hm.get("ID")+"，手机号码和产品有重复数据，下次处理，");
					it.remove(); //移除对象
					continue;					
				}else{
					tmpmap.put(hm.get("USERID")+"##"+hm.get("SPID"),hm.get("ID"));
				}
			}
			logger.info("去重复后数据个数："+list.size());
		}
	}
}
