package com.chinacreator.kafka.sync.quartz;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.BoYuanProductSyncDao;
import com.chinacreator.kafka.mapper.HuiYuanProductSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.util.DesUtil;
import com.chinacreator.kafka.util.HttpInvoke;
import com.chinacreator.kafka.util.MD5;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.*;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 产品同步定时任务
 * @Author: czw
 * @Date: 2022/6/20
 *
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class HuiYuanProductSyncQuartz extends BaseService implements Job {

    private static Logger logger = Logger.getLogger("productwait");
    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        try {
            String resultcode = "5000";
            String resultinfo = "无返回信息";
            String result = ""; //调接口返回的报文
            String channel = "huiyuan_cp";//产品同步办理渠道

            //获取配置信息
            TbKafkaJobConf jobConf = (TbKafkaJobConf) context.getJobDetail().getJobDataMap().get("data");
            HuiYuanProductSyncDao dao = (HuiYuanProductSyncDao) SpringContextUtils.getBean(HuiYuanProductSyncDao.class);
            //将获取要处理的数据
            List<HashMap<String, String>> list = dao.getProductWaitData();
            logger.info("获取数据条数=> " + list.size());
            //获取渠道配置信息
            List<HashMap<String, String>> dataList = dao.getInvokeConfByChannel(channel);

            if (dataList != null && dataList.size() > 0) {
                if (dataList.get(0) != null) {
                    //获取接口配置信息
                    HashMap<String, String> invokeConf = dataList.get(0);
                    for (HashMap<String, String> pw : list) {
                        logger.info("获取数据=> " + pw);
                        logger.info("获取数据=> " + pw.get("ID"));

                        String cpid = pw.get("CPID");
                        String spid = pw.get("SPID");

                        List<HashMap<String,String>> disposeList = dao.getDisposeList(cpid,spid);
                        if(CollectionUtils.isEmpty(disposeList)){
                            dao.updProductWaitSync(pw.get("ID"),"3","当前不需要推送");
                        }else {
                            //获取密码
                            String password = dao.getPassword();
                            String cpKey = dao.getCpKey(cpid, spid);
                            if(StringUtils.isEmpty(cpKey)){
                                continue;
                            }else {
                                try {
                                    dao.updWaitSyncing(pw.get("ID"));
                                    result = BoYuanJSBSync(invokeConf, pw, cpKey, password);
                                    resultinfo = result;
                                    if (StringUtils.isNotEmpty(result)) {
                                        Map map = JSONObject.parseObject(result, Map.class);
                                        resultcode = map.get("code") + "";
                                    }

                                } catch (Exception e) {
                                    if ("无返回信息".equals(resultinfo)) {
                                        resultinfo = "";
                                    }
                                    if (e instanceof VideoException) {
                                        resultcode = ((VideoException) e).getErrorcode();
                                        resultinfo = ((VideoException) e).getMessage() + resultinfo;
                                    } else {
                                        e.printStackTrace();
                                        logger.error("处理数据内部出现异常，" + e.getMessage(), e);
                                        resultcode = "8000";
                                        resultinfo = "处理数据出现异常，" + e.getMessage() + resultinfo;
                                    }
                                } finally {
                                    //当前返回信息非接口文档定义格式时，resultcode为空
                                    if (StringUtils.isEmpty(resultcode)) {
                                        resultcode = "5001";
                                    }
                                    //更新待同步数据处理结果
                                    updWaitSyncRes(resultcode, resultinfo, pw, dao);
                                    logger.info(pw.get("ID") + "<=>" + "=>处理结果：" + resultcode + "，错误信息:" + resultinfo);
                                }
                            }
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("处理出现异常，"+e.getMessage(),e);
        }
    }

    private String BoYuanJSBSync( HashMap<String, String> invokeConf,HashMap<String, String> pw,String cpKey,String password) throws Exception {
        String result = "";
        String params = "";
        try {
            //获取参数
            params = this.getParamsByBoYuanJSB(pw,cpKey,password);
            //调接口
            result = invokeface(invokeConf,params);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("调接口或获取参数出现异常，"+e.getMessage()+"=>"+pw.get("ID"));
            throw e;
        } finally{
            printLog(invokeConf,params,result,pw);
        }

        return result;
    }

    private String getParamsByBoYuanJSB(HashMap<String, String> pw,String cpKey,String password) throws Exception {
        Map<String, Object> params = new HashMap<String, Object>();
        String cpid = pw.get("CPID");
        String spid = pw.get("SPID");
        String cpkey = DesUtil.encode(cpKey,password); //cpid加密
        String activetype = pw.get("ACTIVETYPE"); //领取会员对应活动编码
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String signature = MD5.MD5Encode(cpid + spid + cpkey + activetype + timestamp + password);



        params.put("cpid", cpid);
        params.put("spid", spid);
        params.put("cpkey", cpkey);
        params.put("activetype", activetype);
        params.put("timestamp", timestamp);
        params.put("signature", signature);


        return JSONObject.toJSONString(params);

    }
    private String invokeface(HashMap<String, String> invokeConf, String params) throws Exception {
        String result = "";
        //调接口地址
        String invokeurl =  invokeConf.get("INVOKEURL");
        //超时时间，单位是秒
        int timeout = 30;
        //数据类型
        String contenttype = "application/json;charset=utf-8";
        //调用方法
        String method = "POST";
        //属性
        Map reqProperty = new HashMap();
        if (!StringUtils.isEmpty(contenttype)) {
            reqProperty.put("Content-type", contenttype);
        }

        result = HttpInvoke.sendhttpReq(method, invokeurl, params, reqProperty, timeout*1000);

        return result;
    }

    private void printLog(HashMap<String, String> invokeConf, String params, String result, HashMap<String, String> pw) {
        logger.info(pw.get("ID")+"<=>"+"=>接口配置："+invokeConf+"，请求报文:【"+params+"】响应报文：【"+result+"】");
    }

    private void updWaitSyncRes(String resultcode, String resultinfo, HashMap<String, String> pw, HuiYuanProductSyncDao dao) {
        try {
            if(resultinfo != null && resultinfo.length() > 300){
                resultinfo = resultinfo.substring(0, 300);
            }
            dao.updProductWaitSync(pw.get("ID"), resultcode, resultinfo);
            logger.info("修改信息"+pw.get("ID")+resultcode+resultinfo);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("updProductWaitSync更新出现异常，"+e.getMessage()+"=>"+pw.get("ID"));
        }


    }
}
