package com.chinacreator.kafka.sync.quartz;

import java.util.HashMap;
import java.util.concurrent.CountDownLatch;

import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.PrevAutoCancelDao;

/**
 * 自动退订数据处理JOB
 * @author xu.zhou
 * @date 20220801
 */
public class PrevAutoCancelDeal  implements Runnable{
	private Logger logger;
	
	private int totalSize;
	
	private CountDownLatch threadSignal;
	
	private PrevAutoCancelDao dao;
	
	private HashMap<String, Object> data;
	
	public PrevAutoCancelDeal(HashMap<String, Object> _data, int _totalSize, CountDownLatch _threadSignal,Logger _logger, PrevAutoCancelDao _dao){
		this.dao = _dao;
		this.data = _data;
		this.totalSize = _totalSize;
		this.threadSignal = _threadSignal;
		this.logger = _logger;
	}
	
	@SuppressWarnings("unchecked")
	public void run(){
		HashMap<String, Object> logmap = new HashMap<String, Object>();
		HashMap<String, String> params = new HashMap<String, String>();
		long starttime = System.currentTimeMillis();
		logmap.put("data", data);
		String resultcode = "5000";
		String errorinfo = "";
		try {
			params.put("USERID", data.get("USERID").toString());
			params.put("RELID", data.get("RELID").toString());
			params.put("CPID", data.get("CPID").toString());
			params.put("SPID", data.get("SPID").toString());
			params.put("ORDERTIME", data.get("ORDERTIME").toString());
			params.put("AUTOCANCELTIME", data.get("AUTOCANCELTIME").toString());
			//params.put("ENDTIME", data.get("AUTOCANCELTIME").toString());			
			params.put("ID", data.get("ID").toString());
			params.put("RESULTCODE", "2");
			params.put("RESULTINFO", "处理中");
			//更新为正在处理
			dao.updExecAutocancel(params);
			
			/**
			 * 获取订购关系
			 */
			HashMap relmap = dao.qryRelById(data.get("RELID").toString());
			if(relmap == null || relmap.size() == 0){
				throw new VideoException("9001", "无订购关系", new String[0]);
			}
			if(!"0".equals(relmap.get("STATUS")) || data.get("ENDTIME") != null){
				throw new VideoException("9002", "已退订", new String[0]);
			}
						
			params.put("PROVINCE", (String)relmap.get("PROVINCE"));
			params.put("AREA", (String)relmap.get("AREA"));
			params.put("STATUS", "1");
			params.put("ORDERCHANNEL", (String)relmap.get("ORDERCHANNEL"));
			params.put("CANCLECHANNEL", "autocancel");
			
			logmap.put("params", params);
			
			int cancelres = dao.updRelCancel(params);
			int buswaitres = dao.addBusshandleWait(params);
			logmap.put("cancelres", cancelres);
			logmap.put("buswaitres", buswaitres);
			
			resultcode = "0";
			errorinfo = "OK";
		} catch (Exception e) {
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage()+errorinfo;
			} else {
				e.printStackTrace();
				logger.error(data.get("ID")+"处理数据出现异常，"+e.getMessage(),e);	
				resultcode = "8000";
				errorinfo = "处理数据出现异常，"+e.getMessage()+errorinfo;
			}	
		} finally {
			threadSignal.countDown();
			logmap.put("resultcode", resultcode);
			logmap.put("errorinfo", errorinfo);
			logmap.put("times", System.currentTimeMillis() - starttime);
			params.put("RESULTCODE", resultcode);
			if(!StringUtils.isEmpty(errorinfo) && errorinfo.length()>200){
				errorinfo = errorinfo.substring(0, 300);
			}
			params.put("RESULTINFO", errorinfo);
			try {
				//更新处理结果
				int res = dao.updExecAutocancel(params);
				logmap.put("更新处理结果", "成功");
			} catch (Exception e2) {
				e2.printStackTrace();
				logmap.put("更新处理结果", "失败，"+e2.getMessage());
			}

			//更新待同步数据处理结果
			logger.info(logmap);
			//System.out.println(logmap);
		}
	}
}
