package com.chinacreator.kafka.sync.quartz;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.PrevAutoCancelDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.sync.utils.printLogControl;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 处理自动退订数据定时任务
 * @author 周旭
 * @date 20220801
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class PrevAutoCancelQuartz extends BaseService implements Job {
	
	private static Logger logger = Logger.getLogger("prevAutoCancel");

	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		printLogControl.writeLong("=======================执行开始=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()), logger, "PrevAutoCancelQuartz");
		try {
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			int pages = jobConf.getPAGES();
			PrevAutoCancelDao dao = (PrevAutoCancelDao) SpringContextUtils.getBean(PrevAutoCancelDao.class);
			List<HashMap> list = dao.qryAutoCancelRec();
			//去重复
			delRepeat(list,"USERID", logger);

			if(list != null && list.size()>0){
				if(list != null && list.size()>0){
					CountDownLatch threadSignal = new CountDownLatch(list.size());
					ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
					for (HashMap<String, Object> data : list) {
						PrevAutoCancelDeal deal = new PrevAutoCancelDeal(data, list.size(), threadSignal, logger, dao);
						executorService.execute(deal);
					}				
					//等待多线程处理完
					executorService.shutdown();
					try {
						executorService.awaitTermination(1L, TimeUnit.MINUTES);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}else{
					sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
				}
			}	
		} catch (Exception e) {
			logger.error("处理出现异常，"+e.getMessage(),e);
		}
		
		printLogControl.writeLong("=========================执行完成=>"+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()), logger, "PrevAutoCancelQuartz");
		
	}
}
