package com.chinacreator.kafka.sync.quartz;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ProductSyncDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.util.HttpInvoke;
import com.chinacreator.kafka.util.SpringContextUtils;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.quartz.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 产品同步定时任务
 * @Author: xinlong.liu
 * @Date: 2022/3/30 16:16
 *
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class ProductSyncQuartz extends BaseService implements Job {

    private static Logger logger = Logger.getLogger("productwait");
    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        try {
            String resultcode = "5000";
            String resultinfo = "无返回信息";
            String result = ""; //调接口返回的报文
            String channel = "huasheng_jsb";//产品同步办理渠道

            //获取配置信息
            TbKafkaJobConf jobConf = (TbKafkaJobConf) context.getJobDetail().getJobDataMap().get("data");
            ProductSyncDao dao = (ProductSyncDao) SpringContextUtils.getBean(ProductSyncDao.class);
            //获取要处理的数据
            List<HashMap<String, String>> list = dao.getProductWaitData();
            logger.info("获取数据条数=> " + list.size());
            //获取渠道配置信息
            List<HashMap<String, String>> dataList = dao.getInvokeConfByChannel(channel);
            if (dataList != null && dataList.size() > 0) {
                if (dataList.get(0) != null) {
                    //获取接口配置信息
                    HashMap<String, String> invokeConf = dataList.get(0);
                    for (HashMap<String, String> pw : list) {
                        logger.info("获取数据=> " + pw);
                        logger.info("获取数据=> " + pw.get("ID"));
                        dao.updWaitSyncing(pw.get("ID"));
                        try {
                            result = huashengJSBSync(invokeConf, pw);
                            resultinfo = result;
                            if (StringUtils.isNotEmpty(result)) {
                                Map<String, String> map = JSONObject.parseObject(result, Map.class);
                                resultcode = map.get("resultcode");
                            }

                        } catch (Exception e) {
                            if ("无返回信息".equals(resultinfo)) {
                                resultinfo = "";
                            }
                            if (e instanceof VideoException) {
                                resultcode = ((VideoException) e).getErrorcode();
                                resultinfo = ((VideoException) e).getMessage() + resultinfo;
                            } else {
                                e.printStackTrace();
                                logger.error("处理数据内部出现异常，" + e.getMessage(), e);
                                resultcode = "8000";
                                resultinfo = "处理数据出现异常，" + e.getMessage() + resultinfo;
                            }
                        } finally {
                            //当前返回信息非接口文档定义格式时，resultcode为空
                            if (StringUtils.isEmpty(resultcode)) {
                                resultcode = "5001";
                            }
                            //更新待同步数据处理结果
                            updWaitSyncRes(resultcode, resultinfo, pw, dao);
                            logger.info(pw.get("ID")+"<=>"+"=>处理结果："+resultcode+"，错误信息:"+resultinfo);
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("处理出现异常，"+e.getMessage(),e);
        }
    }

    private String huashengJSBSync( HashMap<String, String> invokeConf,HashMap<String, String> pw) throws Exception {
        String result = "";
        String params = "";
        try {
            //获取参数
            params = this.getParamsByHuashengJSB(pw);
            //调接口
            result = invokeface(invokeConf,params);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("调接口或获取参数出现异常，"+e.getMessage()+"=>"+pw.get("ID"));
            throw e;
        } finally{
            printLog(invokeConf,params,result,pw);
        }

        return result;
    }

    private String getParamsByHuashengJSB(HashMap<String, String> pw) {
        Map<String, Object> params = new HashMap<String, Object>();
        String cpid = pw.get("CPID");
        String spid = pw.get("SPID");
        String spname = pw.get("SPNAME"); //产品名称
        String price = pw.get("PRICE");
        String productid = pw.get("PRODUCTID");  //产品id
        String effectiveType = "0".equals(pw.get("EFFECTIVETYPE"))?"0":pw.get("EFFECTIVETYPE"); // 0：立即生效
        String packageId = StringUtils.isEmpty(pw.get("PACKAGEID"))?"":pw.get("PACKAGEID"); //包id 预留
        String type = pw.get("TYPE"); //类型 1：包月 0：包次 2：Wo卡（虚拟产品，复合产品）4：复合产品
        String flow = pw.get("FLOW"); //产品流量字段
        String pid = pw.get("PID"); // 产品pid字段

        params.put("cpid", cpid);
        params.put("spid", spid);
        params.put("spname", spname);
        params.put("price", price);
        params.put("productid", productid);
        params.put("effectiveType", effectiveType);
        params.put("packageId", packageId);
        params.put("type", type);
        params.put("flow", flow);
        params.put("pid", pid);


        return JSONObject.toJSONString(params);

    }
    private String invokeface(HashMap<String, String> invokeConf, String params) throws Exception {
        String result = "";
        //调接口地址
        String invokeurl =  invokeConf.get("URL");
        //超时时间，单位是秒
        int timeout = Integer.parseInt(invokeConf.get("TIMEOUT"));
        //TLS协议版本
        String tls = invokeConf.get("TLS");
        //数据类型
        String contenttype = invokeConf.get("CONTENTTYPE");
        //调用方法
        String method = invokeConf.get("METHOD");
        //GET方法参数拼接到URL后面
        if("GET".equals(method)){
            //接口地址有问号，且是在最后一位
            if(invokeurl.indexOf("?") != -1 && (invokeurl.length() - invokeurl.indexOf("?")) == 1){
                invokeurl = invokeurl+params;
            }else if(invokeurl.indexOf("?") != -1){	//url有问号，但问号在中间
                invokeurl = invokeurl+"&"+params;
            }else{//接口地址无问号
                invokeurl = invokeurl+"?"+params;
            }
        }
        //属性
        Map reqProperty = new HashMap();
        if (!StringUtils.isEmpty(contenttype)) {
            reqProperty.put("Content-type", contenttype);
        }

        if(invokeurl.startsWith("https")){
            result = HttpInvoke.sendhttpsReq(method, invokeurl, params, reqProperty, timeout*1000, tls);
        }else{
            result = HttpInvoke.sendhttpReq(method, invokeurl, params, reqProperty, timeout*1000);
        }
        return result;
    }

    private void printLog(HashMap<String, String> invokeConf, String params, String result, HashMap<String, String> pw) {
        logger.info(pw.get("ID")+"<=>"+"=>接口配置："+invokeConf+"，请求报文:【"+params+"】响应报文：【"+result+"】");
    }

    private void updWaitSyncRes(String resultcode, String resultinfo, HashMap<String, String> pw, ProductSyncDao dao) {
        try {
            if(resultinfo != null && resultinfo.length() > 300){
                resultinfo = resultinfo.substring(0, 300);
            }
            dao.updProductWaitSync(pw.get("ID"), resultcode, resultinfo);
            logger.info("修改信息"+pw.get("ID")+resultcode+resultinfo);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("updProductWaitSync更新出现异常，"+e.getMessage()+"=>"+pw.get("ID"));
        }


    }
}
