package com.chinacreator.kafka.thread;

import com.chinacreator.kafka.bean.AysnTfBhbean;
import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * 消费线程
 * @author zhengrong.yan
 * @date 20210219
 *生成日志文件：bkPro.log、ltCard.log
 */
public class AsynTfBhTradeThread extends BaseThread{

	private static Logger busilogger = Logger.getLogger("busilog");

	private static Logger datalogger = Logger.getLogger("asynTfBhTrade");



	/**
	 *
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public AsynTfBhTradeThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		super(conf,consumer,sourceDataRecDao);
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		boolean hasRec = false; //是否入库
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				hasRec = false; 
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						hasRec = false; 
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							
							final AysnTfBhbean asynbean = (AysnTfBhbean)JsonUtil.jsonToBean(dataJson, AysnTfBhbean.class);
							//宝卡Pro（产品ID：91000670）及老铁卡（产品ID：90993386）
							//20210427添加 沃派星燃平安卡（产品ID：91018620）、产品：星燃副卡（产品ID：91018554）
							//592是订购，594是退订
							//20210428添加 新产品沃派星燃平安卡（产品ID：91018620）、产品：星燃副卡（产品ID：91018554）
							if("592".equals(asynbean.getTRADE_TYPE_CODE()) || "594".equals(asynbean.getTRADE_TYPE_CODE())) {
								if ("91000670".equals(asynbean.getPRODUCT_ID())) {
									hasRec = true;
									recBean.setOldlogname("bkPro");
								} else if ("90993386".equals(asynbean.getPRODUCT_ID())) {
									hasRec = true;
									recBean.setOldlogname("ltCard");
								} else if ("91018620".equals(asynbean.getPRODUCT_ID())){
									hasRec = true;
									recBean.setOldlogname("xrpaCard");
								}else if ("91018554".equals(asynbean.getPRODUCT_ID())) {
									hasRec = true;
									recBean.setOldlogname("xrfCard");
								}
							}
							
							if(hasRec){
								int rows = 0;
								try {
									//添加到公共数据表
									rows = sourceDataRecDao.addCommomSrcData(recBean);
								} catch (Exception e) {
									e.printStackTrace();
									datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
								}
                                datalogger.info("入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}else{
								datalogger.info("不入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					//ThreadSleep(true);
				}else{
					//ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
