package com.chinacreator.kafka.thread;

import java.util.HashMap;

import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 处理线程父类
 * @author xu.zhou
 * @date 20201204
 */
public class BaseThread implements Runnable{
	
	public SourceDataRecDao sourceDataRecDao;
	
	public KafkaConsumer<String, String> consumer;
	
	public HashMap conf;
	
	public String threadkey;
	
	long reTime = 1 * 30; //刷新问隔, 单位是秒
	
	public BaseThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		this.conf = conf;
		this.consumer = consumer;
		this.sourceDataRecDao = sourceDataRecDao;
		threadkey = conf.get("THREADKEY").toString();
	}

	@Override
	public void run() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * 执行之前
	 * @param exeTime 当前执行时间
	 * @param reTime  刷新间隔
	 * @param logger  日志
	 * @param topic 
	 * @param records
	 */
	public long beforeProc(long exeTime, Logger logger, String topic, ConsumerRecords<String, String> records){
		try{
			setRecords(records.count());
			
			//String topic = conf.get("TOPIC").toString();
			//String groupid = conf.get("GROUPID").toString();
			//String threadkey = topic+"##"+groupid;
			
			long currTime = System.currentTimeMillis()/1000;
			//logger.info("exeTime=>"+exeTime +" , currTime=>"+ currTime+", reTime=>"+ reTime);
			//控制写日志频率
			if((currTime - exeTime) > reTime){
				logger.info("topicname="+threadkey+"，recordscount======="+records.count());
				exeTime = currTime;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return exeTime;
	}
	
	/**
	 * 调置得到数据条数，用于判断是否积压
	 * @param records
	 */
	private void setRecords(int records){		
		try {
			//String topic = conf.get("TOPIC").toString();
			//String groupid = conf.get("GROUPID").toString();
			//String threadkey = topic+"##"+groupid;
			ThreadLimitUtil.setRecordsMap(threadkey, records);
		} catch (Exception e) {
			e.printStackTrace();
		}	
	}
	
	/**
	 * 线程休眠
	 * @param flag true为有数据休眠控制，false为无数据休眠控制
	 * @throws InterruptedException
	 */
	public void ThreadSleep(boolean flag) throws InterruptedException{
		if(flag){
			Thread.sleep(30);
		}else{
			Thread.sleep(3000);
		}
	}
	
	/**
	 * 处理数据之后要执行流程
	 * @return
	 */
	public boolean afterProc(){
		boolean res = false;
		//String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		ThreadLimitUtil.setConsLive(threadkey);
		//如果当前线程数为0，或者大于1，则退出当前线程
		if(ThreadLimitUtil.getThread(threadkey) == 0 || ThreadLimitUtil.getThread(threadkey) > 1){
			ThreadLimitUtil.removeKey(threadkey);
			res = true;
		}
		
		return res;
	}
	
	/**
	 * 处理数据之后要执行流程
	 * @return
	 */
	public boolean afterProc(Logger logger){
		boolean res = false;
		//String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		ThreadLimitUtil.setConsLive(threadkey);
		//如果当前线程数为0，或者大于1，则退出当前线程
		if(ThreadLimitUtil.getThread(threadkey) == 0 || ThreadLimitUtil.getThread(threadkey) > 1){
			ThreadLimitUtil.removeKey(threadkey);
			res = true;
		}
		
		if(res){
			logger.info(threadkey+"=======客户端非活跃状态，进程退出");
		}
		
		return res;
	}
}
