package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;

import org.apache.commons.lang.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import com.chinacreator.kafka.bean.DouyinBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 抖音消费线程
 * @author xu.zhou
 * @date 20201130
 * kafka-ks.sh 获取topic: DOUYIN，处理类：Mainck，生成文件名：ksmsga.log
 */
public class DouYinThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("douyin");
	
	private static Logger log = Logger.getLogger(DouYinThread.class);
	
	private String dyproductid;
		
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public DouYinThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao, String dyproductid){
		super(conf,consumer,sourceDataRecDao);
		this.dyproductid = dyproductid;
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					boolean hasmapp = false;
					for (ConsumerRecord<String, String> record : records) {
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();
						hasmapp = false;
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							recBean.setOldlogname("ksmsga");
							
							if(!StringUtils.isEmpty(dyproductid)){
								String [] dyproductidarry = dyproductid.split("\\|");
								//DouyinBean tmpBean = (DouyinBean) JsonUtil.jsonToBean(dataJson, DouyinBean.class);
								for(String tmpid : dyproductidarry){
									if(!StringUtils.isEmpty(tmpid) 
											&& !StringUtils.isEmpty(dataJson)
											&& dataJson.indexOf(tmpid) != -1
											//&& tmpid.indexOf(tmpBean.getELEMENT_ID()) != -1
											){
										//datalogger.info("匹配到抖音产品ID，"+tmpid);
										hasmapp = true;
										int rows = sourceDataRecDao.addDouyinSrcData(recBean);
										break;
									}
								}
							}else{
								datalogger.info("无抖音产品ID配置");
								int rows = sourceDataRecDao.addDouyinSrcData(recBean);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						} finally{
							//记录消费数据
							datalogger.info("hasmapp:"+hasmapp+","+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
