package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import com.chinacreator.kafka.bean.KsSourceDataRecBean;
import com.chinacreator.kafka.bean.KuaishouBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 快手数据消费线程
 * @author xu.zhou
 * @date 20201130
 * kafka-ks.sh 获取topic: KUAISHOU，处理类：Mainck，生成文件名：ksmsga.log
 */
public class KuaiShouThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("kuaishou");
			
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public KuaiShouThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		super(conf,consumer,sourceDataRecDao);
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		String groupid = conf.get("GROUPID").toString();
		String username = conf.get("USERNAME")+"";
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		KsSourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		//流量消耗值，只有100%、90%、80%的数据才入库
		List<String> percents = Arrays.asList("100","90","80");
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				//System.out.println(String.valueOf(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date()))+"，获取"+topic+"数据===");
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();
						String percent = "";
						String serialNumber = "";
						
						try {
							recBean = new KsSourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setResultcode("1");
							recBean.setResultinfo("待处理");
							recBean.setOldlogname("ksmsgb");
							//新快手数据，设置标识
							if("yibujiexu12-group-1".equals(groupid) || "yibujiexu12".equals(username)){
								recBean.setOldlogname("ksmsgc");
							}
							recBean.setTopic(topic);
							
							datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);	
							KuaishouBean bean = (KuaishouBean) JsonUtil.jsonToBean(dataJson, KuaishouBean.class);
							if(percents.contains(bean.getPERCENT())){//流量消耗100%、90%、80%的才入库
								percent = bean.getPERCENT();
								serialNumber = bean.getSERIAL_NUMBER();
								recBean.setPercent(percent);
								recBean.setSerialNumber(serialNumber);
								recBean.setDealtime(bean.getDEAL_TIME());
								recBean.setElementid(bean.getELEMENT_ID());
								if("100".equals(percent)){//流量消耗百分之百
									int rows = sourceDataRecDao.addKuaishouSrcData(recBean);
								}else{//流量消耗非百分之百
									int rows = sourceDataRecDao.addKuaishouSrcDataTwo(recBean);
								}

								//busilogger.info(topic+"执行数据入库已完成，影响行："+rows);
							}else{
								recBean.setResultcode("8001");
								recBean.setResultinfo("非指定流量消耗，不入库");
							}							
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
					}
					//休息一会
					ThreadSleep(true);
				}else{
					//休息一会
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);			
			}
		}
	}

	/**
	 * 数据处理
	 * @param dataJson
	 */
	/*
	private void procssData(String dataJson){
		try{
			//解析数据
			KuaishouBean bean = truncBean(dataJson);
			if(bean != null){
				//添加数据到表，有惟一约束，同一月份只录入一条百分之百的数据
				try {
					kuaishouDao.insert(bean);
				} catch (Exception e) {
					e.printStackTrace();
					busilogger.error("添加数据出现异常，serialnumber=>"+bean.getSERIAL_NUMBER()+"， errorinfo=>"+e.getMessage());
					//如果违反唯一约束，移除此数据
					if(e.getMessage() != null && e.getMessage().contains("ORA-00001")){
						busilogger.error("添加数据违反唯一约束，serialnumber=>"+bean.getSERIAL_NUMBER());
					}
				}
				//添加到推送表
				datapush(bean);
			}
		}catch(Exception e){		
			e.printStackTrace();
			busilogger.error("处理数据出现异常："+e.getMessage()+"，dataJson："+dataJson);
		}
	}
	*/
	
	/**
	 * 快手数据解析
	 * @param list
	 * @param filename
	 * @return
	 */
	/*
	private KuaishouBean truncBean(String dataJson) throws Exception{
		KuaishouBean kuaishouBean = null;

		String headstr = dataJson.split("\\|\\|\\|")[0];
		String data = dataJson.split("\\|\\|\\|")[1];
		KuaishouBean bean = (KuaishouBean) JsonUtil.jsonToBean(data, KuaishouBean.class);
		if("100".equals(bean.getPERCENT())){
			bean.setGETTIME(headstr.substring(headstr.indexOf("[") + 1, headstr.indexOf(",")));
			bean.setGETTIME(bean.getGETTIME().replaceAll("/", "").replaceAll(":", "").replaceAll(" ", ""));
			bean.setFILENAME("");
			bean.setDEAL_MONTH(bean.getDEAL_TIME().substring(0, 6));
			
			String spid = kuaishouDao.getSpidByElementid(bean.getELEMENT_ID());
			if(spid != null && !"".equals(spid) && !"null".equals(spid)){
				bean.setSPID(spid);
				kuaishouBean = bean;
			}else{
				busilogger.info("元素ID未匹配到SPID=>："+dataJson);
			}
		}else{
			busilogger.info("非百分之百消耗，不处理。"+dataJson);
		}
		
		return kuaishouBean;
	}
	
	private void datapush(KuaishouBean dataBean) throws Exception {
		//当前月份
		String pushmonth = dataBean.getDEAL_MONTH(); //new SimpleDateFormat("yyyyMM").format(new Date());
		//流量数据月份的每一天每一秒
		String pushmonthFirstDay = dataBean.getDEAL_TIME().substring(0, 8)+"000001";
		//手机号码
		String serialNumber = dataBean.getSERIAL_NUMBER();
		//入库表指定月份所有的SPID，多个用#分隔
		String spids = ""; 		
		//查询当月已收到的流量耗尽数据
		List<KuaishouBean> dataList = kuaishouDao.query(serialNumber,pushmonth);
		//logger.info(serialNumber+"----size----" + (dataList != null ? dataList.size():0) );
		if (dataList != null && dataList.size() > 0) {
			//数据按月份分组
			for(KuaishouBean bean : dataList){
				if("".equals(spids)){
					spids += bean.getSPID();
				}else{
					spids += "#"+bean.getSPID();
				}
			}
			//要推送
			hasPush(serialNumber, spids, pushmonth, pushmonthFirstDay);
		}else{
			busilogger.info("入库表指定月份无数据，"+JsonUtil.objectToJson(dataBean));
		}
	}
	*/
	
	/**
	 * 判断是否要推送“流量已耗尽”
	 * @param serialNumber
	 * @param spids
	 * @param pushmonth
	 * @param pushmonthFistDay 流量数据月份的每一天每一秒	 
	 * @return
	 * @throws Exception
	 */
	/*
	private void hasPush(String serialNumber, String spids, String pushmonth,String pushmonthFistDay) throws Exception{

		List<String> spidsList = fullSpid(spids);//Arrays.asList(spids.split("#"));使用这种方式构建的List在移除元素时会出现异常java.lang.UnsupportedOperationException
		
		//按月份获取有效订购数据
		List<OrderInfo> orderList = kuaishouDao.findByUser(serialNumber,pushmonthFistDay);
		//流量消耗SPID比订购关系表的多
		if(orderList == null || spidsList.size() > orderList.size()){
			//throw new VideoException("1003","不推送，要推送的spid个数大于当前有效订购关系spid个数，spids异常，SPIDS:"+spids);
		}
		//流量消耗SPID比订购关系表的少
		if(orderList.size() > spidsList.size()){
			busilogger.info("不推送，当前有效订购关系spid个数大于要推送的spid个数，有流量未用完，SPIDS:"+spids);
			return;
		}
		
		//如果流量消耗的spid当前无订购关系，移除
		String tmpspid = "";
		boolean nonflag = false; //是否包含，默认不包含
		for(int i=0; i<spidsList.size(); i++){
			nonflag = false;
			tmpspid = spidsList.get(i);
			for(OrderInfo orderInfo : orderList){
				if(tmpspid.equals(orderInfo.getSpid())){
					nonflag = true;
					break;
				}
			}
			//SPID无效
			if(!nonflag){
				busilogger.info("serialNumber: "+serialNumber+", SPID:"+tmpspid+", 当前无有效订购关系");
				spidsList.remove(i);
				i--;
			}
		}
		
		if(spidsList.size() == 0){
			busilogger.info("不推送，要推送的spids当前都无有效订购关系，SPIDS:"+spids);
			return;
		}
		
		spids = "";
		//重组SPIDS，（有可能有删减）
		for(String tmp : spidsList){
			if("".equals(spids)){
				spids += tmp;
			}else{
				spids += "#"+tmp;
			}
		}
		
		busilogger.info("重组后的SPIDS:"+spids);
		
		//是否推送标识，默认true,为推送
		boolean result = true;
		
		//所有SPID都包含，再判断是否当月已经有推送
		List<KuaishouPushBean> pushList = kuaishouDao.queryPush(serialNumber, pushmonth, "0");
		if(pushList != null && pushList.size()>0){
			List<String> tmpSpidsList = null;
			for(KuaishouPushBean bean : pushList){
				tmpSpidsList = Arrays.asList(bean.getSpids().split("#"));
				//关联SPID个数相同
				if(tmpSpidsList.size() == spidsList.size()){
					boolean flag = true; //是否SPID都相同，默认相同
					//判断所有SPID是否相同
					for(String spid : tmpSpidsList){
						//有不相同的
						if(!spidsList.contains(spid)){
							//SPID不一样，
							flag = false;
							break;
						}
					}
					//如果已有相同SPID推送，则不再推送
					if(flag){
						result = false;
						break;
					}
				}
			}
		}

		//把推送数据添加到推送记录表。
		if(result){
			KuaishouPushBean pushBean = new KuaishouPushBean();
			pushBean.setId(kuaishouDao.getNo());
			pushBean.setPushmonth(pushmonth);
			pushBean.setPushtype("0"); //pushtype推送类型：0为流量已耗尽，1为流量未耗尽，2为月末推送
			pushBean.setResultcode("1"); //推送结果编码， 1待处理，2处理中，，0成功，其他为失败
			pushBean.setResultinfo("待处理");
			pushBean.setSerial_number(serialNumber);
			pushBean.setSpids(spids);
			//添加到推送表，由active-process定时任务统一处理
			kuaishouDao.insertPush(pushBean);
			
			busilogger.info("推送数据入库成功=> serialNumber："+serialNumber+"，spids："+spids);
		}else{
			busilogger.info("不推送，已推送相同的spids，SPIDS:"+spids);
		}	
	}
	*/
	
	/**
	 * 把spid转为List
	 * @param spids
	 * @return
	 */
	/*
	private List<String> fullSpid(String spids){
		List<String> spidsList = new ArrayList<String>();
		if(spids != null && !"".equals(spids)){
			String [] spidsArray = spids.split("#");
			for(String s : spidsArray){
				spidsList.add(s);
			}
		}
		return spidsList;
	}
	*/
}
