package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Value;

import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 消费线程
 * @author xu.zhou
 * @date 20201130
 * bgkafka5new.sh 处理类：BGmain5New，生成日志文件：bgmsg3.log、oldmsg3.log、wopai.log
 */
public class NewProdECApiThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("newProdECApi");
			
	private String wopaiprodconf; //沃派产品配置信息
	
	//#女王卡产品编码，多个用|分隔
	private String qcproductid;
	
	/**
	 * 
	 * @param conf
	 * @param wopaiprodconf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public NewProdECApiThread(HashMap conf, String wopaiprodconf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao, String qcproductid){
		super(conf,consumer,sourceDataRecDao);
		this.wopaiprodconf = wopaiprodconf;
		this.qcproductid = qcproductid;
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		List<String> wopaiprodList = null;
		String [] wopaiprod = null; //{"66736525","66736560","66687666"};
		if(wopaiprodconf != null && !"".equals(wopaiprodconf) && wopaiprodconf.length()>0){
			wopaiprod = wopaiprodconf.trim().split(",");
			wopaiprodList = Arrays.asList(wopaiprod);
		}
		//busilogger.info("===========================wopaiprodList: "+wopaiprodList);
		boolean hasCancel = false;
		boolean hasRec = false; //是否入库
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				hasRec = false; 
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						hasRec = false; 
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							
							final BGbean bgBean = (BGbean)JsonUtil.jsonToBean(dataJson, BGbean.class);
							//沃派数据
							if (wopaiprodList != null && "p".equals(bgBean.getKAFKA_TYPE()) && wopaiprodList.contains(bgBean.getDISCNT_CODE())){
								hasRec = true;
								recBean.setOldlogname("wopai");
							} else if (bgBean != null && "2".equals(bgBean.getKAFKA_TYPE()) && StringUtils.isNotEmpty(bgBean.getOLD_PRODUCT_ID())) {
								hasRec = true;
								recBean.setOldlogname("bgmsg3");
							} else if (bgBean != null && "o".equals(bgBean.getKAFKA_TYPE())){
								hasRec = true;
								recBean.setOldlogname("oldmsg3");
							}
							
							if(hasRec){
								//记录消费数据
								datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
								//判断 是否女王卡，是，则入到对应的库表
								if(qcproductid != null && !"".equals(qcproductid) && !"null".equals(qcproductid)){
									String [] qcids = qcproductid.split("\\|");
									for(String qcid: qcids){
										if(dataJson.contains("\"productId\":\""+qcid+"\"") 
												|| dataJson.contains("\"NEW_PRODUCT_ID\":\""+qcid+"\"")
												|| dataJson.contains("\"OLD_PRODUCT_ID\":\""+qcid+"\"")
												//|| resstr.contains("\"PRODUCT_ID\":\""+qcid+"\"")
												){
											try {
												//添加到女王卡数据表
												int rows = sourceDataRecDao.addQueenCardSrcData(recBean);
											} catch (Exception e) {
												e.printStackTrace();
												datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
											}
										}
									}
								}
								
								if("oldmsg3".equals(recBean.getOldlogname())){ //oldmsg3数据不处理	
									recBean.setResultcode("8001");
									recBean.setResultinfo("非指定数据，不处理");
								}
								
								try {
									//添加到公共数据表
									int rows = sourceDataRecDao.addCommomSrcData(recBean);
								} catch (Exception e) {
									e.printStackTrace();
									datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
								}													
								//busilogger.info(topic+"执行数据入库已完成，影响行："+rows);

							}else{
								datalogger.info("不入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc(busilogger)){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
