package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Value;

import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 消费线程
 * @author xu.zhou
 * @date 20201130
 * bgkafka5new.sh 处理类：BGmain5New，生成日志文件：bgmsg3.log、oldmsg3.log、wopai.log
 */
public class NewProdECApiThreadTest extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("newProdECApi");
			
	private String wopaiprodconf; //沃派产品配置信息
	
	//#女王卡产品编码，多个用|分隔
	private String qcproductid;
	
	/**
	 * 
	 * @param conf
	 * @param wopaiprodconf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public NewProdECApiThreadTest(HashMap conf, String wopaiprodconf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao, String qcproductid){
		super(conf,consumer,sourceDataRecDao);
		this.wopaiprodconf = wopaiprodconf;
		this.qcproductid = qcproductid;
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//busilogger.info("===========================wopaiprodList: "+wopaiprodList);
		boolean hasCancel = false;
		boolean hasRec = false; //是否入库
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		try{
			String groupid = conf.get("GROUPID").toString();
			String bookerServer = conf.get("BOOKERSERVER").toString(); //服务器地址
			String username = conf.get("USERNAME").toString(); //登录用户名
			String password = conf.get("PASSWORD").toString(); //登录用户
			String authFilePath = conf.get("AUTHFILEPATH")+""; //权限验证文件路径，已不使用此属性
			String reco = conf.get("RECORDS").toString(); //每次取数据条数
			KafkaConsumer<String, String> consumer2 = getConsumerForSASL(bookerServer,groupid,"GBK",username,password,authFilePath,reco, topic);
			
			consumer2.subscribe(Arrays.asList(topic));
			/**指定offset消费
			consumer2.assign(Arrays.asList(new TopicPartition(topic, 2)));
			consumer2.poll(100);
			consumer2.seek(new TopicPartition(topic, 2), 12115878399L);
			**/
			//while (true) {
		    for(int i=0; i<50; i++){		
				ConsumerRecords<String, String> records = consumer2.poll(1000);
				busilogger.info("测试=i"+i+"+++++++"+threadkey+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						hasRec = false; 
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						busilogger.info(threadkey+"++++++++++++++++"+dataJson);
			        }
				}
				
				ThreadSleep(false);
				
				//调用父类的处理流程
				if(afterProc(busilogger)){
					break;
				}
			}
		    try {
				consumer2.close();
				busilogger.info("关闭客户端");
			} catch (Exception e) {
				// TODO: handle exception
			}
		//}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.info(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
	
	/**   
	* @Title: getProperties   
	* @Description: TODO  consumer对象构建
	* @param @param hostStr
	* @param @param groupStr
	* @param @param encodeStr
	* @param @param username
	* @param @param password
	* @param @param authFilePath	登录认证文件路径 
	* @param @return    设定文件   
	* @return Properties    返回类型   
	* @throws   
	*/  
	private KafkaConsumer<String, String> getConsumerForSASL(String hostStr,String groupStr,String encodeStr,String username,String password,String authFilePath, String records, String topic){
		busilogger.info("/**********************************/");
		busilogger.info("bookerServer=>"+hostStr);
		busilogger.info("groupId=>"+groupStr);
		busilogger.info("encode=>"+encodeStr);
		busilogger.info("username=>"+username);
		busilogger.info("password=>"+password);
		busilogger.info("/**********************************/");
		Properties p=new Properties();
		//在启动consumer时配置的broker地址的。不需要将cluster中所有的broker都配置上，因为启动后会自动的发现cluster所有的broker。
		//它配置的格式是：host1:port1;host2:port2…
		p.put("bootstrap.servers", hostStr);
		p.put("security.protocol", "SASL_PLAINTEXT");
		p.put("sasl.mechanism", "PLAIN");
		//设置GROUPID，用于表示该consumer想要加入到哪个group中。默认值是 ""
		p.put("group.id", groupStr);
		//Consumer 在commit offset时有两种模式：自动提交，手动提交。手动提交在前面已经说过。自动提交：是Kafka Consumer会在后台周期性的去commit。
		//默认值是true。
		p.put("enable.auto.commit", "true");//设置自动提交
		//自动提交间隔。范围：[0,Integer.MAX]，默认值是 5000 （5 s）
		p.put("auto.commit.interval.ms", "7000");
		//Consumer session 过期时间。这个值必须设置在broker configuration中的group.min.session.timeout.ms 与 group.max.session.timeout.ms之间。
		//其默认值是：10000 （10 s）
        p.put("session.timeout.ms", "30000");
        //每次取数据条数， 一次调用poll()操作时返回的最大记录数，默认值为500
        p.put("max.poll.records", records); //20
        //请求发起后，并不一定会很快接收到响应信息。这个配置就是来配置请求超时时间的。默认值是：305000 （305 s）
        p.put("request.timeout.ms", "40000");
        /*****
			当consumer向一个broker发起fetch请求时，broker返回的records的大小最小值。如果broker中数据量不够的话会wait，直到数据大小满足这个条件。
			取值范围是：[0, Integer.Max]，默认值是1。
			默认值设置为1的目的是：使得consumer的请求能够尽快的返回。
         */
        p.put("fetch.min.bytes", "1");
        /****
         *  这个配置项，是告诉Kafka Broker在发现kafka在没有初始offset，或者当前的offset是一个不存在的值（如果一个record被删除，就肯定不存在了）时，该如何处理。它有4种处理方式：
			1） earliest：自动重置到最早的offset。
			2） latest：看上去重置到最晚的offset。
			3） none：如果边更早的offset也没有的话，就抛出异常给consumer，告诉consumer在整个consumer group中都没有发现有这样的offset。
			4） 如果不是上述3种，只抛出异常给consumer。
			默认值是latest。
         */
        p.put("auto.offset.reset", "earliest");//earliest,latest
        //if("WoPlus-group-02".equals(groupStr)){ //消费不到数据，使用latest试一下
        	//p.put("auto.offset.reset", "latest");//earliest,latest
        //}
        //Message record 的key, value的反序列化类。
        p.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        p.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        p.put("deserializer.encoding",encodeStr);
        
        
        //System.setProperty("java.security.auth.login.config", "D:\\kafka_client_jaas.conf"); //配置文件路径
        //System.setProperty("java.security.auth.login.config", authFilePath); //配置文件路径
        //Kafka客户端版本大于0.10.2.1，则可以设置属性 sasl.jaas.config = org.apache.kafka.common.security.plain.PlainLoginModule必需的用户名=" USERNAME" password =" PASSWORD";
        //否则报错：Could not find a 'KafkaClient' entry in the JAAS configuration. System property 'java.security.auth.login.config' is not set
        //只能使用System.setProperty的方式，而使用这种方式不能设置多个authFilePath，比如取抖音就不能取橙卡，因为两个KAFKA的服务器地址不一样，用户名和密码也不一样
        p.put("sasl.jaas.config",
                "org.apache.kafka.common.security.plain.PlainLoginModule required username=\""+username+"\" password=\""+password+"\";");
        KafkaConsumer<String, String> consumer=new KafkaConsumer<String, String>(p);
        /**
        if("NewProdECApi".equals(topic)){
            Map<TopicPartition, OffsetAndMetadata> offsets = new HashMap<TopicPartition, OffsetAndMetadata>();
            offsets.put(new TopicPartition(topic, 0), new OffsetAndMetadata(11929113660L));
            offsets.put(new TopicPartition(topic, 1), new OffsetAndMetadata(11962601440L));
            offsets.put(new TopicPartition(topic, 2), new OffsetAndMetadata(12115878296L));
            consumer.commitSync(offsets);
        }
		**/
        /**
        if("TencentAppFree".equals(topic)){
        	//TencentAppFree 0 WoPlus-group-03 4064173
        	//TencentAppFree 1 WoPlus-group-03 4068377
            Map<TopicPartition, OffsetAndMetadata> offsets = new HashMap<TopicPartition, OffsetAndMetadata>();
            offsets.put(new TopicPartition(topic, 0), new OffsetAndMetadata(4064174L));
            offsets.put(new TopicPartition(topic, 1), new OffsetAndMetadata(4068378L));
            //offsets.put(new TopicPartition(topic, 2), new OffsetAndMetadata(12115878296L));
            consumer.commitSync(offsets);
        }
        ***/
        return consumer;
	}
}
