package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Value;

import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 抖音消费线程
 * @author xu.zhou
 * @date 20201130
 * 对应原工程信息：
 * kafkanew.sh 获取topic: NewProdInfo，处理类：MainNew，生成文件名：msglog.log和chengka.log
 */
public class NewProdInfoThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("newProdInfo");
	
	//#女王卡产品编码，多个用|分隔
	private String qcproductid;
		
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public NewProdInfoThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao, String qcproductid){
		super(conf,consumer,sourceDataRecDao);
		this.qcproductid = qcproductid;
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		//String threadkey = conf.get("THREADKEY").toString();
		boolean hasCancel = false;
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							//判断是否为橙卡数据
							String kafkaType = (String) JsonUtil.getJsonValue(dataJson,"kafkaType");
							if("1".equals(kafkaType)){
								recBean.setOldlogname("chengka");
							}else{
								recBean.setOldlogname("msglog");
							}
							//记录消费数据
							datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							if("chengka".equals(recBean.getOldlogname())){					
								int rows = sourceDataRecDao.addChengkaSrcData(recBean);
							}else{
								//判断 是否女王卡，是，则入到对应的库表
								if(qcproductid != null && !"".equals(qcproductid) && !"null".equals(qcproductid)){
									String [] qcids = qcproductid.split("\\|");
									for(String qcid: qcids){
										if(dataJson.contains("\"productId\":\""+qcid+"\"") 
												|| dataJson.contains("\"NEW_PRODUCT_ID\":\""+qcid+"\"")
												|| dataJson.contains("\"OLD_PRODUCT_ID\":\""+qcid+"\"")
												//|| resstr.contains("\"PRODUCT_ID\":\""+qcid+"\"")
												){
											try {
												//添加到女王卡数据表
												int rows = sourceDataRecDao.addQueenCardSrcData(recBean);
											} catch (Exception e) {
												datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
											}
										}
									}
								}
								try {
									//添加到公共数据表
									int rows = sourceDataRecDao.addCommomSrcData(recBean);
								} catch (Exception e) {
									datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
								}
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
