package com.chinacreator.kafka.thread;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import com.alibaba.druid.support.logging.Log;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.DesUtil;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.MD5;
import com.chinacreator.kafka.util.ThreadLimitUtil;
import com.chinacreator.kafka.util.URLUtil;

/**
 * 橙卡数据消费线程
 * @author xu.zhou
 * @date 20201130
 * mainck.sh 获取topic: notice_gprsstop，处理类：Mainck，生成文件名：chengka.log
 * mainck2.sh 获取topic: notice_gprsstop 处理类：Mainck2，生成文件名：chengka.log
 */
public class NoticeGprsstopThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("noticegprsstop");
			
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public NoticeGprsstopThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		super(conf,consumer,sourceDataRecDao);
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		SourceDataRecBean recBean = null;
		topic = "notice_gprsstop";
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		//long reTime = 1 * 30; //刷新问隔, 单位是秒
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("groupid="+groupid+"，topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, threadkey, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						
						try {
							//橙卡数据
							String  kafkaType = (String) JsonUtil.getJsonValue(dataJson,"kafkaType");
							if("1".equals(kafkaType)){
								recBean = new SourceDataRecBean();
								recBean.setOffset(String.valueOf(offset));
								recBean.setPartition(String.valueOf(partition));
								recBean.setSourcedata(dataJson);
								recBean.setTopic(topic);
								if("Online-group-01".equals(groupid)){
									recBean.setOldlogname("chengka1");
								}else{
									recBean.setOldlogname("chengka2");
								}
								
								//记录消费数据
								datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);						
								int rows = sourceDataRecDao.addChengkaSrcData(recBean);
								//busilogger.info(topic+"执行数据入库已完成，影响行："+rows);
							}else{
								datalogger.info("不入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
