package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.VFbean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

/**
 * 消费线程
 * @author xu.zhou
 * @date 20201130
 * vfkafkanew.sh 获取TOPIC: 原处理类：VfmainNew，生成日志文件：vfmsglog.log、chgmsglog.log
 */
public class ProdOrderInfoThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("prodOrderInfo");
		
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public ProdOrderInfoThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		super(conf,consumer,sourceDataRecDao);
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		SourceDataRecBean recBean = null;
		boolean hasRec = false; //是否入库
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				hasRec = false; 
				ConsumerRecords<String, String> records = consumer.poll(100);
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
		
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						hasRec = false; 
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							
							if (dataJson.contains("OLD_PRODUCT_ID")) {
							    BGbean bgBean = (BGbean)JsonUtil.jsonToBean(dataJson, BGbean.class);
							    if (bgBean == null || !bgBean.getKAFKA_TYPE().equals("2")) {
							        continue;
							    }
							    
							    hasRec = true; 
							    recBean.setOldlogname("chgmsglog");			        
							} else {
							    VFbean vfbean = (VFbean)JsonUtil.jsonToBean(dataJson, VFbean.class);
							    if (vfbean == null || (!vfbean.getProductId().equals("90363828") && !vfbean.getProductId().equals("90371531") && !vfbean.getProductId().equals("90365993") && !vfbean.getProductId().equals("90365965"))) {
							        continue;
							    }
							    hasRec = true; 
							    recBean.setOldlogname("vfmsglog");
							}
							
							if(hasRec){
								//记录消费数据
								datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
								int rows = sourceDataRecDao.addCommomSrcData(recBean);
								//busilogger.info(topic+"执行数据入库已完成，影响行："+rows);
							}else{
								datalogger.info("不入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
