package com.chinacreator.kafka.thread;

import com.chinacreator.kafka.bean.AysnTfBhbean;
import com.chinacreator.kafka.bean.BGbean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * 银龄天龙卡消费线程
 * @author xu.zhou
 * @date 20210908
 */
public class SyngnathusThread extends BaseThread{

	private static Logger busilogger = Logger.getLogger("busilog");

	private static Logger datalogger = Logger.getLogger("syngnathus");
	
	private String yltlcroductid;

	/**
	 *
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public SyngnathusThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao, String yltlcroductid){
		super(conf,consumer,sourceDataRecDao);
		this.yltlcroductid = yltlcroductid;
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		boolean hasRec = false; //是否入库
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		List<String> yltlcList = new ArrayList<String>();
		if(!StringUtils.isEmpty(yltlcroductid)){
			yltlcList = Arrays.asList(yltlcroductid.split("\\|"));
		}
		try{
			//指定客户端取哪个topic的数据
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				hasRec = false; 
				ConsumerRecords<String, String> records = consumer.poll(100);
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				//有获取到数据
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						hasRec = false; 
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();	
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							
							final AysnTfBhbean asynbean = (AysnTfBhbean)JsonUtil.jsonToBean(dataJson, AysnTfBhbean.class);
							if(yltlcList != null && yltlcList.size()>0 && yltlcList.contains(asynbean.getPRODUCT_ID())){
									hasRec = true;
									//设置标识
									recBean.setOldlogname("yltlc");								
							}
							//指定数据，要入库
							if(hasRec){
								int rows = 0;
								try {
									//添加到公共数据表
									rows = sourceDataRecDao.addCommomSrcData(recBean);
								} catch (Exception e) {
									e.printStackTrace();
									datalogger.error(threadkey+": 入库出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
								}
                                datalogger.info("入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}else{
								datalogger.info("不入库，"+recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							}
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
