package com.chinacreator.kafka.thread;

import java.util.Arrays;
import java.util.HashMap;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.jboss.logging.Logger;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.util.ThreadLimitUtil;


/**
 * TencentAppFree消费线程
 * @author xu.zhou
 * @date 20201130
 * wkkafkanew.sh 获取topic: TencentAppFree，处理类：WKmainNew，生成文件名：wkmsglog.log
 */
public class TencentAppFreeThread extends BaseThread{
	
	private static Logger busilogger = Logger.getLogger("busilog");
	
	private static Logger datalogger = Logger.getLogger("tencentAppFree");
		
	/**
	 * 
	 * @param conf
	 * @param consumer
	 * @param sourceDataRecDao
	 */
	public TencentAppFreeThread(HashMap conf, KafkaConsumer<String, String> consumer, SourceDataRecDao sourceDataRecDao){
		super(conf,consumer,sourceDataRecDao);
	}

	@Override
	public void run() {
		String topic = conf.get("TOPIC").toString();
		//String groupid = conf.get("GROUPID").toString();
		//String threadkey = topic+"##"+groupid;
		boolean hasCancel = false;
		SourceDataRecBean recBean = null;
		long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
		long reTime = 1 * 30; //刷新问隔, 单位是秒
		//long currTime = 0;
		try{
			consumer.subscribe(Arrays.asList(topic));
			while (true) {
				ConsumerRecords<String, String> records = consumer.poll(100);
				//busilogger.info("topicname="+topic+"，recordscount======="+records.count());
				
				exeTime = beforeProc(exeTime, busilogger, topic, records);
				
				if(records.count() > 0){
					for (ConsumerRecord<String, String> record : records) {
						String rectopic = record.topic();
						int partition = record.partition();
						long offset = record.offset();
						String key = record.key();
						String dataJson = record.value();
						
						try {
							recBean = new SourceDataRecBean();
							recBean.setOffset(String.valueOf(offset));
							recBean.setPartition(String.valueOf(partition));
							recBean.setSourcedata(dataJson);
							recBean.setTopic(topic);
							recBean.setOldlogname("wkmsglog");
							
							//记录消费数据
							datalogger.info(recBean.getOldlogname() + "," + offset + "," + partition + "," + dataJson);
							int rows = sourceDataRecDao.addCommomSrcData(recBean);
							//busilogger.info(topic+"执行数据入库已完成，影响行："+rows);
						} catch (Exception e) {
							e.printStackTrace();
							datalogger.error(threadkey+": 处理出现异常，"+e.getMessage()+"，dataJson："+dataJson,e);
						}
			        }
					//休眠一会
					ThreadSleep(true);
				}else{
					ThreadSleep(false);
				}
				
				//调用父类的处理流程
				if(afterProc()){
					break;
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			hasCancel = true;
			busilogger.error(threadkey+": 执行出现异常，"+e.getMessage()+"，recBean："+(recBean != null ? recBean.getSourcedata() : ""),e);
		}finally {
			if(hasCancel){
				ThreadLimitUtil.removeKey(threadkey);
			}
		}
	}
}
