package com.chinacreator.kafka.util;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;


@Component
public class SendSmsUtil {

    public static Logger logger = Logger.getLogger(SendSmsUtil.class);
    public static final String urlbase = "https://cctv2.mige.tv/smsbusi/sms/send";
    public static SendSmsUtil smsUtil;

    @Autowired
    private KafkaInfoDao kafkaInfoDao;

    @PostConstruct
    public void init() {
        smsUtil = this;
        smsUtil.kafkaInfoDao = this.kafkaInfoDao;
        logger.info("PostConstruct接口的innit()执行");

    }

    /**
     * @return 获取地址
     */
    private static String getSMSSendUrl() {
        String nurl;
        String dbUrl = null;
        try {

            //从数据字典表中查询地址，正式环境结果为：https://cctv2.mige.tv/smsbusi/sms/send
            // 测试环境结果为：https://testapi.mige.tv/smsbusi/sms/send
            dbUrl = smsUtil.kafkaInfoDao.getDictionaryValue("newSmsSendUrl");
        } catch (Exception e) {
            logger.error("smsUtil、dictionaryDao、url存在空指针问题，使用静态地址");
            e.printStackTrace();
        } finally {
            if (dbUrl == null || "".equals(dbUrl)) {
                nurl = urlbase;
                logger.info("使用静态地址发短信，地址是：" + nurl);
            } else {
                nurl = dbUrl;
                logger.info("发短信，地址是>>>>>>>>>>>>>>>>>：" + nurl);
            }
        }
        return nurl;
    }

    /**
     * 带参数的短信下发
     *
     * @param userid
     * @param contentid
     * @param smsid
     * @param pwd
     * @param map
     * @return
     * @throws Exception
     */
    public static String sendSMSWithVars(String userid, String contentid, String smsid, String pwd, Map<String, Object> map) throws Exception {
        String nurl = getSMSSendUrl();
        userid = DesUtil.encode(userid, pwd);
        String vars = JSONObject.toJSONString(map);
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String sign = SHAUtil.shaEncode(smsid + userid + timestamp + vars + contentid + pwd).toLowerCase();
        userid = URLEncoder.encode(userid, "utf-8");
        vars = URLEncoder.encode(vars, "utf-8");
        //含有vars参数
        String url = nurl + "?smsid=" + smsid + "&userid=" + userid + "&timestamp=" + timestamp + "&sign=" + sign + "&vars=" + vars + "&contentid=" + contentid;
        String result = URLUtil.get(url);
        System.out.println("result:" + result);
        return result;
    }

/*
*
* 暂未出现通过contentid获取不含参数的短信，需要使用时应先测试能否实现
public static String sendSMS(String userid, String contentid) throws Exception {

        String dbUrl = "";
        String nurl = getSMSSendUrl(dbUrl);
        String smsid = "34202205273305"; //测试环境闪信网关
        String pwd = "ygd!@#eq";
        userid = DesUtil.encode(userid, pwd);
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String sign = SHAUtil.shaEncode(smsid + userid + timestamp + contentid + pwd).toLowerCase();
        userid = URLEncoder.encode(userid, "utf-8");
//        没有vars参数
       String  url = nurl+"?smsid="+smsid+"&userid="+userid+"&timestamp="+timestamp+"&sign="+sign+"&contentid="+contentid;
        String result = URLUtil.get(url);
        System.out.println("result:" + result);
        return result;
    }*/


    /**
     * 发短信的老模式
     *
     * @param userid  手机号
     * @param content 具体短信内容
     * @return 短信发送结果
     */
    public static String send(String userid, String content) throws Exception {
        String dbUrl = "";
        String nurl = "";
        try {
            //从数据字典表中查询地址，结果为：https://cctv2.mige.tv/smsbusi/sms/send
            dbUrl = smsUtil.kafkaInfoDao.getDictionaryValue("newSmsSendUrl");
            System.out.println("newSmsSendUrl对应的url是：" + dbUrl);
        } catch (Exception e) {
            logger.error("smsUtil、dictionaryDao、url存在空指针问题，使用静态地址");
            e.printStackTrace();
        } finally {
            if (dbUrl == null || "".equals(dbUrl)) {
                nurl = urlbase;
                logger.info("使用静态地址发短信，地址是：" + nurl);
            } else {
                nurl = dbUrl;
                logger.info("发短信，地址是>>>>>>>>>>>>>>>>>：" + nurl);
            }
        }
        String smsid = "10655117";
        String pwd = "wo6bslq2";
        userid = DesUtil.encode(userid, pwd);
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String sign = SHAUtil.shaEncode(smsid + userid + timestamp + content + pwd).toLowerCase();
        userid = URLEncoder.encode(userid, "utf-8");
        content = URLEncoder.encode(content, "utf-8");
        String url = "";
//      url += "http://cctv2.mige.tv/smsbusi/sms/send?smsid=" + smsid + "&userid=" + userid + "&timestamp=" + timestamp + "&sign=" + sign + "&content=" + content;
        url = nurl + "?smsid=" + smsid + "&userid=" + userid + "&timestamp=" + timestamp + "&sign=" + sign + "&content=" + content;
        String result = "";
        logger.info("url是>>>>>>>>>>>>>>>>>>" + url);
        try {
            result = URLUtil.get(url);
            logger.info("短信发送结果是：" + result);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }


    /**
     * 闪信（已停用，如需开启请改为优化后的方法----sendSMSWithVars）
     *
     * @param userid
     * @param contentid
     * @param map
     * @return
     * @throws Exception
     */
    public static String sendFlashSMS(String userid, String contentid, Map<String, String> map) throws Exception {
        String nurl = getSMSSendUrl();
        String smsid = "34202205273305"; //测试环境闪信网关
        String pwd = "ygd!@#eq";
        userid = DesUtil.encode(userid, pwd);
        String vars = JSONObject.toJSONString(map);
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String sign = SHAUtil.shaEncode(smsid + userid + timestamp + vars + contentid + pwd).toLowerCase();
        userid = URLEncoder.encode(userid, "utf-8");
        vars = URLEncoder.encode(vars, "utf-8");
        //含有vars参数
        String url = nurl + "?smsid=" + smsid + "&userid=" + userid + "&timestamp=" + timestamp + "&sign=" + sign + "&vars=" + vars + "&contentid=" + contentid;
        //没有vars参数
//        url = BaseUrl+"?smsid="+smsid+"&userid="+userid+"&timestamp="+timestamp+"&sign="+sign+"&contentid="+contentid;
        String result = URLUtil.get(url);
        System.out.println("result:" + result);
        return result;

    }

/*
    public static String sendByTemplate(String userid, String contentid, Map<String, String> map)throws Exception{
        String nurl = getSMSSendUrl();
        String smsid = "2022060614432"; //快手发送短信渠道
        String pwd = "#?csgj06";
        userid = DesUtil.encode(userid, pwd);
        String vars = JSONObject.toJSONString(map);
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String sign = SHAUtil.shaEncode(smsid + userid + timestamp + vars + contentid + pwd).toLowerCase();
        userid = URLEncoder.encode(userid, "utf-8");
        vars = URLEncoder.encode(vars, "utf-8");
        //含有vars参数
        String url = nurl + "?smsid=" + smsid + "&userid=" + userid + "&timestamp=" + timestamp + "&sign=" + sign + "&vars=" + vars + "&contentid=" + contentid;
        //没有vars参数
//        url = BaseUrl+"?smsid="+smsid+"&userid="+userid+"&timestamp="+timestamp+"&sign="+sign+"&contentid="+contentid;
        String result = URLUtil.get(url);
        System.out.println("result:" + result);
        return result;
    }*/


    public static void main(String args[]) throws Exception {
//        sendTemp();
    }



    private static void sendTemp() {
        //        SendSmsUtil s = new SendSmsUtil();
//        try {
//            SendSmsUtil.send("18673197465","这是一条测试短信");
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        System.out.println("123");
//        Object result = URLUtil.get("http://111.206.133.54/smsbusi/sms/send?smsid=10655117&userid=TxyXhnLm1vfHKMMyMtvycg%3D%3D&timestamp=1583916754&sign=10908a56107fa3479a7225f39d7b66ae0fef5a6a&content=这是一条测试短信");
//        System.out.println(result);


        //String str = DesUtil.encode("18673167944","hjk!@#qw");
//        send("15581368293","测试test");
        //System.out.println(str);

        HashMap<String, String> _smsVarConf = new HashMap();
        String smsVars = null; //短信模板参数
        _smsVarConf.put("1022", "快手定向流量8元包-https://v.mige.tv/hjts15g");
        _smsVarConf.put("1023", "快手定向流量15元包-https://v.mige.tv/kshjts8g");

        _smsVarConf.put("292", "快手12G定向流量合约包-http://v.mige.tv/8ctdx");
        _smsVarConf.put("293", "快手12G定向流量合约包-http://v.mige.tv/8ctdx");

        _smsVarConf.put("294", "快手25G定向流量合约包-http://v.mige.tv/8ctdx");
        _smsVarConf.put("295", "快手25G定向流量合约包-http://v.mige.tv/8ctdx");

        _smsVarConf.put("296", "快手12G定向流量月包-http://v.mige.tv/vAjMbu");
        _smsVarConf.put("297", "快手12G定向流量月包-http://v.mige.tv/vAjMbu");

        _smsVarConf.put("298", "快手25G定向流量月包-http://v.mige.tv/FzuQFr");
        _smsVarConf.put("299", "快手25G定向流量月包-http://v.mige.tv/FzuQFr");
        smsVars = _smsVarConf.get("292");
        String[] vars = smsVars.split("-");
        HashMap<String, String> map = new HashMap<>();
        map.put("businame", vars[0]);
        map.put("url", vars[1]);
//        SendSmsUtil.sendByTemplate("15581368293","202206090912014411",map);
    }


}
